/*
 -----------------------------------------
|xix designs                        _ O X |
|-----------------------------------------|
|buttonClass - Specialized windows buttons|
|Designed & Coded by Michael L. Spencer   |
 -----------------------------------------
*/

// buttonClass.h

#ifndef _BUTTONCLASS_H_
#define _BUTTONCLASS_H_

// includes
#include <windows.h>

/*
***********************
 b u t t o n C l a s s
***********************
*/
class buttonClass
{
	// public members
	public:
		// default procs
		// constructors
		buttonClass();
		buttonClass(char cCaption[], int X, int Y, int Width, int Height, HWND hParent, WNDPROC WndProc, unsigned int uCmdMsg, HINSTANCE hInstance = GetModuleHandle(NULL), char cFontName[] = NULL);
		// initializer
		void initialize(char cCaption[], int X, int Y, int Width, int Height, HWND hParent, WNDPROC WndProc, unsigned int uCmdMsg, HINSTANCE hInstance = GetModuleHandle(NULL), char cFontName[] = NULL);
		// destructor
		~buttonClass();

		// procs
		HWND		hWnd();															// Returns the button's window handle
		bool		value();														// Returns the (static) button value
		void		setStatic(bool bStatic = false);								// Turns the button's static property on or off
		int			left();															// Returns the button's horizontal position
		void		setLeft(int X);													// Sets the button's horizontal position
		int			top();															// Returns the button's vertical position
		void		setTop(int Y);													// Sets the button's vertical position
		int			width();														// Returns the button's width
		void		setWidth(int Width);											// Sets the button's width
		int			height();														// Returns the button's height
		void		setHeight(int Height);											// Sets the button's height
		void		move(int X = 0, int Y = 0);										// Moves the button to a specific position in the parent window
		void		size(int Width = 80, int Height = 30);							// Sizes the button to new width and height
		void		paint();														// Repaints the button in its current state
		int			imageOffsetX();													// Returns the horizontal image offset
		void		setImageOffsetX(int X = 0);										// Sets the horizontal image offset
		int			imageOffsetY();													// Returns the vertical image offset
		void		setImageOffsetY(int Y = 0);										// Sets the vertical image offset
		void		setImageOffset(int X = 0, int Y = 0);							// Sets the image offset
		int			captionOffsetX();												// Returns the horizontal caption offset
		void		setCaptionOffsetX(int X = 0);									// Sets the horizontal caption offset
		int			captionOffsetY();												// Returns the vertical caption offset
		void		setCaptionOffsetY(int Y = 0);									// Sets the vertical caption offset
		void		setCaptionOffset(int X = 0, int Y = 0);							// Sets the caption offset
		void		getCaption(char cBuf[]);										// Returns the caption
		void		setCaption(char cCaption[] = NULL);								// Sets the caption of the button
		COLORREF	color();														// Returns the color of the button
		void		setColor(COLORREF color);										// Sets the color of the button
		void		setColor(int R, int G, int B);									// Sets the color of the button
		COLORREF	captionColor();													// Returns the color of the button's text
		void		setCaptionColor(COLORREF color);								// Sets the color of the button's text
		void		setCaptionColor(int R, int G, int B);							// Sets the color of the button's text
		void		getFont(char cBuf[]);											// Returns the button's text font
		void		setFont(char cFontName[] = NULL);								// Changes the font of the button caption
		bool		bold();															// Returns bold state
		void		setBold(bool bBoldState = true);								// Changes the font's bold attribute
		bool		italic();														// Returns italic state
		void		setItalic(bool bItalicState = true);							// Changes the font's italic attribute
		bool		underline();													// Returns underline state
		void		setUnderline(bool bUnderlineState = true);						// Changes the font's underline attribute
		bool		strikeOut();													// Returns strike out state
		void		setStrikeOut(bool bStrikeOutState = true);						// Changes the font's strike out attribute
		void		setImage(int iImageResource = 0);								// Sets an image resource to be used on the button
		void		setImage(char cImageFileName[] = NULL);							// Sets an image resource to be used on the button
		void		setMask(int iMaskResource = 0);									// Sets a mask resource to be used on the picture
		void		setMask(char cMaskFileName[] = NULL);							// Sets a mask resource to be used on the picture
		void		replaceUpStateImage(int iImageResource = 0);					// Replaces the up state image with a new one
		void		replaceUpStateImage(char cImageFileName[] = NULL);				// Replaces the up state image with a new one
		void		replaceDownStateImage(int iImageResource = 0);					// Replaces the down state image with a new one
		void		replaceDownStateImage(char cImageFileName[] = NULL);			// Replaces the down state image with a new one
		void		replaceUpStateMask(int iMaskResource = 0);						// Replaces the up state mask with a new one
		void		replaceUpStateMask(char cMaskFileName[] = NULL);				// Replaces the up state mask with a new one
		void		replaceDownStateMask(int iMaskResource = 0);					// Replaces the down state mask with a new one
		void		replaceDownStateMask(char cMaskFileName[] = NULL);				// Replaces the down state mask with a new one
		void		shape(int iShapeMaskResource, COLORREF crTransparent = 0);		// Shapes the button to match a shape mask
		void		shape(char cShapeMaskResource[], COLORREF crTransparent = 0);	// Shapes the button to match a shape mask
		void		unshape();														// Returns the button to its original rectangle shape

	// private members
	private:
		// window procs
		static		LRESULT CALLBACK buttonClassProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
		static void	CALLBACK buttonClassTimerProc(HWND hWnd, UINT uMsg, UINT uId, DWORD dSysTime);

		// procs
		HWND	makeWindow(char cClass[], char cTitle[], int iX, int iY, int iWidth, int iHeight, HWND hParent, WNDPROC MsgProc, HINSTANCE hInstance);
		void	drawUpState(char cCaption[], int iWidth, int iHeight);
		void	drawDownState(char cCaption[], int iWidth, int iHeight);
		void	shapeButton(HBITMAP hShapeMask, COLORREF crTransparent = 0);
		int		DIV(double dNum1, double dNum2);
		double	MOD(double dNum1, double dNum2);

		// button info
		HWND		hButton;			// handle
		HBITMAP		hButtonUpBM,		// button up bitmap
					hButtonDownBM;		// button down bitmap
		int			iBCI,				// position of external button info
					iImageOffsetX,		// horizontal offset of image
					iImageOffsetY,		// vertical offset of image
					iTextOffsetX,		// horizontal offset of text
					iTextOffsetY,		// vertical offset of text
					iImage,				// image resource value
					iMask,				// image mask resource value
					iRImageUp,			// replacement up state image resource value
					iRImageDown,		// replacement down state image resource value
					iRMaskUp,			// replacement up state mask resource value
					iRMaskDown;			// replacement down state mask resource value
		char		* cText,			// button text
					* cFont,			// button text font
					* cImage,			// bitmap file containing image
					* cMask,			// bitmap file containing mask
					* cRImageUp,		// bitmap file containing replacement up state image
					* cRImageDown,		// bitmap file containing replacement down state image
					* cRMaskUp,			// bitmap file containing replacement up state mask
					* cRMaskDown;		// bitmap file containing replacement down state mask
		COLORREF	crCenter,			// color of button center
					crLight,			// color of light borders
					crDark,				// color of dark borders
					crText;				// color of button text
		int			iBold;				// bold text
		byte		bItalic,			// italic text
					bUnderline,			// underlined text
					bStrikeOut;			// strike out text
};

#endif

// buttonClass
