/*
 --------------------------------------
|xix designs                     _ O X |
|--------------------------------------|
|buttonClass usage example             |
|Designed & Coded by Michael L. Spencer|
 --------------------------------------
*/

// winSkel.cpp

#define STRICT

// includes
#include <windows.h>
#include "winSkel.h"
#include "buttonClass.h"
#pragma comment (lib, "buttonClass.lib")
#include "resource.h"

// definitions
#define BUTTON1		501
#define BUTTON2		502
#define BUTTON4		504

// procs
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK buttonProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK lightButtonProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
HWND SetUpWindow(char * cClass, char * cTitle, int nWidth, int nHeight, HINSTANCE hInstance);

// globals
HWND		hWndMe;	// window
buttonClass	bTest,	// buttons
			bTest2,
			bTest3,
			bTest4,
			bTest5;
bool		bLight;

// main
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int nCmdShow)
{
	char		* cClass = new char[13];	// name of window class
	char		* cTitle = new char[18];	// text in window title bar
	const int	nWidth = 234;			// width of window
	const int	nHeight = 155;		// height of window
	MSG msg;
	
	strcpy(cClass, "ButtonSample");
	strcpy(cTitle, "Turn On The Light");
	// create window
	hWndMe = SetUpWindow(cClass, cTitle, nWidth, nHeight, hInstance);

	// clean up
	delete [] cClass;
	delete [] cTitle;
	cClass	= 0;
	cTitle	= 0;

	bLight	= false;

	// initialize (create) buttons
	bTest.initialize(NULL, 20, 20, 80, 30, hWndMe, NULL, (unsigned int) BUTTON1, hInstance);
	bTest2.initialize("Button2", 130, 20, 80, 30, hWndMe, &buttonProc, (unsigned int) BUTTON2, hInstance, NULL);
	bTest3.initialize("Button 3", 20, 80, 80, 30, hWndMe, &buttonProc, 0, hInstance, "Courier New");
	bTest4.initialize(NULL, 130, 80, 80, 30, hWndMe, NULL, (unsigned int) BUTTON4, hInstance);
	bTest5.initialize(NULL, 105, 55, 25, 25, hWndMe, &lightButtonProc, NULL, hInstance);

	// Set button color
	bTest.setColor(0);
	bTest2.setColor(255, 0, 0);
	bTest3.setColor(0, 255, 0);
	bTest4.setColor(0, 0, 255);

	// Set text color
	bTest2.setCaptionColor(0, 0, 255);
	bTest3.setCaptionColor(255, 0, 0);
	bTest4.setCaptionColor(0, 255, 0);

	// Set font
	bTest4.setFont("Courier New");

	// Set caption
	bTest4.setCaption("Static");

	// Set font attributes
	bTest2.setBold();
	bTest3.setItalic();
	bTest3.setStrikeOut();
	bTest4.setBold();
	bTest4.setItalic();
	bTest4.setUnderline();

	// set image and mask
	bTest2.setImage(CUT_IMG);
	bTest2.setMask(CUT_MASK);

	// offset image and caption
	bTest2.setImageOffsetY(-5);
	bTest2.setCaptionOffsetY(7);

	// replace button state images
	bTest.replaceUpStateImage("button_blue.bmp");
	bTest.replaceUpStateMask("button_mask.bmp");
	bTest.replaceDownStateImage("button_green.bmp");
	bTest.replaceDownStateMask("button_mask.bmp");
	bTest5.replaceUpStateImage(LIGHT_OFF);
	bTest5.replaceUpStateMask(LIGHT_MASK);
	bTest5.replaceDownStateImage(LIGHT_OFF_DOWN);
	bTest5.replaceDownStateMask(LIGHT_MASK);

	// shape button to mask
	// transparent color defaults to 0 (black) unless otherwise specified
	bTest.shape("button_mask.bmp");
	bTest5.shape(LIGHT_MASK);

	// make button static
	bTest4.setStatic(true);

	// show windows
	ShowWindow(hWndMe, 1);		// window
	ShowWindow(bTest.hWnd(), 0);	// button
	ShowWindow(bTest2.hWnd(), 0);
	ShowWindow(bTest3.hWnd(), 0);
	ShowWindow(bTest4.hWnd(), 0);
	ShowWindow(bTest5.hWnd(), 1);

	// update window
	UpdateWindow(hWndMe);
	
	// receive messages
	while(GetMessage(&msg,NULL,0,0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

// window message handler
LRESULT CALLBACK WndProc(HWND hWnd,UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_DESTROY:			// Program closed
		{
			PostQuitMessage(0);
			return 0;
			break;
		}
		case WM_COMMAND:			// Object Messages
		{
			switch (LOWORD(wParam))
			{
				case BUTTON1:		// BUTTON1 clicked
				{
					MessageBox(hWnd, "Command value to parent - Button 1", "Parent WndProc", MB_OK);
					break;
				}
				case BUTTON2:		// BUTTON2 clicked
				{
					MessageBox(hWnd, "Command value to parent - Button 2", "Parent WndProc", MB_OK);
					break;
				}
				case BUTTON4:
				{
					if (bTest4.value() == false)
					{
						MessageBox(hWnd, "Value is now false.", "Static Button Proc", MB_OK);
					} else {
						MessageBox(hWnd, "Value is now true.", "Static Button Proc", MB_OK);
					}
				}
			}
			break;
		}
		case WM_PAINT:
		{
			HDC		hDC;
			HPEN	hPen,
					hOldPen;
			HBRUSH	hBrush,
					hOldBrush;
			RECT	rRect;

			if (bLight)
			{
				hPen	= CreatePen(PS_SOLID, 0, 0xbfbfbf);
				hBrush	= CreateSolidBrush(0xbfbfbf);
			} else {
				hPen	= CreatePen(PS_SOLID, 0, 0);
				hBrush	= CreateSolidBrush(0);
			}

			GetClientRect(hWndMe, &rRect);
			hDC			= GetDC(hWndMe);
			hOldPen		= (HPEN)	SelectObject(hDC, hPen);
			hOldBrush	= (HBRUSH)	SelectObject(hDC, hBrush);
			
			Rectangle(hDC, 0, 0, rRect.right - rRect.left, rRect.bottom - rRect.top);

			SelectObject(hDC, hOldPen);
			SelectObject(hDC, hOldBrush);

			ReleaseDC(hWndMe, hDC);
			DeleteDC(hDC);
			DeleteObject(hPen);
			DeleteObject(hBrush);
			DeleteObject(hOldPen);
			DeleteObject(hOldBrush);

			InvalidateRect(hWndMe, &rRect, false);

			break;
		}
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

// button message handler
LRESULT CALLBACK buttonProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_LBUTTONUP:
		{
			MessageBox(hWnd, "Message from shared button message proc.", "Shared Button Proc", MB_OK);
			break;
		}
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

// light button message handler
LRESULT CALLBACK lightButtonProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_LBUTTONUP:
		{
			if (bLight)
			{
				bTest5.replaceUpStateImage(LIGHT_OFF);
				bTest5.replaceDownStateImage(LIGHT_OFF_DOWN);
				bLight = false;

				SendMessage(hWndMe, WM_PAINT, 0, NULL);

				ShowWindow(bTest.hWnd(), 0);
				ShowWindow(bTest2.hWnd(), 0);
				ShowWindow(bTest3.hWnd(), 0);
				ShowWindow(bTest4.hWnd(), 0);
			} else {
				bTest5.replaceUpStateImage(LIGHT_ON);
				bTest5.replaceDownStateImage(LIGHT_ON_DOWN);
				bLight = true;

				SendMessage(hWndMe, WM_PAINT, 0, NULL);

				ShowWindow(bTest.hWnd(), 1);
				ShowWindow(bTest2.hWnd(), 1);
				ShowWindow(bTest3.hWnd(), 1);
				ShowWindow(bTest4.hWnd(), 1);
			}

			

			break;
		}
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

// sets up the window
HWND SetUpWindow(char* cClass, char* cTitle, int nWidth, int nHeight, HINSTANCE hInstance)
{
	WNDCLASSEX wClass;
	HWND hWnd;

	wClass.cbSize = sizeof(wClass);
	wClass.style = CS_HREDRAW | CS_VREDRAW;
	wClass.lpfnWndProc = WndProc;				// Message Handler Procedure
	wClass.cbClsExtra = 0;
	wClass.cbWndExtra = 0;
	wClass.hInstance = hInstance;
	wClass.hIcon = LoadIcon(NULL,IDI_APPLICATION);
	wClass.hCursor = LoadCursor(NULL,IDC_ARROW);
	wClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wClass.lpszMenuName = NULL;
	wClass.lpszClassName = cClass;
	wClass.hIconSm = LoadIcon(NULL,IDI_APPLICATION);

	RegisterClassEx(&wClass);

	hWnd = CreateWindow(cClass, cTitle, WS_OVERLAPPEDWINDOW, 0, 0, nWidth, nHeight, NULL, NULL, hInstance, NULL);

	return hWnd;
}

// winSkel
