/*
 --------------------------------------
|xix designs                     _ O X |
|--------------------------------------|
|Window Skeleton File                  |
|Designed & Coded by Michael L. Spencer|
 --------------------------------------
*/

// winSkel.h
#ifndef _WINSKEL_H_
#define _WINSKEL_H_

#include <windows.h>
#include <fstream.h>
#include <io.h>

// Get Text from edit/hWnd
void GetText(HWND hWnd, char* cBuf, int nBufLen)
{
	SendMessage(hWnd, WM_GETTEXT, (long) nBufLen, (LPARAM)(LPCSTR) cBuf);
}

// Set text to any edit/hWnd
void SetText(HWND hWnd, char* cBuf)
{
	SendMessage(hWnd, WM_SETTEXT, NULL, (LPARAM)(LPCSTR) cBuf);
}

// Get index of selected item in a list
long GetListIndex(HWND hWnd)
{
	long lIndex = SendMessage(hWnd, LB_GETCURSEL, 0, NULL); 
	return lIndex;
}

// Set index of selected item in a list
void SetListIndex(HWND hWnd, long lIndex)
{
	SendMessage(hWnd, LB_SETCURSEL, 0, lIndex); 
}

// Get item count from a list
long GetListCount(HWND hWnd)
{
	long lCount = SendMessage(hWnd, LB_GETCOUNT, 0, NULL);
	return lCount;
}

// Add an item to a list
void AddListItem(HWND hWnd, char* cBuf)
{
	if(strlen(cBuf))
	{
		SendMessage(hWnd, LB_ADDSTRING, 0, (LPARAM)(LPCSTR) cBuf);
	}
}

// Insert an item into a list
void InsertListItem(HWND hWnd, long lIndex, char* cBuf)
{
	if(strlen(cBuf))
	{
		SendMessage(hWnd, LB_INSERTSTRING, lIndex, (LPARAM)(LPCSTR) cBuf);
	}
}

// Delete an item from a list
void DeleteListItem(HWND hWnd, long lIndex)
{
	SendMessage(hWnd, LB_DELETESTRING, lIndex, NULL);
}

// Delete all items in list
void DeleteAllListItems(HWND hWnd)
{
	SendMessage(hWnd,LB_RESETCONTENT,0,NULL);
}

// Get item from a list
void GetListItem(HWND hWnd, long lIndex, char* cBuf)
{
	SendMessage(hWnd, LB_GETTEXT, lIndex,(LPARAM) cBuf);
}

// Get value of a check box
int GetCheckState(HWND hWnd)
{
	int nCheck;
	nCheck = SendMessage(hWnd, BM_GETCHECK, 0, NULL);
	return nCheck;
}

// Set value of a check box
void SetCheckState(HWND hWnd, int nVal)
{
	SendMessage(hWnd, BM_SETCHECK, 0, nVal);
}

// Passes messages during loops to eliminate program lock-up
void DoEvents()
{
	MSG msg;

	if (PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
}

// Loads a text file to a listbox (line by line)
// 0 = load successful
// 1 = file error
int LoadList(HWND hList, char cFile[9999])
{
	if ((access(cFile, 00)) || (cFile[0] == 0)) { return 1; }

	char cLine[256];
	cLine[0] = 0;

	ifstream iFile(cFile);

	if (iFile.fail())
	{
		iFile.close();
		return 1;
	}

	while (!iFile.eof())
	{
		iFile.getline(&cLine[0], 256, '\n');	// Get line from file
		AddListItem(hList, cLine);
	}

	return 0;
}

// Saves a list to a text file (line by line)
// 0 = list saved
// 1 = file error
int SaveList(HWND hList, char cFile[9999])
{
	char cLine[9999];
	char cCrLf[3];
	int nCount;

	if (!access(cFile, 00)) { remove(cFile); }
	
	ofstream oFile(cFile);

	if (oFile.fail())
	{
		oFile.close();
		return 1;
	}

	cCrLf[0] = 13;
	cCrLf[1] = 10;
	cCrLf[2] = 0;

	nCount = (int) GetListCount(hList);
	for (int nLCV = 0; nLCV < nCount; nLCV++)
	{
		cLine[0] = 0;
		GetListItem(hList, (long) nLCV, cLine);
		if (nLCV < (nCount - 1))
		{
			strcat(&cLine[0], &cCrLf[0]);
		}

		oFile<<&cLine[0];
	}
	return 0;
}

// Get the right-most nLen # of characters from a string (char array)
char *Right(char *cIn, int nLen)
{
	char *cOut;
	int nLength = strlen(cIn);
	int nLCV;

	cOut = new char[nLen + 1];

	if (nLen < 0)
	{
		cOut[0] = 0;
		return &cOut[0];
	}

	if (nLen >= nLength)
	{
		for (nLCV = 0; nLCV < nLength; nLCV++)
		{
			cOut[nLCV] = cIn[nLCV];
		}
		cOut[nLCV] = 0;
		return &cOut[0];
	}
	
	for (nLCV = 0; nLCV < nLen; nLCV++)
	{
		cOut[nLCV] = cIn[nLCV + (nLength - nLen)];
	}
	cOut[nLCV] = 0;

	return &cOut[0];
}

// Get the left-most nLen # of characters from a string (char array)
char *Left(char *cIn, int nLen)
{
	char *cOut;
	int nLength = strlen(cIn);
	int nLCV;

	cOut = new char[nLen + 1];

	if (nLen < 0)
	{
		cOut[0] = 0;
		return &cOut[0];
	}

	if (nLen >= nLength)
	{
		for (nLCV = 0; nLCV < nLength; nLCV++)
		{
			cOut[nLCV] = cIn[nLCV];
		}
		cOut[nLCV] = 0;
		return &cOut[0];
	}
	
	for (nLCV = 0; nLCV < nLen; nLCV++)
	{
		cOut[nLCV] = cIn[nLCV];
	}
	cOut[nLCV] = 0;

	return &cOut[0];
}

// Get nLen # of characters from a string (char array) starting at nStart
char *Mid(char *cIn, int nStart, int nLen)
{
	char *cOut;
	int nLength = strlen(cIn);
	int nLCV;

	cOut = new char[nLen + 1];

	if ((nLen < 0) || (nStart < 0) || (nStart >= nLength))
	{
		cOut[0] = 0;
		return &cOut[0];
	}

	if (nLen >= (nLength - nStart))
	{
		for (nLCV = 0; nLCV < (nLength - nStart); nLCV++)
		{
			cOut[nLCV] = cIn[nLCV + nStart];
		}
		cOut[nLCV] = 0;
		return &cOut[0];
	}
	
	for (nLCV = 0; nLCV < nLen; nLCV++)
	{
		cOut[nLCV] = cIn[nLCV + nStart];
	}
	cOut[nLCV] = 0;

	return &cOut[0];
}

// Get the integer division of two numbers
int Div(double dNum1, double dNum2)
{
	return (int) (dNum1 / dNum2);
}

// Get the remainder of an integer division of two numbers
double Mod(double dNum1, double dNum2)
{
	return (double) (dNum1 - (dNum2 * Div(dNum1, dNum2)));
}

// Fade (gradient) a window left to right
void FadeHorz(HWND hWnd, COLORREF Color1, COLORREF Color2, int nX1, int nY1, int nX2, int nY2)
{
	HDC hDC;
	HPEN hPen, oldPen;
	HBRUSH hBrush, oldBrush;
	RECT rRect;
	int nLCV,
		nTot;
	double r1,
		r2,
		rCur,
		g1,
		g2,
		gCur,
		b1,
		b2,
		bCur,
		c1,
		c2,
		rInc,
		gInc,
		bInc;

	c1 = (double) Color1;
	c2 = (double) Color2;

	b1 = Div(c1, 65536);
	c1 = Mod(c1, 65536);
	g1 = Div(c1, 256);
	c1 = Mod(c1, 256);
	r1 = (int) c1;

	b2 = Div(c2, 65536);
	c2 = Mod(c2, 65536);
	g2 = Div(c2, 256);
	c2 = Mod(c2, 256);
	r2 = (int) c2;

	nTot = nX2 - nX1 + 1;

	rInc = (r2 - r1) / (nTot - 1);
	gInc = (g2 - g1) / (nTot - 1);
	bInc = (b2 - b1) / (nTot - 1);

	rCur = r1;
	gCur = g1;
	bCur = b1;

	hDC = GetDC(hWnd);
	GetClientRect(hWnd, &rRect);
	for (nLCV = nX1; nLCV <= nX2; nLCV++)
	{	
		if ((nLCV <= rRect.right) && (nLCV >= 0))
		{
			hPen = CreatePen(PS_SOLID, 0, RGB((int) rCur, (int) gCur, (int) bCur));
			hBrush = CreateSolidBrush(RGB((int) rCur, (int) gCur, (int) bCur));

			oldPen = (HPEN) SelectObject(hDC, hPen);
			oldBrush = (HBRUSH) SelectObject(hDC, hBrush);

			Rectangle(hDC, nLCV, nY1, nLCV + 1, nY2);

			DeleteObject(oldPen);
			DeleteObject(oldBrush);

			rCur += rInc;
			gCur += gInc;
			bCur += bInc;
		}
	}

	ReleaseDC(hWnd,hDC);
	DeleteObject(hPen);
	DeleteObject(hBrush);
	InvalidateRect(hWnd, 0, false);
}

// Fade (gradient) a window from top to bottom
void FadeVert(HWND hWnd, COLORREF Color1, COLORREF Color2, int nX1, int nY1, int nX2, int nY2)
{
	HDC hDC;
	HPEN hPen, oldPen;
	HBRUSH hBrush, oldBrush;
	RECT rRect;
	int nLCV,
		nTot;
	double r1,
		r2,
		rCur,
		g1,
		g2,
		gCur,
		b1,
		b2,
		bCur,
		c1,
		c2,
		rInc,
		gInc,
		bInc;

	c1 = (double) Color1;
	c2 = (double) Color2;

	b1 = Div(c1, 65536);
	c1 = Mod(c1, 65536);
	g1 = Div(c1, 256);
	c1 = Mod(c1, 256);
	r1 = (int) c1;

	b2 = Div(c2, 65536);
	c2 = Mod(c2, 65536);
	g2 = Div(c2, 256);
	c2 = Mod(c2, 256);
	r2 = (int) c2;

	nTot = nY2 - nY1 + 1;

	rInc = (r2 - r1) / (nTot - 1);
	gInc = (g2 - g1) / (nTot - 1);
	bInc = (b2 - b1) / (nTot - 1);

	rCur = r1;
	gCur = g1;
	bCur = b1;

	hDC = GetDC(hWnd);
	GetClientRect(hWnd, &rRect);
	for (nLCV = nY1; nLCV <= nY2; nLCV++)
	{	
		if ((nLCV <= rRect.bottom) && (nLCV >= 0))
		{
			hPen = CreatePen(PS_SOLID, 0, RGB((int) rCur, (int) gCur, (int) bCur));
			hBrush = CreateSolidBrush(RGB((int) rCur, (int) gCur, (int) bCur));

			oldPen = (HPEN) SelectObject(hDC, hPen);
			oldBrush = (HBRUSH) SelectObject(hDC, hBrush);

			Rectangle(hDC, nX1, nLCV, nX2, nLCV + 1);

			DeleteObject(oldPen);
			DeleteObject(oldBrush);

			rCur += rInc;
			gCur += gInc;
			bCur += bInc;
		}
	}

	ReleaseDC(hWnd,hDC);
	DeleteObject(hPen);
	DeleteObject(hBrush);
	InvalidateRect(hWnd, 0, false);
}

// Replaces a specified color with a new color in a rectangular range on a given device context
void SetDCColor(HDC hDC, int nStartX, int nStartY, int nWidth, int nHeight, COLORREF OldColor, COLORREF NewColor)
{
	int LCVX, LCVY;	

	for (LCVY = nStartY; LCVY < nStartY + nHeight; LCVY++)
	{
		for (LCVX = nStartX; LCVX < nStartX + nWidth; LCVX++)
		{
			if (GetPixel(hDC, LCVX, LCVY) == OldColor)
			{
				SetPixel(hDC, LCVX, LCVY, NewColor);
			}
		}
	}
}

#endif

// winSkel
