/*
  Header for the 'bkdacc' library by Claude L. Simon (csimon@epeios.org)
  Copyright (C) 2000,2001 Claude L. SIMON (csimon@epeios.org) 

  This file is part of the Epeios (http://epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: bkdacc.h,v 1.3 2001/10/30 14:49:34 csimon Exp $

#ifndef BKDACC__INC
#define BKDACC__INC

#define BKDACC_NAME		"BKDACC"

#define	BKDACC_VERSION	"$Revision: 1.3 $"	

#define BKDACC_OWNER		"the Epeios project (http://epeios.org/)"

#include "ttr.h"

extern class ttr_tutor &BKDACCTutor;

#if defined( XXX_DBG ) && !defined( BKDACC_NODBG )
#define BKDACC_DBG 
#endif

/* Begin of automatic documentation generation part. */

//V $Revision: 1.3 $
//C Claude L. SIMON (csimon@epeios.org)
//R $Date: 2001/10/30 14:49:34 $

/* End of automatic documentation generation part. */

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

/* Addendum to the automatic documentation generation part. */
//D BacKenD ACCess.
/* End addendum to automatic documentation generation part. */


#include "err.h"
#include "flw.h"
#include "bkdtpm.h"
#include "bkdcst.h"
#include "bkdcmd.h"
#include "bso.h"
#include "str.h"
#include "ctn.h"

#define BKDACC_UNDEFINED_CAST		255
#define BKDACC_UNDEFINED_TYPE		65535
#define BKDACC_UNDEFINED_OBJECT		65535
#define BKDACC_UNDEFINED_COMMAND	65535

#define	BKDACC_MASTER_OBJECT	BKDACC_UNDEFINED_OBJECT
#define BKDACC_MASTER_TYPE		BKDACC_UNDEFINED_TYPE
#define BKDACC_MASTER_COMMAND	BKDACC_UNDEFINED_COMMAND


namespace bkdacc {
	using namespace bkdtpm;
	typedef bkdtpm::id16__ command__;
	typedef bkdtpm::id16__ type__;

	//c The backend access core.
	class backend_access___
	{
	private:
		id16__ Commands_[bkdcmd::c_amount];
		char Message_[200];
		flw::ioflow___ *Channel_;
		void TestBackendCasts_( void );
		command__ GetBackendDefaultCommand_( void );
		void GetGetCommandsCommand_( command__ DefaultCommand );
		void GetBackendCommands_( void );
		void TestBackend_( void )
		{
				command__ DefaultCommand;

				TestBackendCasts_();

				DefaultCommand = GetBackendDefaultCommand_();

				GetGetCommandsCommand_( DefaultCommand );

				GetBackendCommands_();
		}
		void Internal_( bkdcmd::command Command )
		{
			PushHeader( BKDACC_MASTER_OBJECT, Commands_[Command] );
		}
		void TestOutput_( bkdcst::cast Cast )
		{
			Channel_->Put( Cast );
		}
		void TestInput_( bkdcst::cast Cast )
		{
			if ( Channel_->Get() != Cast )
				ERRb();
		}
	public:
		void reset( bool P = true )
		{
			if ( P ) {
				if ( Channel_ != NULL )
					Disconnect();
			}
				
			Channel_ = NULL;
		}
		backend_access___( void )
		{
			reset( false );
		}
		~backend_access___( void )
		{
			reset( true );
		}
		//f Initialization with 'Channel' to parse/answer the request.
		void Init( flw::ioflow___ &Channel )
		{
			reset();

			Channel_ = &Channel;

			TestBackend_();
		}
		//f Add header with object 'Object' and command 'Command'.
		void PushHeader(
			object__ Object,
			command__ Command )
		{
			flw::Put( Object, *Channel_ );
			flw::Put( Command, *Channel_ );
		}
		//f Put 'Object'.
		void PushObject( object__ Object )
		{
			TestOutput_( bkdcst::cObject );
			bkdtpm::PutObject( Object, *Channel_ );
		}
		//f Put 'Boolean'.
		void PushBoolean( boolean__ Boolean )
		{
			TestOutput_( bkdcst::cBoolean );
			bkdtpm::PutBoolean( Boolean, *Channel_ );
		}
		//f Put 'Id8'.
		void PushId8( id8__ Id8 )
		{
			TestOutput_( bkdcst::cId8 );
			bkdtpm::PutId8( Id8, *Channel_ );
		}
		//f Put 'Ids8'.
		void PushIds8( const ids8_ &Ids8 )
		{
			TestOutput_( bkdcst::cIds8 );
			bkdtpm::PutIds8( Ids8, *Channel_ );
		}
		//f Put 'Id16'.
		void PushId16( id16__ Id16 )
		{
			TestOutput_( bkdcst::cId16 );
			bkdtpm::PutId16( Id16, *Channel_ );
		}
		//f Put 'Ids16'.
		void PushIds16( const ids16_ &Ids16 )
		{
			TestOutput_( bkdcst::cIds16 );
			bkdtpm::PutIds16( Ids16, *Channel_ );
		}
		//f Put 'Id32'.
		void PushId32( id32__ Id32 )
		{
			TestOutput_( bkdcst::cId32 );
			bkdtpm::PutId32( Id32, *Channel_ );
		}
		//f Put 'Ids32'.
		void PushIds8( const ids32_ &Ids32 )
		{
			TestOutput_( bkdcst::cIds32 );
			bkdtpm::PutIds32( Ids32, *Channel_ );
		}
		//f Put 'Char'.
		void PushChar( char__ Char )
		{
			TestOutput_( bkdcst::cChar );
			bkdtpm::PutChar( Char, *Channel_ );
		}
		//f Put 'String'.
		void PushString( const string_ &String )
		{
			TestOutput_( bkdcst::cString );
			bkdtpm::PutString( String, *Channel_ );
		}
		//f Put 'Strings'.
		void PushStrings( const strings_ &Strings )
		{
			TestOutput_( bkdcst::cStrings );
			bkdtpm::PutStrings( Strings, *Channel_ );
		}
		//f Put 'Byte'.
		void PushByte( byte__ Byte )
		{
			TestOutput_( bkdcst::cByte );
			bkdtpm::PutByte( Byte, *Channel_ );
		}
		//f Put 'Binary'.
		void PushBinary( const binary_ &Binary )
		{
			TestOutput_( bkdcst::cBinary );
			bkdtpm::PutBinary( Binary, *Channel_ );
		}
		//f Put 'Binaries'.
		void PushBinaries( const binaries_ &Binaries )
		{
			TestOutput_( bkdcst::cBinaries );
			bkdtpm::PutBinaries( Binaries, *Channel_ );
		}
		//f Put 'Items8'
		void PushItems8( const items8_ &Items8 )
		{
			TestOutput_( bkdcst::cItems8 );
			bkdtpm::PutItems8( Items8, *Channel_ );
		}
		//f Put 'Items16'
		void PushItems16( const items16_ &Items16 )
		{
			TestOutput_( bkdcst::cItems16 );
			bkdtpm::PutItems16( Items16, *Channel_ );
		}
		//f Put 'Items32'
		void PushItems32( const items32_ &Items32 )
		{
			TestOutput_( bkdcst::cItems32 );
			bkdtpm::PutItems32( Items32, *Channel_ );
		}
		//f Put 'XItems8'
		void PushItems8( const xitems8_ &XItems8 )
		{
			TestOutput_( bkdcst::cXItems8 );
			bkdtpm::PutXItems8( XItems8, *Channel_ );
		}
		//f Put 'Items16'
		void PushXItems16( const xitems16_ &XItems16 )
		{
			TestOutput_( bkdcst::cXItems16 );
			bkdtpm::PutXItems16( XItems16, *Channel_ );
		}
		//f Put 'XItems32'
		void PushXItems32( const xitems32_ &XItems32 )
		{
			TestOutput_( bkdcst::cXItems32 );
			bkdtpm::PutXItems32( XItems32, *Channel_ );
		}
		//f Put 'CommandsDetails'.
		void PushCommandsDetails( const commands_details_ &CommandsDetails )
		{
			TestOutput_( bkdcst::cCommandsDetails );
			bkdtpm::PutCommandsDetails( CommandsDetails, *Channel_ );
		}
		//f Put 'ObjectsReferences'.
		void PushObjectsReferences( const objects_references_ &ObjectsReferences )
		{
			TestOutput_( bkdcst::cObjectsReferences );
			bkdtpm::PutObjectsReferences( ObjectsReferences, *Channel_ );
		}
		//f Send the request.
		bso::bool__ Send( err::handle ERRHandle = err::hUsual )
		{
			Channel_->Put( 0 );	// End of request

			Channel_->Synchronize();

			if ( ( !flw::GetString( *Channel_, Message_, sizeof( Message_ ) ) ) )
				ERRl();

			if ( Message_[0] != 0 ) {
				if( ERRHandle == err::hUsual )
					ERRb();
				return false;
			} else
				return true;

		}
		//f Get 'Object'.
		void PopObject( object__ &Object )
		{
			TestInput_( bkdcst::cObject );
			bkdtpm::GetObject( *Channel_, Object );
		}
		//f Get 'Boolean'.
		void PopBoolean( boolean__ &Boolean )
		{
			TestInput_( bkdcst::cBoolean );
			bkdtpm::GetBoolean( *Channel_, Boolean );
		}
		//f Get 'Id8'.
		void PopId8( id8__ &Id8 )
		{
			TestInput_( bkdcst::cId8 );
			bkdtpm::GetId8( *Channel_, Id8 );
		}
		//f Get 'IDs8'.
		void PopIds8( ids8_ &Ids8 )
		{
			TestInput_( bkdcst::cIds8 );
			bkdtpm::GetIds8( *Channel_, Ids8 );
		}
		//f Get 'Id16'.
		void PopId16( id16__ &Id16 )
		{
			TestInput_( bkdcst::cId16 );
			bkdtpm::GetId16( *Channel_, Id16 );
		}
		//f Get 'IDs16'.
		void PopIds16( ids16_ &Ids16 )
		{
			TestInput_( bkdcst::cIds16 );
			bkdtpm::GetIds16( *Channel_, Ids16 );
		}
		//f Get 'Id32'.
		void PopId32( id32__ &Id32 )
		{
			TestInput_( bkdcst::cId32 );
			bkdtpm::GetId32( *Channel_, Id32 );
		}
		//f Get 'IDs32'.
		void PopIds32( ids32_ &Ids32 )
		{
			TestInput_( bkdcst::cIds32 );
			bkdtpm::GetIds32( *Channel_, Ids32 );
		}
		//f Get 'Char'.
		void PopChar( char__ &Char )
		{
			TestInput_( bkdcst::cChar );
			bkdtpm::GetChar( *Channel_, Char );
		}
		//f Get 'String'.
		void PopString( string_ &String )
		{
			TestInput_( bkdcst::cString );
			bkdtpm::GetString( *Channel_, String );
		}
		//f Get 'Strings'.
		void PopStrings( strings_ &Strings )
		{
			TestInput_( bkdcst::cStrings );
			bkdtpm::GetStrings( *Channel_, Strings );
		}
		//f Get 'Byte'.
		void PopByte( byte__ &Byte )
		{
			TestInput_( bkdcst::cByte );
			bkdtpm::GetByte( *Channel_, Byte );
		}
		//f Get 'Binary'.
		void PopBinary( binary_ &Binary )
		{
			TestInput_( bkdcst::cBinary );
			bkdtpm::GetBinary( *Channel_, Binary );
		}
		//f Get 'Binaries'.
		void PopBinaries( binaries_ &Binaries )
		{
			TestInput_( bkdcst::cBinaries );
			bkdtpm::GetBinaries( *Channel_, Binaries );
		}
		//f Get 'Items8'.
		void PopItems8( items8_ &Items8 )
		{
			TestInput_( bkdcst::cItems8 );
			bkdtpm::GetItems8( *Channel_, Items8 );
		}
		//f Get 'Items16'.
		void PopItems16( items16_ &Items16 )
		{
			TestInput_( bkdcst::cItems16 );
			bkdtpm::GetItems16( *Channel_, Items16 );
		}
		//f Get 'Items32'.
		void PopItems32( items32_ &Items32 )
		{
			TestInput_( bkdcst::cItems32 );
			bkdtpm::GetItems32( *Channel_, Items32 );
		}
		//f Get 'XItems8'.
		void PopXItems8( xitems8_ &XItems8 )
		{
			TestInput_( bkdcst::cXItems8 );
			bkdtpm::GetXItems8( *Channel_, XItems8 );
		}
		//f Get 'XItems16'.
		void PopXItems16( xitems16_ &XItems16 )
		{
			TestInput_( bkdcst::cXItems16 );
			bkdtpm::GetXItems16( *Channel_, XItems16 );
		}
		//f Get 'XItems32'.
		void PopXItems32( xitems32_ &XItems32 )
		{
			TestInput_( bkdcst::cXItems32 );
			bkdtpm::GetXItems32( *Channel_, XItems32 );
		}
		//f Get 'CommandsDetails'.
		void PopCommandsDetails( commands_details_ &CommandsDetails )
		{
			TestInput_( bkdcst::cCommandsDetails );
			bkdtpm::GetCommandsDetails( *Channel_, CommandsDetails );
		}
		//f Get 'ObjectsReferences'.
		void PopObjectsReferences( objects_references_ &ObjectsReferences )
		{
			TestInput_( bkdcst::cObjectsReferences );
			bkdtpm::GetObjectsReferences( *Channel_, ObjectsReferences );
		}
		//f Return the explnation messag, if any.
		const char *GetExplanationMessage( void )
		{
			return Message_;
		}
		//f Verify if all the reponse handled.
		void TestCompletion( void )
		{
			if ( Channel_->Get() != bkdcst::cEnd )
				ERRb();
		}
		//f Return the channel used to handle the request as input flow.
		flw::iflow___ &Input( void )
		{
			return *Channel_;
		}
		//f Return the channel used to handle the request as ouput flow.
		flw::oflow___ &Output( void )
		{
			return *Channel_;
		}
		//f Set error breaking status to 'B'. Return previous error breaking status.
		boolean__ SetErrorBreakingStatus( boolean__ B )
		{
			boolean__ R;

			Internal_( bkdcmd::cSetErrorBreakingStatus );

			PushBoolean( B );

			Send();

			PopBoolean( R );

			TestCompletion();

			return R;
		}
		//f Get error breaking status.
		boolean__ GetErrorBreakingStatus( void )
		{
			boolean__ R;

			Internal_( bkdcmd::cGetErrorBreakingStatus );

			Send();

			PopBoolean( R );

			TestCompletion();

			return R;
		}
		//f Return the id of a new object of type 'Type'.
		object__ GetNewObject( type__ Type )
		{
			object__ O;

			Internal_( bkdcmd::cGetNewObject );

			PushId16( Type );

			Send();

			PopObject( O );

			TestCompletion();

			return O;
		}
		//f Return the type id. of type 'TypeName'.
		type__ GetType( const string_ &TypeName )
		{
			type__ Type;

			Internal_( bkdcmd::cGetType );

			PushString( TypeName );

			Send();

			PopId16( Type );

			TestCompletion();

			return Type;
		}
		//f Return the command of object type 'Type', named 'Name' with parameter 'Parameters'.
		command__ GetCommand(
			type__ Type,
			const string_ &Name,
			const ids8_ &Parameters )
		{
			command__ Command;

			Internal_( bkdcmd::cGetCommand );

			PushId16( Type );
			PushString( Name );
			PushIds8( Parameters );

			Send();

			PopId16( Command );

			TestCompletion();

			return Command;
		}
		/*f Put in 'Commands' commands from object of type 'Type',
		whith names 'Names' and casts 'Casts'. */
		void GetCommands(
			type__ Type,
			const commands_details_ &CommandDetails,
			ids16_ &Commands )
		{
			Internal_( bkdcmd::cGetCommands );

			PushId16( Type );
			PushCommandsDetails( CommandDetails );

			Send();

			PopIds16( Commands );

			TestCompletion();
		}
		//f Remove object 'Object'.
		void RemoveObject( object__ Object )
		{
			Internal_( bkdcmd::cRemoveObject );

			PushObject( Object );

			Send();

			TestCompletion();
		}
		//f Put in 'Name' and 'Version' the name and version of the backend.
		void About(
			string_ &Name,
			string_ &Version)
		{
			Internal_( bkdcmd::cAbout );

			Send();

			PopString( Name );
			PopString( Version );

			TestCompletion();
		}
		//f Disconnection.
		void Disconnect( void )
		{
			Internal_( bkdcmd::cDisconnect );

			Send();

			TestCompletion();
			
			Channel_ = NULL;
		}
		//f Put in 'Items' the language names and ids.
		void GetLanguagesIDAndName( items16_ &LanguagesItems )
		{
			Internal_( bkdcmd::cGetLanguagesIDAndName );

			Send();

			PopItems16( LanguagesItems );

			TestCompletion();
		}
		//f Put in 'TypeXItems' the types prefix, name and id.
		void GetTypesIDAndPrefixAndName( xitems16_ &TypeXItems )
		{
			Internal_( bkdcmd::cGetTypesIDAndPrefixAndName );

			Send();

			PopXItems16( TypeXItems );

			TestCompletion();
		}
		//f Put in 'Items' the commands name and id of an object of type 'Type'.
		void GetCommandsIDAndName(
			type__ Type,
			items16_ &CommandItems )
		{
			Internal_( bkdcmd::cGetCommandsIDAndName );

			PushId16( Type );

			Send();

			PopItems16( CommandItems );

			TestCompletion();
		}
		//f Put in 'Parameters' parameters of command 'Command' from object type 'Type'.
		void GetParameters(
			type__ Type,
			command__ Command,
			ids8_ &Parameters )
		{
			Internal_( bkdcmd::cGetParameters );
			PushId16( Type );
			PushId16( Command );

			Send();

			PopIds8( Parameters );

			TestCompletion();
		}
	};
};

/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

#endif
