/*
  'bkdcmd' library by Claude L. Simon (csimon@epeios.org)
  Requires the 'bkdcmd' header file ('bkdcmd.h').
  Copyright (C) 2000,2001 Claude L. SIMON (csimon@epeios.org).

  This file is part of the Epeios (http://epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: bkdcmd.cpp,v 1.3 2001/10/30 14:49:34 csimon Exp $

#define BKDCMD__COMPILATION

#include "bkdcmd.h"

class bkdcmdtutor
: public ttr_tutor
{
public:
	bkdcmdtutor( void )
	: ttr_tutor( BKDCMD_NAME )
	{
#ifdef BKDCMD_DBG
		Version = BKDCMD_VERSION "\b\bD $";
#else
		Version = BKDCMD_VERSION;
#endif
		Owner = BKDCMD_OWNER;
		Date = "$Date: 2001/10/30 14:49:34 $";
	}
	virtual ~bkdcmdtutor( void ){}
};

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

using namespace bkdcmd;

const char *bkdcmd::CommandsNames[bkdcmd::c_amount] = {
	"_SetErrorBreakingStatus$1",
	"_GetErrorBreakingStatus$1",
	"_GetNewObject$1",
	"_GetType$1",
	"_GetCommand$1",
	"_GetCommands$1",
	"_GetTypeAndCommands$1",
	"_RemoveObject$1",
	"_GetLanguagesIDAndName$1",
	"_About$1",
	"_Disconnect$1",
	"_GetTypesIDAndPrefixAndName$1",
	"_GetCommandsIDAndName$1",
	"_GetParameters$1",
};

static const cast SetErrorBreakingStatus[] = {
		cBoolean,	// Boolean of the new error modus.
	cEnd,
		cBoolean,	// Boolean of the old error modus.
	cEnd };
static const cast GetErrorBreakingStatus[] = {
	cEnd,
		cBoolean,	// Boolean of the old error modus.
	cEnd };
static const cast GetNewObject[] = {
		cId16,		// The type of the new object.
	cEnd,
		cObject,	// The asked object.
	cEnd };
static const cast GetType[] = {
		cString,	// The name of the type.
	cEnd,
		cId16,		// The type asked.
	cEnd };
static const cast GetCommand[] = {
		cId16,		// The object type.
		cString,	// The command name.
		cIds8,	// The casts of the command.
	cEnd,
		cId16,	// The asked command.
	cEnd };
static const cast GetCommands[] = {
		cId16,		// The object type.
		cCommandsDetails,	// The wanted commands caracteristics.
	cEnd,
		cIds16,		// The asked commands.
	cEnd };
static const cast GetTypeAndCommands[] = {
		cString,	// The name of the type.
		cCommandsDetails,		// The wanted commands caracteristics.
	cEnd,
		cId16,		// The asked type.
		cIds16,		// The asked commands.
	cEnd };
static const cast RemoveObject[] = {
		cObject,	// The object to remove.
	cEnd,
	cEnd };
static const cast GetLanguagesIDAndName[] = {
	cEnd,
		cItems16,	// The language IDs and name.
	cEnd };
static const cast About[] = {
	cEnd,
		cString,	// String containing the name of the backend.
		cString,	// String containing the version of the backend.
	cEnd };
static const cast Disconnect[] = {
	cEnd,
	cEnd };
static const cast GetTypesIDAndPrefixAndName[] = {
	cEnd,
		cXItems16,	// The types ID, prefix and name.
	cEnd };
static const cast GetCommandsIDAndName[] = {
		cId16,		// Type of objects from which you want the commands name.
	cEnd,
		cItems16,	// The ids and names of the commands.
	cEnd };
static const cast GetParameters[] = {
		cId16,		// Type of objects from which you want the parameters.
		cId16,	// Command of objects from which you want the parameters.
	cEnd,
		cIds8,	// The parameters of the commands.
	cEnd };

const cast *bkdcmd::CommandsParameters[bkdcmd::c_amount] = {
	SetErrorBreakingStatus,
	GetErrorBreakingStatus,
	GetNewObject,
	GetType,
	GetCommand,
	GetCommands,
	GetTypeAndCommands,
	RemoveObject,
	GetLanguagesIDAndName,
	About,
	Disconnect,
	GetTypesIDAndPrefixAndName,
	GetCommandsIDAndName,
	GetParameters,
};



/* Although in theory this class is inaccessible to the different modules,
it is necessary to personalize it, or certain compiler would not work properly */

class bkdcmdpersonnalization
: public bkdcmdtutor
{
public:
	bkdcmdpersonnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the launching of the application  */
	}
	~bkdcmdpersonnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the ending of the application  */
	}
};


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

// 'static' by GNU C++.

static bkdcmdpersonnalization Tutor;

ttr_tutor &BKDCMDTutor = Tutor;
