/*
  Header for the 'bkdcst' library by Claude L. Simon (csimon@epeios.org)
  Copyright (C) 2000,2001 Claude L. SIMON (csimon@epeios.org) 

  This file is part of the Epeios (http://epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: bkdcst.h,v 1.2 2001/10/30 14:49:34 csimon Exp $

#ifndef BKDCST__INC
#define BKDCST__INC

#define BKDCST_NAME		"BKDCST"

#define	BKDCST_VERSION	"$Revision: 1.2 $"	

#define BKDCST_OWNER		"the Epeios project (http://epeios.org/)"

#include "ttr.h"

extern class ttr_tutor &BKDCSTTutor;

#if defined( XXX_DBG ) && !defined( BKDCST_NODBG )
#define BKDCST_DBG 
#endif

/* Begin of automatic documentation generation part. */

//V $Revision: 1.2 $
//C Claude L. SIMON (csimon@epeios.org)
//R $Date: 2001/10/30 14:49:34 $

/* End of automatic documentation generation part. */

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

/* Addendum to the automatic documentation generation part. */
//D BacKenD CaSTs.
/* End addendum to automatic documentation generation part. */

#include "err.h"
#include "flw.h"

//d Version casts version.
#define BKDCST_CASTS_VERSION	"1"

namespace bkdcst {
	//e casts.
	enum cast {
		//i End of request or end of parameters list.
		cEnd = 0,
		//i Object ('bso::ushort__')'.
		cObject,
		//i Boolean ('bso::bool__')
		cBoolean,
		//i 8 bits identifier ( 'bso::ubyte__' ).
		cId8,
		//i Array of 8 bits identifiers ( 'SET_( bso::ubyte__ )' )
		cIds8,
		//i 16 bits identifier ( 'bso::ushort__' ).
		cId16,
		//i Array of 16 bits identifiers ( 'SET_( bso::ushort__ )' )
		cIds16,
		//i 32 bits identifier ('bso::ulong__').
		cId32,
		//i Array of 32 bits identifiers ( 'SET_( bso::ulong__ )' )
		cIds32,
		//i Character ('bso::char__' ).
		cChar,
		//i String ('str::string_').
		cString,
		//i Array of strings ( 'XMCONTAINER_( str::string_ )' )
		cStrings,
		/*i A byte, as data common data cluster for all plateform.
		Has no intrensec signification ('bso::raw__'). */
		cByte,
		//i Binary 'SET_( bso::raw__ )'.
		cBinary,
		//i Array of binaries ( 'XMCONTAINER_( SET_( bso::raw__ ) )' ).
		cBinaries,
		//i An array of 8 bits items ( 'XMCONTAINER_( { bso::ubyte__, str::string_ } )' ).
		cItems8,
		//i An array of 16 bits items ( 'XMCONTAINER_( { bso::ushort__, str::string_ } )' ).
		cItems16,
		//i An array of 32 bits items ( 'XMCONTAINER_( { bso::ulong__, str::string_ } )' ).
		cItems32,
		//i An array of 8 bits extended items ( 'XMCONTAINER_( { bso::ubyte__, XMCONTAINER_( str::string_ ) } )' ).
		cXItems8,
		//i An array of 16 bits extended items ( 'XMCONTAINER_( { bso::ubyte__, XMCONTAINER_( str::string_ ) } )' ).
		cXItems16,
		//i An array of 32 bits extended items ( 'XMCONTAINER_( { bso::ubyte__, XMCONTAINER_( str::string_ ) } )' ).
		cXItems32,
		// The following casts are used for some special commands.
		//i Array of command descriptions ( 'XCONTAINER_( { str::string_, SET_( bso::ubyte__ ) } )' ).
		cCommandsDetails,
		//i Array of object types and identifiers. ( 'SET_( { bso::ushort__, bso::ushort__ } )' ).
		cObjectsReferences,
		//i Amount of casts.
		c_amount,
		//i Not a cast, but signalize an invalid cast.
		cInvalid,
	};	// A modification must be reflected by 'CastsNames'.

	//o The casts names.
	extern const char *CastsNames[c_amount];

	//f Get id from cast named 'CastName'.
	cast GetID( const char *CastName );

};

/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

#endif
