/*
  'bkddcb' library by Claude L. Simon (csimon@epeios.org)
  Requires the 'bkddcb' header file ('bkddcb.h').
  Copyright (C) 2000,2001 Claude L. SIMON (csimon@epeios.org).

  This file is part of the Epeios (http://epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: bkddcb.cpp,v 1.3 2001/10/30 14:49:34 csimon Exp $

#define BKDDCB__COMPILATION

#include "bkddcb.h"

class bkddcbtutor
: public ttr_tutor
{
public:
	bkddcbtutor( void )
	: ttr_tutor( BKDDCB_NAME )
	{
#ifdef BKDDCB_DBG
		Version = BKDDCB_VERSION "\b\bD $";
#else
		Version = BKDDCB_VERSION;
#endif
		Owner = BKDDCB_OWNER;
		Date = "$Date: 2001/10/30 14:49:34 $";
	}
	virtual ~bkddcbtutor( void ){}
};

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

using namespace bkddcb;

using namespace bkdacc;

namespace {

	void HandleCommands_(
		const commands_items_ &Items,
		type__ Type,
		backend_access___ &Backend,
		commands_ &Commands )
	{
	ERRProlog
		ctn::E_CMITEM( command_item_ ) Item;
		tym::row__ P =NONE;
		command Command;
	ERRBegin
		Item.Init( Items );

		P = Items.First();

		while( P != NONE ) {
			Command.Init();
			Command.Identification.Value = Item( P ).Value;
			Command.Identification.ID( Item( P ).ID() );
			Backend.GetParameters( Type, Item( P ).ID(), Command.Parameters );
			Commands.Add( Command );
			P = Items.Next( P );
		}
	ERRErr
	ERREnd
	ERREpilog
	}

	void GetCommands_(
		type__ Type,
		bkdacc::backend_access___ &Backend,
		commands_ &Commands )
	{
	ERRProlog
		commands_items Items;
	ERRBegin
		Items.Init();

		Backend.GetCommandsIDAndName( Type, Items );

		HandleCommands_( Items, Type, Backend, Commands );
	ERRErr
	ERREnd
	ERREpilog
	}

	void HandleType_(
		const type_item_ &Item,
		type_ &Type )
	{
		tym::row__ P = NONE;
		ctn::E_CMITEM( str::string_ ) Value;

		Value.Init( Item.Values );

		Type.ID( Item.ID() );

		P = Item.Values.First();

		if ( P == NONE) 
			ERRb();

		Type.Prefix = Value( P );

		P = Item.Values.Next( P );

		if ( P == NONE) 
			ERRb();

		Type.Name = Value( P );

		P = Item.Values.Next( P );

		if ( P != NONE) 
			ERRb();
	}

	void HandleTypes_(
		const types_items_ &Items,
		bkdacc::backend_access___ &Backend,
		types_ &Types )
	{
	ERRProlog
		ctn::E_CITEM( xitem16_ ) Item;
		tym::row__ P = NONE;
		type Type;
	ERRBegin
		Item.Init( Items );

		P = Items.First();

		while( P != NONE ) {

			Type.Init();

			HandleType_( Item( P ), Type );

			GetCommands_( Item( P ).ID(), Backend, Type.Commands );

			Types.Add( Type );
			P = Items.Next( P );
		}
	ERRErr
	ERREnd
	ERREpilog
	}

	void PutMasterType_( types_items_ &Items )
	{
	ERRProlog
		type_item Item;
	ERRBegin
		Item.Init();

	Item.Values.Add( str::string( "" ) );
	Item.Values.Add( str::string( "" ) );
		Item.ID( -1 );

		Items.Add( Item );
	ERRErr
	ERREnd
	ERREpilog
	}
}

void bkddcb::GetDescription(
	bkdacc::backend_access___ &Backend,
	types_ &Types )
{
ERRProlog
	types_items Items;
ERRBegin
	Items.Init();
	
	PutMasterType_( Items );
	
	Backend.GetTypesIDAndPrefixAndName( Items );
	
	HandleTypes_( Items, Backend, Types );
ERRErr
ERREnd
ERREpilog
}


/* Although in theory this class is inaccessible to the different modules,
it is necessary to personalize it, or certain compiler would not work properly */

class bkddcbpersonnalization
: public bkddcbtutor
{
public:
	bkddcbpersonnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the launching of the application  */
	}
	~bkddcbpersonnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the ending of the application  */
	}
};


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

// 'static' by GNU C++.

static bkddcbpersonnalization Tutor;

ttr_tutor &BKDDCBTutor = Tutor;
