/*
  'bkdmng' library by Claude L. Simon (csimon@epeios.org)
  Requires the 'bkdmng' header file ('bkdmng.h').
  Copyright (C) 2000,2001 Claude L. SIMON (csimon@epeios.org).

  This file is part of the Epeios (http://epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: bkdmng.cpp,v 1.4 2001/10/30 14:49:34 csimon Exp $

#define BKDMNG__COMPILATION

#include "bkdmng.h"

class bkdmngtutor
: public ttr_tutor
{
public:
	bkdmngtutor( void )
	: ttr_tutor( BKDMNG_NAME )
	{
#ifdef BKDMNG_DBG
		Version = BKDMNG_VERSION "\b\bD $";
#else
		Version = BKDMNG_VERSION;
#endif
		Owner = BKDMNG_OWNER;
		Date = "$Date: 2001/10/30 14:49:34 $";
	}
	virtual ~bkdmngtutor( void ){}
};

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

using namespace bkdmng;

static const char *PrimaryCommandName = bkdcmd::CommandsNames[bkdcmd::cGetCommand];

namespace {
	cast GetCastID_( const str::string_ &Name )
	{
		cast C = cInvalid;
	ERRProlog
		char *N = NULL;
	ERRBegin
		N = Name.Convert();

		C = GetID( N );
	ERRErr
	ERREnd
		if ( N != NULL )
			free( N );
	ERREpilog
		return C;
	}
}

struct master_data__ {
	bso::bool__ Deconnexion;
	void *UP;
};

void master_module::Handle_(
	index__,
	request_manager___ &Requete,
	void *PU )
{
	command__ C;
	master_data__ &MasterData = *(master_data__ *)PU;

	flw::Get( Requete.Input(), C );

	if (  C < Descriptions.Amount() )
	{
		Requete.SetDescription( Descriptions( C ) );
		(*(function__)UPs( C ))( *Backend(), *this, C, Requete, MasterData.Deconnexion, MasterData.UP );
	}
	else if ( C == BKDMNG_MASTER_COMMAND )
	{
		char Car;

		if ( ( Car = Requete.Input().Get() ) != 0 )
		{
ERRProlog
			cast__ Cast;
			str::string S;
ERRBegin
			Requete.Output().Put( 0 );	// No explanation message;

			do {
				S.Init();

				S.Add( Car );

				while( ( Car = Requete.Input().Get() ) != 0 )
					S.Add( Car );

				Cast = GetCastID_( S );

				flw::Put( Cast, Requete.Output() );

			} while ( ( Car = Requete.Input().Get() ) != 0 );
ERRErr
ERREnd
ERREpilog
		}
		else
		{
			tym::row__ P = Descriptions.First();
			command__ C = BKDMNG_INVALID_COMMAND;

			while ( ( P != NONE ) && ( Descriptions( P ).Name != str::string( PrimaryCommandName ) ) )
				P = Descriptions.Next( P );

			if ( P != NONE )
				C = (command__)*P;

			Requete.Output().Put( 0 );	// No explanation message;

			flw::Put( C, Requete.Output() );

		}
	}
	else
		ERRb();
}


// Donne la liste des identificateurs et des libells des langues.
static void GetLanguagesIDAndName_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
ERRProlog
	items16 Items;
	item16 Item;
ERRBegin
	Items.Init();
	Item.Init();

	Item.ID( lgg::lDefault );
	Item.Value = LGGLanguageName[lgg::lDefault];
	Items.Add( Item );

	for ( bso::ushort__ I = 0; I < Backend.Langues_.Nombre; I++ )
	{
		Item.Init();
		Item.ID( Backend.Langues_.Identificateurs[I] );
		Item.Value = LGGLanguageName[Backend.Langues_.Identificateurs[I]];
		Items.Add( Item );
	}

	Requete.PushItems16( Items );
	Requete.Complete();
ERRErr
ERREnd
ERREpilog
}

// Donne la liste des identificateurs et des libells de types
static void GetTypesIDAndPrefixAndName_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
ERRProlog
	xitems16 XItems;
	xitem16 XItem;
	tym::row__ P;
	type__ Type;
ERRBegin
	XItems.Init();

	P = Backend.Modules.First();

	while( P != NONE )
	{
		XItem.Init();

#ifdef BKDMNG_DEBUG
		if ( P > BKDMNG_TYPE_MAX )
			ERRc();
#endif
		Type = (type_t__)*P;

		XItem.ID( *Type );
		XItem.Values.Add( str::string( Backend.Module( Type ).Prefix() ) );
		XItem.Values.Add( str::string( Backend.Module( Type ).Name() ) );
		XItems.Add( XItem );
		P = Backend.Modules.Next( P );
	}

	Requete.PushXItems16( XItems );
	Requete.Complete();
ERRErr
ERREnd
ERREpilog
}

static void WriteCommandsIDAndName_(
	const descriptions_ &Descriptions,
	request_manager___ &Requete )
{
ERRProlog
	items16 Items;
	item16 Item;
	ctn::E_CITEM( description_ ) Description;
	tym::row__ P;
	command__ Command;
ERRBegin
	Items.Init();

	P = Descriptions.First();

	Description.Init( Descriptions );

	while( P != NONE )
	{
		Item.Init();

#ifdef BKDMNG_DEBUG
		if ( P > BKDMNG_COMMAND_MAX )
			ERRC();
#endif
		Command = (command__)*P;

		Item.ID( Command );
		Item.Value = Description( P ).Name;
		
		if ( Item.Value( 0 ) != '_' )
			Items.Add( Item );
			
		P = Descriptions.Next( P );
	}

	Requete.PushItems16( Items );
ERRErr
ERREnd
ERREpilog
}


static void GetCommandsIDAndName_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
	id16__ Type;

	Requete.PopId16( Type );

	WriteCommandsIDAndName_( Backend.Module( (type__)*Type ).Descriptions, Requete );

	Requete.Complete();
}

static inline void WriteParameters_(
	const description_ &Description,
	request_manager___ &Requete )
{
	Requete.PushIds8( Description.Casts );
}


static void WriteParameters_(
	const descriptions_ &Descriptions,
	command__ Command,
	request_manager___ &Requete )
{
	ctn::E_CITEM( description_ ) Description;

	Description.Init( Descriptions );

	WriteParameters_( Description( Command ), Requete );
}


static void GetParameters_(
	backend_ &Backend,
	untyped_module &Module,
	command__,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
	id16__ Type;
	id16__ Command;

	Requete.PopId16( Type );
	Requete.PopId16( Command );

	WriteParameters_( Backend.Module( (type__)*Type ).Descriptions, *Command, Requete );

	Requete.Complete();
}


// Donne le libell et la version du serveur d'objets.
static void About_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
	Requete.PushString( string( Backend.Name_ ) );
	Requete.PushString( string( Backend.Version_ ) );
	Requete.Complete();
}

// Dfinit le comportement provoqu par une erreur et retourne son ancien comportement.
static void SetErrorBreakingStatus_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
	bso::bool__ Retour;
	boolean__ O;

	Retour = (bso::raw__)Backend.ErrorBreaking();

	Requete.PopBoolean( O );

	Backend.ErrorBreaking( O != 0  );

	Requete.PushBoolean( Retour );

	Requete.Complete();
}

// Retourne l'error breaking.
static void GetErrorBreakingStatus_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
	bso::bool__ Retour;

	Retour = (bso::raw__)Backend.ErrorBreaking();

	Requete.PushBoolean( Retour );

	Requete.Complete();
}

// Retourne un nouvel objet.
static void GetNewObject_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
	object__ O;
	id16__ T;

	Requete.PopId16( T );

	if ( *T >= Backend.Modules.Amount() )
		ERRb();

	if ( T == BKDMNG_INVALID_TYPE )
		ERRb();

	O = Backend.New( (type__)*T );

	if ( O != BKDMNG_INVALID_TYPE )
		Requete.PushObject( O );
	else
		Requete.SendExplanationMessage( "No such object type." );

	Requete.Complete();
}

// Retourne l'identificateur correspondant  un type donn.
static void GetType_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
ERRProlog
	str::string Type;
	type__ T;
ERRBegin
	Type.Init();

	Requete.PopString( Type );

	T = Backend.Type( Type );

	if ( ( T != BKDMNG_INVALID_TYPE )  )
		Requete.PushId16( *T );
	else
		Requete.SendExplanationMessage( "No such object type name." );

	Requete.Complete();
ERRErr
ERREnd
ERREpilog
}

// Supprime un objet.
static void RemoveObject_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
	object__ O;

	Requete.PopObject( O );

/*	if ( !Backend.Valide( O ) )
		ERRb();
*/
	Backend.Remove( O );

	Requete.Complete();
}

static void FillCommands_(
	backend_ &Backend,
	type__ Type,
	const commands_details_ &CommandsDetails,
	ids16_ &Commands )
{
ERRProlog
	id16__ Command;
	tym::row__ Position = CommandsDetails.First();
	ctn::E_CITEM( command_detail_ ) CommandDetail;
	description Description;
ERRBegin
	CommandDetail.Init( CommandsDetails );

	while( Position != NONE )
	{
		Description.Init();

		Description.Name = CommandDetail( Position ).Name;
		Description.Casts = CommandDetail( Position ).Casts;

		if ( ( Command = Backend.Command( Type, Description ) ) == BKDMNG_INVALID_COMMAND )
			ERRb();

		Commands.Add( Command );

		Position = CommandsDetails.Next( Position );
	}
ERRErr
ERREnd
ERREpilog
}



// Retourne l'identificateur de type et les identificateurs de commande demand.
static void GetTypeAndCommands_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
ERRProlog
	string Name;
	commands_details CommandsDetails;
	type__ Type;
	ids16 Commands;
ERRBegin
	Name.Init();
	CommandsDetails.Init();

	Requete.PopString( Name );
	Requete.PopCommandsDetails( CommandsDetails );

	if ( ( Type = Backend.Type( Name ) ) == BKDMNG_INVALID_TYPE )
		ERRb();

	Commands.Init();

	FillCommands_( Backend, Type, CommandsDetails, Commands );

	Requete.PushId16( *Type );
	Requete.PushIds16( Commands );
	Requete.Complete();
ERRErr
ERREnd
ERREpilog
}


// Retourne l'identificateur d'une commande donne pour un type donn.
static void GetCommand_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
ERRProlog
	id16__ Type;
	description Description;
	command__ Command;
ERRBegin
	Requete.PopId16( Type );

	Description.Init();
	Requete.PopString( Description.Name );
	Requete.PopIds8( Description.Casts );

/*	if ( !Backend.Valide( T ) )
		if ( T() != BKDEND9_TYPE_MAITRE )
			ERRb();
*/
	Command = Backend.Command( *Type, Description );

	if ( ( Command != BKDMNG_INVALID_COMMAND ) )
		Requete.PushId16( Command );
	else
		Requete.SendExplanationMessage( "No such command name or with such description" );

	Requete.Complete();
ERRErr
ERREnd
ERREpilog
}


// Retourne l'identificateur des commandes demandes pour un type donn.
static void GetCommands_(
	backend_ &Backend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
ERRProlog
	id16__ Type;
	commands_details CommandsDetails;
	ids16 Commands;
ERRBegin
	Requete.PopId16( Type );

	CommandsDetails.Init();
	Requete.PopCommandsDetails( CommandsDetails );

	Commands.Init();

	FillCommands_( Backend, *Type, CommandsDetails, Commands );

	Requete.PushIds16( Commands );
	Requete.Complete();
ERRErr
ERREnd
ERREpilog
}


// Deconnection
static void Disconnect_(
	backend_ &BAckend,
	untyped_module &Module,
	command__ Command,
	request_manager___ &Requete,
	bso::bool__ &Deconnexion,
	void * )
{
	Deconnexion = true;
}

#define ADD( I )	Backend.Add( bkdcmd::CommandsNames[bkdcmd::c##I##], ::I##_, bkdcmd::CommandsParameters[bkdcmd::c##I] )

namespace bkdmng {
	// Initialisation avec rattachement  l'interface 'Frontend'.
	void master_module::Init( bkdmng::backend_ &Backend )
	{
		untyped_module::Init( NULL, NULL );
		Backend_ = &Backend;

		// Get and configure the error modus.
		ADD( SetErrorBreakingStatus );

		ADD( GetErrorBreakingStatus );

		// Create a new object.
		ADD( GetNewObject );

		// Give the type corresponding to the given name.
		ADD( GetType );

		// Give the command of the given object type and command name.
		ADD( GetCommand );

		// Give the commands of the given object type and command names.
		ADD( GetCommands );

		// Give the type and the commands of the given type name, commands names and cats.
		ADD( GetTypeAndCommands );

		// Remove the given object.
		ADD( RemoveObject );

		// The id and name of the languages supported.
		ADD( GetLanguagesIDAndName );

		// The string of the interface version.
		ADD( About );

		// Disconnection.
		ADD( Disconnect );

		// The id and name of the supported object types.
		ADD( GetTypesIDAndPrefixAndName );

		// The id and name of the commands of an object type.
		ADD( GetCommandsIDAndName );

		// The parameters (cast) of a command of an object type.
		ADD( GetParameters );

	}

	type__ backend::Type( str::string_ &Name )
	{
		tym::row__ C = Modules.First();

		while ( ( C != NONE ) && ( str::string( Modules(C)->Name() ) != Name )  )
			C = Modules.Next( C );

		if ( C == NONE )
			C = BKDMNG_INVALID_TYPE;
		else if ( *C > BKDMNG_INVALID_TYPE )
			ERRl();

		return (type_t__)*C;
	}

	bso::bool__ backend_::Handle(
		flw::ioflow___ &Channel,
		void *PU )
	{
		master_data__ MasterData;
	ERRProlog
		request_manager___ Requete;
		object__ O;
	ERRBegin
		MasterData.Deconnexion = false;
		MasterData.UP = PU;

		Requete.Init( Channel );

		flw::Get( Channel, O );

		if ( ( *O >= Links.Amount() ) && ( O != BKDMNG_MASTER_OBJECT ) )
			ERRb();

		if ( O != BKDMNG_MASTER_OBJECT )
			Module_( O ).Handle( Index_( O ), Requete, PU );
		else
			Master_.Handle( (index__)0, Requete, &MasterData );

	ERRErr
		Requete.SendExplanationMessage( err::Message() );

		if ( !Master_.ErrorBreaking() )
			ERRRst();
	ERREnd
		Requete.Complete();
	ERREpilog
		return !MasterData.Deconnexion;
	}
#if 0	
	tym::row__ untyped_module::Add(
		const char *Name,
		const void *UP,
		... )
	{
		tym::row__ P;
	ERRProlog
		va_list L;
		description Description;
		bkdcst::cast Cast;
	ERRBegin
		Description.Init( Name );
		
		va_start( L, UP );

		while ( ( Cast = va_arg( L, bkdcst::cast ) ) != cEnd )
			Description.Add( Cast );
			
		Description.Add( Cast );
		
		while( ( Cast = va_arg( L, bkdcst::cast ) ) != cEnd )
			Description.Add( Cast );
			
		va_end( L );
		
		P = Descriptions.Add( Description );
		
		if ( UPs.Add( UP ) != P )
			ERRc();
	ERRErr
	ERREnd
	ERREpilog
		return P;
	}
#endif
	
}
/* Although in theory this class is inaccessible to the different modules,
it is necessary to personalize it, or certain compiler would not work properly */

class bkdmngpersonnalization
: public bkdmngtutor
{
public:
	bkdmngpersonnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the launching of the application  */
	}
	~bkdmngpersonnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the ending of the application  */
	}
};


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

// 'static' by GNU C++.

static bkdmngpersonnalization Tutor;

ttr_tutor &BKDMNGTutor = Tutor;
