/*
  Header for the 'bkdmng' library by Claude L. Simon (csimon@epeios.org)
  Copyright (C) 2000,2001 Claude L. SIMON (csimon@epeios.org) 

  This file is part of the Epeios (http://epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: bkdmng.h,v 1.3 2001/10/30 14:49:34 csimon Exp $

#ifndef BKDMNG__INC
#define BKDMNG__INC

#define BKDMNG_NAME		"BKDMNG"

#define	BKDMNG_VERSION	"$Revision: 1.3 $"	

#define BKDMNG_OWNER		"the Epeios project (http://epeios.org/)"

#include "ttr.h"

extern class ttr_tutor &BKDMNGTutor;

#if defined( XXX_DBG ) && !defined( BKDMNG_NODBG )
#define BKDMNG_DBG 
#endif

/* Begin of automatic documentation generation part. */

//V $Revision: 1.3 $
//C Claude L. SIMON (csimon@epeios.org)
//R $Date: 2001/10/30 14:49:34 $

/* End of automatic documentation generation part. */

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

/* Addendum to the automatic documentation generation part. */
//D BacKenD MaNaGer
/* End addendum to automatic documentation generation part. */


#include "err.h"
#include "flw.h"
#include "lst.h"
#include "lgg.h"
#include "bkdcst.h"
#include "bkdcmd.h"
#include "bkdrqm.h"

//d An invalid value for a command.
#define BKDMNG_INVALID_COMMAND		BKDRQM_INVALID_COMMAND
#define BKDMNG_COMMAND_MAX			BKDRQM_COMMAND_MAX

//d An invalid value for a type.
#define BKDMNG_INVALID_TYPE			BKDRQM_INVALID_TYPE
#define BKDMNG_TYPE_MAX				BKDRQM_TYPE_MAX

//d An invalid value for an object.
#define BKDMNG_INVALID_OBJECT		BKDRQM_INVALID_OBJECT
#define BKDMNG_OBJECT_MAX			BKDRQM_OBJECT_MAX

#define BKDMNG_MASTER_OBJECT		BKDMNG_INVALID_OBJECT

#define BKDMNG_MASTER_TYPE		 	BKDMNG_INVALID_TYPE

#define BKDMNG_MASTER_COMMAND		BKDMNG_INVALID_COMMAND


namespace bkdmng {
	using namespace bkdrqm;
	using tym::memory;

	typedef tym::row__ index__;
	typedef bkdtpm::id16_t__	command__;

	typedef bkdtpm::id16_t__	type_t__;
	TYPEDEF( type_t__, type__ );

	using bkdtpm::object_t__;
	using bkdtpm::object__;

	typedef void (* function__ )(
		class backend &Backend,
		class untyped_module &Module,
		command__ Command,
		request_manager___ &Request,
		bso::bool__ &,
		void *UP );

	//c An untyped module.
	class untyped_module
	{
	private:
		// Prefix for the module/object name.
		const char *Prefix_;
		// Libelle du type de l'objet, du module.
		const char *Name_;
		// L'interface auquel le module est rattach.
		class backend *Backend_;
	protected:
		//v To get the index of a new object.
		virtual index__ BKDMNGNew( void )
		{
			ERRb();
			return 0;	// Pour viter un warning.
		}
		//v To remove the object with index 'Index'.
		virtual void BKDMNGRemove( index__ Index )
		{
			ERRb();
		}
		//v To get a pointer of the object of index 'Index'.
		virtual void *BKDMNGObject( index__ Index )
		{
			ERRb();
			return NULL;	// Pour viter un 'warning'
		}
		// Fonction appele pour traiter la requte 'Requete' pour l'objet d'index 'Index'.
		virtual void Handle_(
			index__ Index,
			request_manager___ &Requete,
			void *PU ) = 0;
	public:
		//r The description of the request.
		descriptions Descriptions;
		//r User pointers.
		bch::E_BUNCH( const void *) UPs;
		untyped_module( void )
		{
			Prefix_ = NULL;
			Name_ = NULL;
			Backend_ = NULL;
		}
		//f Give the index of a new object.
		index__ New( void )
		{
			return BKDMNGNew();
		}
		//f Remove the object of index 'Index'.
		void Remove( index__ Index )
		{
			BKDMNGRemove( Index );
		}
		//f Give the name of the object.
		const char *Name( void ) const
		{
			return Name_;
		}
		//f Give the prefix of the object.
		const char *Prefix( void ) const
		{
			return Prefix_;
		}
		//f Give an pointer to the object of index 'Index'.
		void *Object( index__ Index )
		{
			return BKDMNGObject( Index );
		}
		/*f Initialization with the object prefix 'Prefix' and name 'Name'.
		'Prefix' and 'Name' are not duplicated. They should NOT be modified. */
		void Init(
			const char *Prefix,
			const char *Name )
		{
			Descriptions.Init();
			UPs.Init();

			Prefix_ = Prefix;
			Name_ = Name;
		}
		//f Handle the request 'Request' for object of index 'Index'.
		void Handle(
			index__ Index,
			request_manager___ &Requete,
			void *UP )
		{
			Handle_( Index, Requete, UP );
		}
		//f Return the backend attached to this module.
		backend *Backend( void )
		{
			return Backend_;
		}
		//f Return the command which has 'Description' as description, or 'BKDMNG_INVALID_COMMAND' if non-existant.
		command__ Command( const description_ &Description ) const
		{
			tym::row__ P = Descriptions.Position( Description );

			if ( P == NONE )
				P = BKDMNG_INVALID_COMMAND;
			else if ( *P > BKDMNG_COMMAND_MAX )
				ERRl();

			return (command__)*P;
		}
		tym::row__ Add(
			const char *Name,
			const cast *Casts,
			const void *UP )
		{
			tym::row__ P = Descriptions.Add( Name, Casts );
			
			if ( UPs.Add( UP ) != P )
				ERRc();
				
			return P;
		}			
		tym::row__ Add(
			const char *Name,
			const void *UP,
			... )
		{
			return Add( Name, (cast *)(&UP + 1), UP );
		}
		friend class backend;
		friend class master_module;
	};

	//c A module for an object of type 't'.
	template <class t> class module
	: public untyped_module
	{
	private:
		void Traiter_(
			t &Objet,
			bkdrqm::request_manager___ &Requete,
			void *PU )
		{
			command__ C;

			flw::Get( Requete.Input(), C );

			if ( C > Descriptions.Amount() )
				ERRu();

			Requete.SetDescription( Descriptions( C ) );

			Objet.HANDLE( *Backend(), *this, C, Requete, PU );
		}
	protected:
		virtual void Handle_(
			index__ Index,
			bkdrqm::request_manager___ &Requete,
			void *PU )
		{
			Traiter_( *(t *)untyped_module::Object( Index ), Requete, PU );
		}
	public:
		//f Initialization.
		void Init( void )
		{
			untyped_module::Init( t::PREFIX, t::NAME  );

			t::NOTIFY( *this );
		}
	};
	
	typedef lst::E_LIST	list;

	//c A module with object stored in RAM.
	template <class t, class st> class ram_module
	: public module<t>,
	  private list
	{
	protected:
		virtual void LSTAllocate( tym::size__ Size )
		{
			Objets.Allocate( Size, aem::mFit );
		}
		virtual index__ BKDMNGNew( void )
		{
			st *S = NULL;
			t *Pointeur = NULL;

			if ( ( S = new st ) == NULL )
				ERRm();

			if ( ( Pointeur = new t( *S ) ) == NULL )
			{
				delete S;
				ERRm();
			}

			index__ Index = list::CreateEntry();

			Objets.Write( Pointeur, Index );

			return Index;
		}
		virtual void BKDMNGRemove( index__ Index )
		{
			delete Objets( Index );
			list::Remove( Index );
		}
		virtual void *BKDMNGObject( index__ Index )
		{
			if ( *Index >= Objets.Amount() )
				ERRu();

			return (void *)Objets( Index );
		}
	public:
		//r The pointer object.
		bch::E_BUNCH( t * ) Objets;
		//f Initialization.
		void Init( void )
		{
			list::Init();
			Objets.Init();
			module<t>::Init();
		}
	};
	
	//c A module with object stored in standard memory.
	template <class t, class st> class standard_module
	: public module<t>,
	  private lst::E_LIST
	{
	private:
		ctn::E_ITEM( t ) Element_;
	protected:
		virtual void LSTAllocate( tym::size__ Size )
		{
			Element_.Sync();
			Objets.Allocate( Size, aem::mFit );
		}
		virtual index__ BKDMNGNew( void )
		{
			Element_.Sync();
			return list::CreateEntry();
		}
		virtual void BKDMNGRemove( index__ Index )
		{
			Element_( Index ).reset();
			list::Remove( Index );
			Element_.Sync();
		}
		virtual void *BKDMNGObject( index__ Index )
		{
			if ( *Index >= Objets.Amount() )
				ERRu();

			return (void *)&Element_( Index );
		}
	public:
		//r Contient les objets.
		ctn::E_CONTAINER( t ) Objets;
		void reset( bool P = true )
		{
			Element_.Sync();
			Objets.reset( P );
			list::reset( P );
		}
		standard_module( void )
		{
			reset( false );
		}
		~standard_module( void )
		{
			reset( true );
		}
		// Initialisation.
		void Init( void )
		{
			lst::E_LIST::Init();
			Objets.Init();
			Element_.Init( Objets );
			untyped_module::Init( t::NAME );
			module<t>::Init();
		}
	};

	// Module matre, qui fait tout le boulot.
	class master_module
	: public untyped_module
	{
	private:
		// Si vrai, une erreur arrte le thread, sinon celui-ci se poursuit.
		bso::bool__ ErrorBreaking_;
	protected:
		virtual void Handle_(
			index__ Index,
			request_manager___ &Requete,
			void *PU );
	public:
		master_module( void )
		{
			ErrorBreaking_ = true;
		}
		// Initialisation avec rattachement  l'interface 'Backend'.
		void Init( backend &Backend );
		bso::bool__ ErrorBreaking( void  )
		{
			return ErrorBreaking_;
		}
		void ErrorBreaking( bso::bool__ Value )
		{
			ErrorBreaking_ = Value;
		}
	};

	struct link__
	{
		// Le type de l'objet
		type__ Type;
		// L'index de l'objet;
		index__ Index;
	};

	//t To by-pass a visual C++ bug.
	typedef lst::E_LIST	list;

	// Classe de gestion des liens entre module et objets.
	class links
	: private tym::E_MEMORY( link__ ),
	  private lst::E_LIST
	{
	protected:
		void LSTAllocate( tym::size__ Size )
		{
			E_MEMORY( link__ )::Allocate( Size );
		}
	public:
		// Initialisation.
		void Init( void )
		{
			list::Init();
			E_MEMORY( link__ )::Init();
		}
		object__ New(
			type__ IdType,
			index__ Index )
		{
			link__ Lien;
			object__ IdObjet;
			tym::row__ P;
			
			P = list::CreateEntry();

			if ( *P > BKDMNG_TYPE_MAX )
				ERRl();

			IdObjet = (object_t__)*P;

			Lien.Type = IdType;
			Lien.Index = Index;

			E_MEMORY( link__ )::Write( Lien, *IdObjet );

			return IdObjet;
		}
		void Remove( object__ IdObjet )
		{
			list::Remove( *IdObjet );
		}
		type__ Type( object__ IdObjet ) const
		{
			return E_MEMORY( link__ )::Read( *IdObjet ).Type;
		}
		index__ Index( object__ IdObjet ) const
		{
			return E_MEMORY( link__ )::Read( *IdObjet ).Index;
		}
		tym::size__ Amount( void )
		{
			return list::Amount();
		}
	};


	//c A backend, which handles objects od different type and request to this object.
	class backend
	{
	private:
		master_module Master_;
		// Retourne le module correspondant  'IdType'.
		untyped_module &Module_( type__ IdType ) const
		{
			if ( IdType != BKDMNG_MASTER_TYPE )
				return *Modules( *IdType );
			else
				return (untyped_module &)Master_;	// Not very happy about this conversion, 
		}
		// Retourne le module correspondant  'IdObjet'.
		untyped_module &Module_( object__ IdObjet ) const
		{
			if ( IdObjet != BKDMNG_MASTER_OBJECT )
				return Module_( Type_( IdObjet ) );
			else
				return (untyped_module &)Master_; // Not very happy about this conversion, 
		}
		// Retourne le type correpondant  l'objet d'indetificateur 'IdObjet'.
		type__ Type_( object__ IdObjet ) const
		{
			return Links.Type( IdObjet );
		}
		// Retourne l'indexcorrespondant  l'objet d'identificateur 'IdObjet'.
		index__ Index_( object__ IdObjet ) const
		{
			return Links.Index( IdObjet );
		}
	public:
		// Contient le libell du serveur d'objet. Usage interne.
		const char *Name_;
		// Contient le libell de la version du serveur d'objet. Usage interne.
		const char *Version_;
		// Contient les informations sur les langues supportes. Usage interne.
		struct
		{
			// Nombre de langue supporte.
			bso::ushort__ Nombre;
			// Les identificateurs des langues supportes.
			const lgg::language *Identificateurs;
			// La langue courante.
			lgg::language Courante;
		} Langues_;
		//o The different modules.
		bch::E_BUNCH( untyped_module * ) Modules;
		//o The relation between modules an index.
		links Links;
		backend( void )
		{
			Langues_.Nombre = 0;
			Langues_.Identificateurs = NULL;
			Langues_.Courante = lgg::lDefault;

			Name_ = "";
			Version_ = "";
		}
		/*f Initialization with 'Name' and 'Version'.
		'Name' and 'Version' are NOT duplicated and should not be modified. */
		void Init(
			const char *Name,
			const char *Version )
		{
			Master_.Init( *this );

			Modules.Init();
			Links.Init();

			Name_ = Name;
			Version_ = Version;
		}
		//f Add 'Module' to the interface.
		void Add( untyped_module &Module )
		{
			Module.Backend_ = this;
			Modules.Add( &Module );
		}
		/*f Handle the request which come by 'Channel' and write the answer to 'Channel'.
		If 'true' is returned, than the request contains a deconnection request. */
		bso::bool__ Handle(
			flw::ioflow___ &Channel,
			void *PU = NULL );
		/*f Return the command corresponding at request description 'Description' and
		object type 'Type'. 'BKDMNG_INVALID_COMMAND' is returned if command not found. */
		command__ Command(
			type__ IdType,
			const description_ &Description )
		{
			return Module_( IdType ).Command( Description );
		}
		//f Return the error breaking status.
		bso::bool__ ErrorBreaking( void  )
		{
			return Master_.ErrorBreaking();
		}
		//f Set to 'Value' the error breaking status.
		void ErrorBreaking( bso::bool__ Value )
		{
			Master_.ErrorBreaking( Value );
		}
		//f Give the module for the object of type 'Type'.
		const untyped_module &Module( type__ Type ) const
		{
			return Module_( Type );
		}
		//f Give the module for object 'Object'.
		const untyped_module &Module( object__ Object ) const
		{
			return Module_( Object );
		}
		//f Return a pointer to the 'Object' object.
		const void *Object( object__ Object )
		{
			return Module_( Object ).Object( Links.Index( Object ) );
		}
		//f Give a new object.
		object__ New( type__ Type )
		{
			if ( (unsigned long)Links.Amount() >= (unsigned long)BKDMNG_TYPE_MAX )
				ERRl();

			return Links.New( Type, Module_( Type ).New() );
		}
		//f Give the type of the object 'Object'.
		type__ Type( object__ Object)
		{
			return Type_( Object );
		}
		//f Give the type of a nobject named 'Name'.
		type__ Type( str::string_ &Name );
		//f Remove object 'Object'.
		void Remove( object__ Object )
		{
			Module_( Object ).Remove( Links.Index( Object ) );
		}
		/*f Tell that 'Amount' language indicated in 'Languages' are supported.
		'Languages' is not copied and should NOT be modified. */
		void Languages(
			bso::ushort__ Amount,
			const lgg::language *Languages )
		{
			Langues_.Nombre = Amount;
			Langues_.Identificateurs = Languages;
		}
		//f Return the current language.
		lgg::language Language( void )
		{
			return Langues_.Courante;
		}
		/*f 'Language' brcomes the current language. 'Language' must, of course' be supported. */
		void Langue( lgg::language Langue )
		{
			Langues_.Courante = Langue;
		}
		/*f Add a request descrption with name 'Name', function pointer 'FP'
		and a list of casts 'Casts'. The list must contain 2 'cEnd', the first
		at the end of the parameters casts,	and 1 of the end of returned values casts. */
		tym::row__ Add(
			const char *Name,
			function__ FP,
			const cast *Casts )
		{
			return Master_.Add( Name, Casts, (void *)FP );
		}
		tym::row__ Add(
			const char *Name,
			function__ FP,
			cast Cast,	// Added to avoid confusion with function above.
			... )
		{
			return Add( Name, FP, &Cast );
		}

	};

	typedef backend backend_;

};

//d A ram module of an object of type 't'.
#define BKDMNG_RAM_MODULE( t )	bkdmng::ram_module<t, t::s>	

//d A standard module of an object of type 't'.
#define BKDMNG_STANDARD_MODULE( t )	bkdmng::standard_module<t, t::s>	


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

#endif
