/*
  Header for the 'bkdrqm' library by Claude L. Simon (csimon@epeios.org)
  Copyright (C) 2000,2001 Claude L. SIMON (csimon@epeios.org) 

  This file is part of the Epeios (http://epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: bkdrqm.h,v 1.3 2001/10/30 14:49:34 csimon Exp $

#ifndef BKDRQM__INC
#define BKDRQM__INC

#define BKDRQM_NAME		"BKDRQM"

#define	BKDRQM_VERSION	"$Revision: 1.3 $"	

#define BKDRQM_OWNER		"the Epeios project (http://epeios.org/)"

#include "ttr.h"

extern class ttr_tutor &BKDRQMTutor;

#if defined( XXX_DBG ) && !defined( BKDRQM_NODBG )
#define BKDRQM_DBG 
#endif

/* Begin of automatic documentation generation part. */

//V $Revision: 1.3 $
//C Claude L. SIMON (csimon@epeios.org)
//R $Date: 2001/10/30 14:49:34 $

/* End of automatic documentation generation part. */

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

/* Addendum to the automatic documentation generation part. */
//D BacKenD ReQuest Manager.
/* End addendum to automatic documentation generation part. */


#include "err.h"
#include "flw.h"
#include "bso.h"
#include "str.h"
#include "ctn.h"
#include "bkdtpm.h"
#include "bkdcst.h"

#ifdef CPE__VC
#	undef GetObject
#endif

#define BKDRQM_INVALID_COMMAND		((unsigned long)BSO_USHORT_MAX)
#define BKDRQM_COMMAND_MAX			( BKDRQM_INVALID_COMMAND - 1UL )

#define BKDRQM_INVALID_TYPE			((unsigned long)BSO_USHORT_MAX)
#define BKDRQM_TYPE_MAX				( BKDRQM_INVALID_TYPE - 1UL )

#define BKDRQM_INVALID_OBJECT		((unsigned long)BSO_USHORT_MAX)
#define BKDRQM_OBJECT_MAX			( BKDRQM_INVALID_OBJECT - 1UL )

namespace bkdrqm {
	namespace {
		using namespace bkdcst;
		using namespace ctn;
	}

	using namespace bkdtpm;

	typedef id8__ cast__;
	using bkdcst::cast;

	//c The description of a request.
	class description_
	{
	public:
		//o Name of the command.
		str::string_ Name;
		/*o Parameters and return value types. Separated by 'cEnd'. The 'cEnd'
		to signalize the end of the request must NOT be put. */
		ids8_ Casts;
		struct s
		{
			str::string_::s Name;
			ids8_::s Casts;
		} &S_;
		description_( s &S )
		: S_( S ),
		  Name( S.Name ),
		  Casts( S.Casts )
		{}
		void reset( bool P = true )
		{
			Name.reset( P );
			Casts.reset( P );
		}
		description_ &operator =( const description_ &D )
		{
			Name = D.Name;
			Casts = D.Casts;

			return *this;
		}
		void plug( mmm::multimemory_ &M )
		{
			Name.plug( M );
			Casts.plug( M );
		}
		//f Initialization. The name ans casts are added one per one.
		void Init( void )
		{
			Name.Init();
			Casts.Init();
		}
		/*f Initializing with name 'Name' and a the list of casts 'Casts'. The list
		must contain 2 'cEnd', the first at the end of the parameters casts,
		and 1 of the end of returned values casts. */
		void Init(
			const char *Name,
			const cast *Casts );
		//f Initialization with name 'Name'. The casts would be given using 'New()'.
		void Init( const char *Name )
		{
			this->Name.Init();
			Casts.Init();
			
			this->Name = Name;
		}
		//f Add 'Cast'.
		void Add( bkdcst::cast Cast )
		{
			Casts.Add( (cast__)Cast );
		}
	};

	AUTO( description )

	//c Requests descriptions.
	class descriptions_
	: public E_XCONTAINER_( description_ )
	{
	public:
		struct s
		: public E_XCONTAINER_( description_ )::s
		{};
		descriptions_( s &S )
		: ctn::E_XCONTAINER_( description_ )( S )
		{}
		void reset( bool P = true )
		{
			E_XCONTAINER_( description_ )::reset( P );
		}
		void plug( mmm::multimemory_ &M )
		{
			E_XCONTAINER_( description_ )::plug( M );
		}
		descriptions_ &operator =( const descriptions_ &D )
		{
			E_XCONTAINER_( description_ )::operator =( D );

			return *this;
		}
		//f Initialization.
		void Init( void )
		{
			E_XCONTAINER_( description_ )::Init();
		}
		//f Adding 'Description'. Return the position where added.
		tym::row__ Add( const description_ &Description )
		{
			return E_XCONTAINER_( description_ )::Add( Description );
		}
		/*f See 'descriptions_::Add()' for the descriptions of the parameters.
		return the position where request description added. */
		tym::row__ Add(
			const char *Name,
			const cast *Casts )
		{
			description Description;

			Description.Init( Name, Casts );
			
			return Add( Description );
		}
		/*f Return the position of the description 'Description'. */
		tym::row__ Position( const description_ &Description ) const;
	};

	AUTO( descriptions )

	//c A request manager.
	class request_manager___
	{
	private:
		// The corresponding request description.
		const description_ *Description_;
		// Position in the Description_;
		tym::row__ Position_;
		// At true if all the answer be sent.
		bso::bool__ Closed_;
		// Cast buffer.
		cast Cast_;
		// At true if the request parsed.
		bso::bool__ Parsed_;
		// The input/output channel for the request.
		flw::ioflow___ *Channel_;
		void Test_( cast Cast )
		{
			if ( Description_->Casts.Read( Position_ ) != Cast )
				ERRu();

			Position_ = Description_->Casts.Next( Position_ );
		}
		void TestInput_( cast Cast )
		{
			Test_( Cast );

			if ( Channel_->Get() != Cast )
				ERRu();
		}
		void TestOutput_( cast Cast )
		{
			if ( !Parsed_ ) {
				Test_( cEnd );
				
				Channel_->Put( 0 );	// Empty explanation message.

				Parsed_ = true;
			}

			Test_( Cast );
			Channel_->Put( Cast );
		}
	public:
		void reset( bool = true )
		{
			Description_ = NULL;

			Position_ = NONE;
			Closed_ = true;
			Cast_ = cInvalid;
			Parsed_ = false;

			Channel_ = NULL;
		}
		request_manager___( void )
		{
			reset( false );
		}
		~request_manager___( void )
		{
			reset( true );
		}
		//f Initialization with 'Channel' to parse/answer the request.
		void Init( flw::ioflow___ &Channel )
		{
			reset();

			Closed_ = false;

			Channel_ = &Channel;
		}
		//f 'Description' is the description of the request being handled.
		void SetDescription( const description_ &Description )
		{
			Description_ = &Description;
			Position_ = Description_->Casts.First();
		}
		//f Get 'Object'.
		void PopObject( object__ &Object )
		{
			TestInput_( cObject );
			bkdtpm::GetObject( *Channel_, Object );
		}
		//f Get 'Boolean'.
		void PopBoolean( boolean__ &Boolean )
		{
			TestInput_( cBoolean );
			bkdtpm::GetBoolean( *Channel_, Boolean );
		}
		//f Get 'Id8'.
		void PopId8( id8__ &Id8 )
		{
			TestInput_( cId8 );
			bkdtpm::GetId8( *Channel_, Id8 );
		}
		//f Get 'IDs8'.
		void PopIds8( ids8_ &Ids8 )
		{
			TestInput_( cIds8 );
			bkdtpm::GetIds8( *Channel_, Ids8 );
		}
		//f Get 'Id16'.
		void PopId16( id16__ &Id16 )
		{
			TestInput_( cId16 );
			bkdtpm::GetId16( *Channel_, Id16 );
		}
		//f Get 'IDs16'.
		void PopIds16( ids16_ &Ids16 )
		{
			TestInput_( cIds16 );
			bkdtpm::GetIds16( *Channel_, Ids16 );
		}
		//f Get 'Id32'.
		void PopId32( id32__ &Id32 )
		{
			TestInput_( cId32 );
			bkdtpm::GetId32( *Channel_, Id32 );
		}
		//f Get 'IDs32'.
		void PopIds32( ids32_ &Ids32 )
		{
			TestInput_( cIds32 );
			bkdtpm::GetIds32( *Channel_, Ids32 );
		}
		//f Get 'Char'.
		void PopChar( char__ &Char )
		{
			TestInput_( cChar );
			bkdtpm::GetChar( *Channel_, Char );
		}
		//f Get 'String'.
		void PopString( string_ &String )
		{
			TestInput_( cString );
			bkdtpm::GetString( *Channel_, String );
		}
		//f Get 'Strings'.
		void PopStrings( strings_ &Strings )
		{
			TestInput_( cStrings );
			bkdtpm::GetStrings( *Channel_, Strings );
		}
		//f Get 'Byte'.
		void PopByte( byte__ &Byte )
		{
			TestInput_( cByte );
			bkdtpm::GetByte( *Channel_, Byte );
		}
		//f Get 'Binary'.
		void PopBinary( binary_ &Binary )
		{
			TestInput_( cBinary );
			bkdtpm::GetBinary( *Channel_, Binary );
		}
		//f Get 'Binaries'.
		void PopBinaries( binaries_ &Binaries )
		{
			TestInput_( cBinaries );
			bkdtpm::GetBinaries( *Channel_, Binaries );
		}
		//f Get 'Items8'.
		void PopItems8( items8_ &Items8 )
		{
			TestInput_( cItems8 );
			bkdtpm::GetItems8( *Channel_, Items8 );
		}
		//f Get 'Items16'.
		void PopItems16( items16_ &Items16 )
		{
			TestInput_( cItems16 );
			bkdtpm::GetItems16( *Channel_, Items16 );
		}
		//f Get 'Items32'.
		void PopItems32( items32_ &Items32 )
		{
			TestInput_( cItems32 );
			bkdtpm::GetItems32( *Channel_, Items32 );
		}
		//f Get 'XItems8'.
		void PopXItems8( xitems8_ &XItems8 )
		{
			TestInput_( cXItems8 );
			bkdtpm::GetXItems8( *Channel_, XItems8 );
		}
		//f Get 'XItems16'.
		void PopXItems16( xitems16_ &XItems16 )
		{
			TestInput_( cXItems16 );
			bkdtpm::GetXItems16( *Channel_, XItems16 );
		}
		//f Get 'XItems32'.
		void PopXItems32( xitems32_ &XItems32 )
		{
			TestInput_( cXItems32 );
			bkdtpm::GetXItems32( *Channel_, XItems32 );
		}
		//f Get 'CommandsDetails'.
		void PopCommandsDetails( commands_details_ &CommandsDetails )
		{
			TestInput_( cCommandsDetails );
			bkdtpm::GetCommandsDetails( *Channel_, CommandsDetails );
		}
		//f Get 'ObjectsReferences'.
		void PopObjectsReferences( objects_references_ &ObjectsReferences )
		{
			TestInput_( cObjectsReferences );
			bkdtpm::GetObjectsReferences( *Channel_, ObjectsReferences );
		}
		//f Put 'Object'.
		void PushObject( object__ Object )
		{
			TestOutput_( cObject );
			bkdtpm::PutObject( Object, *Channel_ );
		}
		//f Put 'Boolean'.
		void PushBoolean( boolean__ Boolean )
		{
			TestOutput_( cBoolean );
			bkdtpm::PutBoolean( Boolean, *Channel_ );
		}
		//f Put 'Id8'.
		void PushId8( id8__ Id8 )
		{
			TestOutput_( cId8 );
			bkdtpm::PutId8( Id8, *Channel_ );
		}
		//f Put 'Ids8'.
		void PushIds8( const ids8_ &Ids8 )
		{
			TestOutput_( cIds8 );
			bkdtpm::PutIds8( Ids8, *Channel_ );
		}
		//f Put 'Id16'.
		void PushId16( id16__ Id16 )
		{
			TestOutput_( cId16 );
			bkdtpm::PutId16( Id16, *Channel_ );
		}
		//f Put 'Ids16'.
		void PushIds16( const ids16_ &Ids16 )
		{
			TestOutput_( cIds16 );
			bkdtpm::PutIds16( Ids16, *Channel_ );
		}
		//f Put 'Id32'.
		void PushId32( id32__ Id32 )
		{
			TestOutput_( cId32 );
			bkdtpm::PutId32( Id32, *Channel_ );
		}
		//f Put 'Ids32'.
		void PushIds8( const ids32_ &Ids32 )
		{
			TestOutput_( cIds32 );
			bkdtpm::PutIds32( Ids32, *Channel_ );
		}
		//f Put 'Char'.
		void PushChar( char__ Char )
		{
			TestOutput_( cChar );
			bkdtpm::PutChar( Char, *Channel_ );
		}
		//f Put 'String'.
		void PushString( const string_ &String )
		{
			TestOutput_( cString );
			bkdtpm::PutString( String, *Channel_ );
		}
		//f Put 'Strings'.
		void PushStrings( const strings_ &Strings )
		{
			TestOutput_( cStrings );
			bkdtpm::PutStrings( Strings, *Channel_ );
		}
		//f Put 'Byte'.
		void PushByte( byte__ Byte )
		{
			TestOutput_( cByte );
			bkdtpm::PutByte( Byte, *Channel_ );
		}
		//f Put 'Binary'.
		void PushBinary( const binary_ &Binary )
		{
			TestOutput_( cBinary );
			bkdtpm::PutBinary( Binary, *Channel_ );
		}
		//f Put 'Binaries'.
		void PushBinaries( const binaries_ &Binaries )
		{
			TestOutput_( cBinaries );
			bkdtpm::PutBinaries( Binaries, *Channel_ );
		}
		//f Put 'Items8'
		void PushItems8( const items8_ &Items8 )
		{
			TestOutput_( cItems8 );
			bkdtpm::PutItems8( Items8, *Channel_ );
		}
		//f Put 'Items16'
		void PushItems16( const items16_ &Items16 )
		{
			TestOutput_( cItems16 );
			bkdtpm::PutItems16( Items16, *Channel_ );
		}
		//f Put 'Items32'
		void PushItems32( const items32_ &Items32 )
		{
			TestOutput_( cItems32 );
			bkdtpm::PutItems32( Items32, *Channel_ );
		}
		//f Put 'XItems8'
		void PushXItems8( const xitems8_ &XItems8 )
		{
			TestOutput_( cXItems8 );
			bkdtpm::PutXItems8( XItems8, *Channel_ );
		}
		//f Put 'XItems16'
		void PushXItems16( const xitems16_ &XItems16 )
		{
			TestOutput_( cXItems16 );
			bkdtpm::PutXItems16( XItems16, *Channel_ );
		}
		//f Put 'XItems32'
		void PushXItems32( const xitems32_ &XItems32 )
		{
			TestOutput_( cXItems32 );
			bkdtpm::PutXItems32( XItems32, *Channel_ );
		}
		//f Put 'CommandsDetails'.
		void PushCommandsDetails( const commands_details_ &CommandsDetails )
		{
			TestOutput_( cCommandsDetails );
			bkdtpm::PutCommandsDetails( CommandsDetails, *Channel_ );
		}
		//f Put 'ObjectsReferences'.
		void PushObjectsReferences( const objects_references_ &ObjectsReferences )
		{
			TestOutput_( cObjectsReferences );
			bkdtpm::PutObjectsReferences( ObjectsReferences, *Channel_ );
		}
		//f Tell that the request is complete (parsed and answered).
		void Complete( void )
		{
			if ( Closed_ )
				return;
				
			if ( Description_ != NULL ) /* If == 'NULL', it means that the request was handled
								   by handling DIRECTLY the underlying flows. */
			{
				if ( !Parsed_ ) {
					Test_( cEnd );
					
					Channel_->Put( 0 );	// Empty explanation message.
				}
			
				if ( Channel_->Get() != bkdcst::cEnd )
					ERRb();

				if ( Position_ != NONE )
					ERRu();
			}

			bkdtpm::PutId8( cEnd, *Channel_ );

			Closed_ = true;

			Parsed_ = false;

			Channel_->Synchronize();
		}
		//f Send a message that explain the reason of no treatment.
		void SendExplanationMessage( const char *Message )
		{
			if ( Description_ != NULL )
				if ( Parsed_ )
					ERRb();
				else
					TestInput_( cEnd );

			if ( !Message[0] )
				ERRu();

			flw::PutString( Message, *Channel_ );

			Closed_ = true;

			Channel_->Synchronize();
		}
		//f Return the channel used to handle the request as input flow.
		flw::iflow___ &Input( void )
		{
			return *Channel_;
		}
		//f Return the channel used to handle the request as ouput flow.
		flw::oflow___ &Output( void )
		{
			return *Channel_;
		}
	};
}

/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

#endif
