/*
  'bkdtpm' library by Claude L. Simon (csimon@epeios.org)
  Requires the 'bkdtpm' header file ('bkdtpm.h').
  Copyright (C) 2000,2001 Claude L. SIMON (csimon@epeios.org).

  This file is part of the Epeios (http://epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: bkdtpm.cpp,v 1.3 2001/11/02 10:32:08 csimon Exp $

#define BKDTPM__COMPILATION

#include "bkdtpm.h"

class bkdtpmtutor
: public ttr_tutor
{
public:
	bkdtpmtutor( void )
	: ttr_tutor( BKDTPM_NAME )
	{
#ifdef BKDTPM_DBG
		Version = BKDTPM_VERSION "\b\bD $";
#else
		Version = BKDTPM_VERSION;
#endif
		Owner = BKDTPM_OWNER;
		Date = "$Date: 2001/11/02 10:32:08 $";
	}
	virtual ~bkdtpmtutor( void ){}
};

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

using namespace bkdtpm;

#define M1	255
#define M2	65535
#define M3	16777215

namespace {
	inline void PutSize_(
		bso::ulong__ Size,
		flw::oflow___ &Flow )
	{
		if ( Size >= M1 )
		{
			Flow.Put( M1 );
			Size -= M1;

			if ( Size >= M2 )
			{
				Flow.Put( M1 );
				Flow.Put( M1 );
				Size -= M2;

				if ( Size >= M3 )
				{
					Flow.Put( M1 );
					Flow.Put( M1 );
					Flow.Put( M1 );

					Size -= M3;

					Flow.Put( ( (flw::data__)( Size & ( 255 << 24 ) ) >> 24 ) );
				}
				Flow.Put( (flw::data__)( ( Size & ( 255 << 16 ) ) >> 16 ) );
			}
			Flow.Put( (flw::data__)( ( Size & ( 255 << 8 ) ) >> 8 ) );
		}
		Flow.Put( (flw::data__)( Size & 255 ) );
	}

	inline bso::ulong__ GetSize_( flw::iflow___ &IFlow )
	{
		bso::ulong__ Size = IFlow.Get();

		if ( Size == M1 )
		{
			Size += ( IFlow.Get() << 8 ) | IFlow.Get();

			if ( Size == ( M1 + M2 ) )
			{
				Size += ( IFlow.Get() << 16 ) | ( IFlow.Get() << 8 ) | IFlow.Get();

				if ( Size == ( M1 + M2 + M3 ) )
					Size += ( IFlow.Get() << 24 ) | ( IFlow.Get() << 16 ) | ( IFlow.Get() << 8 ) | IFlow.Get();
			}
		}

		return Size;
	}
}

namespace {
	template <typename s> inline void PutSet_(
		const s &S,
		flw::oflow___ &OFlow )
	{
		tym::row__ P = S.First();

		PutSize_( S.Amount(), OFlow );

		while( P != NONE ) {
			flw::Put( S( P ), OFlow );
			P = S.Next( P );
		}
	}

	template <typename s, typename i> inline void GetSet_(
		flw::iflow___ &IFlow,
		s &S )
	{
		i I;
		bso::ulong__ Size = GetSize_( IFlow );

		while( Size-- ) {
			flw::Get( IFlow, I );
			S.Add( I );
		}
	}
}

void bkdtpm::PutIds8(
	const ids8_ &Ids8,
	flw::oflow___ &OFlow )
{
	PutSet_( Ids8, OFlow );
}

void bkdtpm::GetIds8(
	flw::iflow___ &IFlow,
	ids8_ &Ids8 )
{
	GetSet_<ids8_, id8__>( IFlow, Ids8 );
}

void bkdtpm::PutIds16(
	const ids16_ &Ids16,
	flw::oflow___ &OFlow )
{
	PutSet_( Ids16, OFlow );
}

void bkdtpm::GetIds16(
	flw::iflow___ &IFlow,
	ids16_ &Ids16 )
{
	GetSet_<ids16_, id16__>( IFlow, Ids16 );
}

void bkdtpm::PutIds32(
	const ids32_ &Ids32,
	flw::oflow___ &OFlow )
{
	PutSet_( Ids32, OFlow );
}

void bkdtpm::GetIds32(
	flw::iflow___ &IFlow,
	ids32_ &Ids32 )
{
	GetSet_<ids32_, id32__>( IFlow, Ids32 );
}

namespace {
	template <typename t> inline void Put_(
		const item_<t> &I,
		flw::oflow___ &OFlow )
	{
		flw::Put( I.ID(), OFlow );
		OFlow << I.Value;
	}

	template <typename t> inline void Get_(
		flw::iflow___ &IFlow,
		item_<t> &I )
	{
		flw::Get( IFlow, I.S_.ID );
		IFlow >> I.Value;
	}

	inline void Put_(
		const str::string_ &String,
		flw::oflow___ &OFlow )
	{
		OFlow << String;
	}

	inline void Get_(
		flw::iflow___ &IFlow,
		string_ &String )
	{
		IFlow >> String;
	}

	inline void Put_(
		const binary_ &Binary,
		flw::oflow___ &OFlow )
	{
		PutSet_( Binary, OFlow );
	}

	inline void Get_(
		flw::iflow___ &IFlow,
		binary_ &Binary )
	{
		GetSet_<binary_, bso::raw__>( IFlow, Binary );
	}

	inline void Put_(
		const command_detail_ &CD,
		flw::oflow___ &OFlow )
	{
		OFlow << CD.Name;
		PutSet_( CD.Casts, OFlow );
	}

	inline void Get_(
		flw::iflow___ &IFlow,
		command_detail_ &CD )
	{
		IFlow >> CD.Name;
		GetSet_<bch::E_BUNCH_( id8__ ), id8__>( IFlow, CD.Casts );
	}

	template <typename t> inline void Put_(
		const extended_item_<t> &I,
		flw::oflow___ &OFlow );

	template <typename c, typename i> inline void GenericPutContainer_(
		const c &C,
		flw::oflow___ &OFlow )
	{
		i Item;
		tym::row__ P = C.First();

		Item.Init( C );

		PutSize_( C.Amount(), OFlow );

		while ( P != NONE ) {
			Put_( Item( P ), OFlow );
			P = C.Next( P );
		}
	}

	template <typename c, typename i> inline void PutContainer_(
		const c &C,
		flw::oflow___ &OFlow )
	{
		GenericPutContainer_<c, ctn::E_CITEM( i )>( C, OFlow );
	}

	template <typename c, typename i> inline void PutMContainer_(
		const c &C,
		flw::oflow___ &OFlow )
	{
		GenericPutContainer_<c, ctn::E_CMITEM( i )>( C, OFlow );
	}

	template <typename t> inline void Put_(
		const extended_item_<t> &I,
		flw::oflow___ &OFlow )
	{
		flw::Put( I.ID(), OFlow );
		PutMContainer_<strings_, string_>( I.Values, OFlow );
	}

	template <typename c, typename i> inline void GetContainer_(
		flw::iflow___ &IFlow,
		c &C )
	{
		i Item;
		bso::ulong__ Size = GetSize_( IFlow );

		while( Size-- ) {
			Item.Init();

			Get_( IFlow, Item );
			C.Add( Item );
		}
	}

	template <typename t> inline void Get_(
		flw::iflow___ &IFlow,
		extended_item_<t> &I )
	{
		flw::Get( IFlow, I.S_.ID );
		GetContainer_<strings_, string>( IFlow, I.Values );
	}
}

void bkdtpm::PutStrings(
	const strings_ &Strings,
	flw::oflow___ &OFlow )
{
	PutMContainer_<strings_, string_>( Strings, OFlow );
}

void bkdtpm::GetStrings(
   flw::iflow___ &IFlow,
   strings_ &Strings )
{
	GetContainer_<strings_, string>( IFlow, Strings );
}

void bkdtpm::PutBinary(
	const binary_ &Binary,
	flw::oflow___ &OFlow )
{
	PutSet_( Binary, OFlow );
}

void bkdtpm::GetBinary(
   flw::iflow___ &IFlow,
   binary_ &Binary )
{
	GetSet_<binary_, bso::raw__>( IFlow, Binary );
}

void bkdtpm::PutBinaries(
	const binaries_ &Binaries,
	flw::oflow___ &OFlow )
{
	PutMContainer_<binaries_, binary_>( Binaries, OFlow );
}

void bkdtpm::GetBinaries(
	flw::iflow___ &IFlow,
	binaries_ &Binaries )
{
	GetContainer_<binaries_, binary>( IFlow, Binaries );
}

void bkdtpm::PutItems8(
	const items8_ &Items8,
	flw::oflow___ &OFlow )
{
	PutMContainer_<items8_, item8_>( Items8, OFlow );
}

void bkdtpm::GetItems8(
   flw::iflow___ &IFlow,
   items8_ &Items8 )
{
	GetContainer_<items8_, item8>( IFlow, Items8 );
}

void bkdtpm::PutItems16(
	const items16_ &Items16,
	flw::oflow___ &OFlow )
{
	PutMContainer_<items16_, item16_>( Items16, OFlow );
}

void bkdtpm::GetItems16(
   flw::iflow___ &IFlow,
   items16_ &Items16 )
{
	GetContainer_<items16_, item16>( IFlow, Items16 );
}

void bkdtpm::PutItems32(
	const items32_ &Items32,
	flw::oflow___ &OFlow )
{
	PutMContainer_<items32_, item32_>( Items32, OFlow );
}

void bkdtpm::GetItems32(
   flw::iflow___ &IFlow,
   items32_ &Items32 )
{
	GetContainer_<items32_, item32>( IFlow, Items32 );
}

void bkdtpm::PutXItems8(
	const xitems8_ &XItems8,
	flw::oflow___ &OFlow )
{
	PutContainer_<xitems8_, xitem8_>( XItems8, OFlow );
}

void bkdtpm::GetXItems8(
   flw::iflow___ &IFlow,
   xitems8_ &XItems8 )
{
	GetContainer_<xitems8_, xitem8>( IFlow, XItems8 );
}

void bkdtpm::PutXItems16(
	const xitems16_ &XItems16,
	flw::oflow___ &OFlow )
{
	PutContainer_<xitems16_, xitem16_>( XItems16, OFlow );
}

void bkdtpm::GetXItems16(
   flw::iflow___ &IFlow,
   xitems16_ &XItems16 )
{
	GetContainer_<xitems16_, xitem16>( IFlow, XItems16 );
}

void bkdtpm::PutXItems32(
	const xitems32_ &XItems32,
	flw::oflow___ &OFlow )
{
	PutContainer_<xitems32_, xitem32_>( XItems32, OFlow );
}

void bkdtpm::GetXItems32(
   flw::iflow___ &IFlow,
   xitems32_ &XItems32 )
{
	GetContainer_<xitems32_, xitem32>( IFlow, XItems32 );
}

void bkdtpm::PutObjectsReferences(
	const objects_references_ &ObjectsReferences,
	flw::oflow___ &OFlow )
{
	PutSet_( ObjectsReferences, OFlow );
}

void bkdtpm::GetObjectsReferences(
	flw::iflow___ &IFlow,
	objects_references_ &ObjectsReferences )
{
	GetSet_<objects_references_, object_reference__>( IFlow, ObjectsReferences );
}

void bkdtpm::PutCommandsDetails(
	const bkdtpm::commands_details_ &CommandsDetails,
	flw::oflow___ &OFlow )
{
	PutContainer_<commands_details_, command_detail_ >( CommandsDetails, OFlow );
}

void bkdtpm::GetCommandsDetails(
	flw::iflow___ &IFlow,
	bkdtpm::commands_details_ &CommandsDetails )
{
	GetContainer_<commands_details_, command_detail >( IFlow, CommandsDetails );
}

/* Although in theory this class is inaccessible to the different modules,
it is necessary to personalize it, or certain compiler would not work properly */

class bkdtpmpersonnalization
: public bkdtpmtutor
{
public:
	bkdtpmpersonnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the launching of the application  */
	}
	~bkdtpmpersonnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the ending of the application  */
	}
};


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

// 'static' by GNU C++.

static bkdtpmpersonnalization Tutor;

ttr_tutor &BKDTPMTutor = Tutor;
