/*
  'mthitg' library by Claude L. Simon (csimon@epeios.org)
  Requires the 'mthitg' header file ('mthitg.h').
  Copyright (C) 2000,2001 Claude L. SIMON (csimon@epeios.org).

  This file is part of the Epeios (http://epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: mthitg.cpp,v 1.1 2001/11/16 17:09:06 csimon Exp $

#define MTHITG__COMPILATION

#include "mthitg.h"

class mthitgtutor
: public ttr_tutor
{
public:
	mthitgtutor( void )
	: ttr_tutor( MTHITG_NAME )
	{
#ifdef MTHITG_DBG
		Version = MTHITG_VERSION "\b\bD $";
#else
		Version = MTHITG_VERSION;
#endif
		Owner = MTHITG_OWNER;
		Date = "$Date: 2001/11/16 17:09:06 $";
	}
	virtual ~mthitgtutor( void ){}
};

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

#include <math.h>
#include "stf.h"

using namespace mthitg;

// Return the rest of the operation, and 'Res' is shifted before adding the result of the operation.
integer mthitg::Divide_(
	const integer_ &Num,
	const integer_ &Den,
	integer_ &Res )
{
	bso::ulong__ Inf = 0UL, Sup = 0x10000UL, Rep = 0x10000UL >> 1;
	integer Inter, Comp;

	Inter.Init();
	Comp.Init();

	Inter = 0;
	Comp = 0;
	
	for (;;) {
		if ( ( Inter = Num - Den * integer( Rep ) ) < Comp )
		{
			Sup = Rep;
			Rep = ( Sup + Inf ) >> 1;
		}
		else if ( Inter >= Den )
		{
			Inf = Rep;
			Rep = Inf + ( ( Sup - Inf ) >> 1 );
				// ( Sup + Inf ) >> 1 ne marche pas (dpassement de capacit)
				// ni ( Sup >> 1 ) + ( Inf >> 1 ) (perte de prcision)
		}
		else
		{
			Res = Res * integer( 0x10000L ) + integer( Rep );
			return Inter;
		}
	}
}

int mthitg::Comp_(
	const integer_ &Op1,
	const integer_ &Op2 )
{
	int Res;

	if ( Op1.GetSign_() < Op2.GetSign_() )
		return 1;
	else if ( Op2.GetSign_() < Op1.GetSign_() )
		return -1;

	if ( Op1.GetSize_() < Op2.GetSize_() )
		Res = -1;
	else if ( Op1.GetSize_() > Op2.GetSize_() )
		Res = 1;
	else
	{
		size__ Indice = Op2.GetSize_();

		do
		{
			Indice--;
		}
		while( ( Op1.Core( Indice )
				 == Op2.Core( Indice ) )
				&& Indice );

		if ( Op1.Core( Indice ) != Op2.Core( Indice ) )
			if ( Op1.Core( Indice ) > Op2.Core( Indice ) )
				Res = 1;
			else
				Res = -1;
		else return 0;

	}

	return (int)( Op1.GetSign_() ? -Res : Res );
}		

integer mthitg::Add_(
	const integer_ &Op1,
	const integer_ &Op2 )
{
	integer ResTemp;
	const integer_ *Max = &Op1, *Min = &Op2;
	bso::ushort__ Indice = 0;
	bso::ulong__ Retenue = 0;
	bso::ushort__ Limite;

	ResTemp.Init();

	if ( ( Max->GetSize_() ) < ( Min->GetSize_() ) )
	{
		Max = &Op2;
		Min = &Op1;
	}

	ResTemp.Core.Allocate( Max->GetSize_() + 1 );

	Limite = Min->GetSize_();

	while( Indice < Limite )
	{
		Retenue = ( Retenue >> 16 )
			  + Min->Core( Indice )
			  + Max->Core( Indice);

		ResTemp.Core.Write( (base__)( Retenue & 0xffff ), Indice++ );
	}

	Limite = Max->GetSize_();

	while( Indice < Limite )
	{
		Retenue = ( Retenue >> 16 )
			  + Max->Core( Indice );

		ResTemp.Core.Write( (base__)( Retenue & 0xffff ), Indice++ );
	}

	if ( Indice >= MTHITG_MAX_SIZE )
		ERRl();

	ResTemp.Core.Write( (base__)( Retenue >> 16 ), Indice++ );
	ResTemp.PutSize_( Indice );
	ResTemp.PutSign_( 0 );
	ResTemp.Adjust_();

	return ResTemp;
}
	
integer mthitg::Sub_(
	const integer_ &Op1,
	const integer_ &Op2 )
{
	integer Res;
	unsigned Indice = 0;
	size__ Limite;
	bso::ulong__ Inter = 0;
	const integer_ *Max = &Op1, *Min = &Op2;

	Res.Init();

	if ( *Max < *Min )
	{
		Res.PutSign_( 1 );
		Max = &Op2;
		Min = &Op1;
	}
	else
		Res.PutSign_( 0 );

	Res.Core.Allocate( Max->GetSize_() );

	Res.PutSize_( Max->GetSize_() );

	Limite = Min->GetSize_();

	while ( Indice < Limite )
	{
		if ( Max->Core( Indice ) < ( Inter += Min->Core( Indice ) ) )
		{
			Res.Core.Write(
				( (base__)( ( Max->Core( Indice ) | ( 1 << 16 ) )
					- ( Inter & 0xffff ) ) ),
				Indice );

			Inter = 1;
		}
		else
		{
			Res.Core.Write( (base__)( Max->Core(Indice) - ( Inter & 0xffff ) ), Indice );

			Inter = 0;
		}

		Indice++;
	}

	Limite = Max->GetSize_();

	while( Indice < Limite )
	{
		if ( Max->Core( Indice ) < Inter )

		{
			Res.Core.Write(
				( (base__)( ( Max->Core( Indice ) | ( 1 << 16 ) )
					- ( Inter & 0xffff ) ) ),
					Indice );

			Inter = 1;
		}
		else
		{
			Res.Core.Write( (base__)( Max->Core( Indice ) - ( Inter & 0xffff ) ), Indice );

			Inter = 0;
		}

		Indice++;
	}

	Res.Adjust_();

	return Res;
}


integer mthitg::Mul_(
	const integer_ &Op1,
	const integer_ &Op2 )
{
	integer Inter, Res;
	unsigned long Retenue;
	size__ LimiteS, LimiteP, IndiceS = 0, IndiceP = 0;
	const integer_ *Max = &Op1, *Min = &Op2;

	Inter.Init();
	Res.Init();

	Res = 0;

	if ( *Max < *Min )
	{
		Max = &Op2;
		Min = &Op1;
	}

	if ( ( Max->GetSize_() + Min->GetSize_() ) > MTHITG_MAX_SIZE )
		ERRl();

	Inter.PutSign_( 0 );

	Inter.Core.Allocate( Max->GetSize_() + Min->GetSize_() );

	LimiteS = Min->GetSize_();
	LimiteP = Max->GetSize_();

	while ( IndiceS < LimiteS )
	{
		IndiceP = 0;
		Retenue = 0;

		while ( IndiceP < LimiteP )
		{
			Retenue = ( Retenue >> 16 )
				  + (unsigned long)Max->Core(IndiceP)
					* (unsigned long)Min->Core(IndiceS);
			Inter.Core.Write( (base__)( Retenue & 0xffff ), IndiceP+IndiceS );
			IndiceP++;
		}

		Inter.Core.Write( (base__)( Retenue >> 16 ), IndiceP+IndiceS );
		Inter.PutSize_( IndiceP + IndiceS + 1 );

		Res += Inter;
		IndiceS++;

		for( size__ C = 0; C < LimiteS + LimiteP; C++ )
			Inter.Core.Write( 0, C );

	}

	Res.Adjust_();

	return Res;
}


integer mthitg::Div_(
	const integer_ &Num,
	const integer_ &Den,
	integer_ &Reste )
{
	integer Res;
	unsigned long I;

	Res.Init();
	Res = 0;

	if ( Den > Num )
	{
		Reste = Num;
		return Res;
	}

	Reste.Init();
	
	Reste.Core.Allocate( Den.GetSize_() );

	Reste.PutSize_( Den.GetSize_() );
	Reste.PutSign_( 0 );

	Reste.Core.Write( Num.Core, Den.GetSize_(), I = Num.GetSize_() - Den.GetSize_() );
	
	while( I-- )
	{
		Reste = Divide_( Reste, Den, Res ) * integer( 0x10000UL ) + integer( Num.Core( I ) );

/*		Reste = Divide_( Reste, Den, Res );

		Reste.Core.Allocate( Reste.GetSize_() + 1 );
		Reste.Core.Write( Reste.Core, Reste.GetSize_(), 1 );
		Reste.PutSize_( Reste.GetSize_() + 1 );
		Reste.Core.Write( Num.Core( I ), 0 );
*/	}

	Reste = Divide_( Reste, Den, Res );

	return Res;
}


integer integer_::Add( const integer_ &Op2 ) const
{
	const integer_ &Op1 = *this;

	if ( Op1.GetSign_() && Op2.GetSign_() )
		return -Add_( -Op1, -Op2 );
	else if ( Op1.GetSign_() )
		return Sub_( Op2, -Op1 );
	else if ( Op2.GetSign_() )
		return Sub_( Op1, -Op2 );
	else
		return Add_( Op1, Op2 );
}

integer integer_::Sub( const integer_ &Op2 ) const
{
	const integer_ &Op1 = *this;

	if ( Op1.GetSign_() && Op2.GetSign_() )
		return Sub_( -Op2, -Op1 );
	else if ( Op1.GetSign_() )
		return -Add_( -Op1, Op2 );
	else if ( Op2.GetSign_() )
		return Add_( Op1, -Op2 );
	else
		return Sub_( Op1, Op2 );
}

integer integer_::Mul( const integer_ &Op2 ) const
{
	const integer_ &Op1 = *this;

	if ( Op1.GetSign_() && Op2.GetSign_() )
		return Mul_( -Op2, -Op1 );
	else if ( Op1.GetSign_() )
		return -Mul_( -Op1, Op2 );
	else if ( Op2.GetSign_() )
		return -Mul_( Op1, -Op2 );
	else
		return Mul_( Op1, Op2 );
}

integer integer_::Div( const integer_ &Den ) const
{
	const integer_ &Num = *this;
	integer Dummy;

	Dummy.Init();
	
	if ( Num.GetSign_() != Den.GetSign_() )
		return -Div_( ::Abs( Num ), ::Abs( Den ), Dummy );
	else
		return Div_( ::Abs( Num ), ::Abs( Den ), Dummy );
}

integer integer_::Mod( const integer_ &Den ) const
{
	const integer_ &Num = *this;
	integer Result;

	Result.Init();

	Div_( ::Abs( Num ), ::Abs( Den ), Result );

	if ( Num.GetSign_() != Den.GetSign_() )
		Result = Den - Result;

	if ( Num.GetSign_() )
		Result = -Result;

	return Result;
}

	
void integer_::Adjust_( aem::mode Mode )
{
	size__ Limit = GetSize_();

	while( --Limit && !Core( Limit ) );

	if ( Limit < ( GetSize_() - 1 ) )
	{
		Core.Allocate( Limit + 1 );

		PutSize_( Limit + 1 );
	}
}

bso::lfloat__ integer_::GetLongFloat( void ) const
{
	bso::lfloat__ Resultat = 0;
	size__ Compteur = GetSize_();

	while( Compteur-- )
		Resultat += Core(Compteur) * pow( 0x10000UL, Compteur );

	if ( GetSign_() )
		return -Resultat;
	else
		return Resultat;
}

integer Exp(
	const integer_ &X,
	int Y )
{
	integer Res;

	Res.Init();
	Res = 1;
	
	while( Y-- )
		Res *= X;
		
	return Res;
}

txf::text_oflow___ &operator <<(
	txf::text_oflow___ &Flow,
	const integer_ &Integer )
{
	int Compteur, Chiffre;
	integer I;

	I.Init();

	I = Integer;

	if ( !I )
		return Flow << 0UL;
	else if ( I < integer( 0 ) )
	{
		Flow << '-';
		I = -I;
	}

	Compteur = (int)log10( I.GetLongFloat() );

	while( Compteur )
	{
		Chiffre =  (int)( I.GetLongFloat() / pow( 10, Compteur ) );

		Flow << (unsigned long)Chiffre;

		I -= Exp( integer( 10 ), Compteur ) * integer( Chiffre );
		Compteur--;
	}

	return Flow << I.GetLongFloat();
}

/* Although in theory this class is inaccessible to the different modules,
it is necessary to personalize it, or certain compiler would not work properly */

class mthitgpersonnalization
: public mthitgtutor
{
public:
	mthitgpersonnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the launching of the application  */
	}
	~mthitgpersonnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the ending of the application  */
	}
};


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

// 'static' by GNU C++.

static mthitgpersonnalization Tutor;

ttr_tutor &MTHITGTutor = Tutor;
