/*
  'tmcbsc_' library by Claude L. Simon (epeios@epeios.org)
  Requires the 'tmcbsc_' header file ('tmcbsc_.h').
  Copyright (C) 2000 Claude L. SIMON (epeios@epeios.org).

  This file is part of the Epeios (http://www.epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: tmcbsc_.cpp,v 1.7 2001/07/04 08:13:56 uid30395 Exp $

#define TMCBSC___COMPILATION

#include "tmcbsc_.h"

class tmcbsc_tutor
: public ttr_tutor
{
public:
	tmcbsc_tutor( void )
	: ttr_tutor( TMCBSC__NAME )
	{
#ifdef TMCBSC__DBG
		Version = TMCBSC__VERSION "\b\bD $";
#else
		Version = TMCBSC__VERSION;
#endif
		Owner = TMCBSC__OWNER;
		Date = "$Date: 2001/07/04 08:13:56 $";
	}
	virtual ~tmcbsc_tutor( void ){}
};

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

using namespace tmcbsc_;

// MUST reflect the order of 'tmcbsc::format'.
static const char *FormatsLabels[tmcbsc::f_amount] = {
	"h:m:s:i",
	"m's\"i"
};

// MUST reflect the order of 'tmcbsc::standard'.
static const char *StandardsLabels[tmcbsc::s_amount] = {
	"25 i/s",
	"30 i/s NDF",
	"30 i/s DF"
};

static void GetStandards_(
	bkdmng::backend_ &,
	bkdmng::untyped_module &,
	bkdmng::command__,
	bkdrqm::request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
ERRProlog
	bkdmng::items8 Items;
	bkdmng::item8 Item;
ERRBegin
	Items.Init();

	for( bso::ubyte__ i = 0; i < tmcbsc::s_amount; i++ ) {
		Item.Init();

		Item.Value = StandardsLabels[i];
		Item.ID( i );

		Items.Add( Item );
	}

	Requete.PushItems8( Items );
	Requete.Complete();
ERRErr
ERREnd
ERREpilog
}

static inline void GetStandardLabel_(
	bkdmng::backend_ &,
	bkdmng::untyped_module &,
	bkdmng::command__,
	bkdrqm::request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
	bkdmng::id8__ ID;

	Requete.PopId8( ID );

	if ( *ID >= tmcbsc::s_amount )
		Requete.SendExplanationMessage( "No such standard id." );
	else
		Requete.PushString( str::string( StandardsLabels[*ID] ) );
}

static void GetFormats_(
	bkdmng::backend_ &,
	bkdmng::untyped_module &,
	bkdmng::command__,
	bkdrqm::request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
ERRProlog
	bkdmng::items8 Items;
	bkdmng::item8 Item;
ERRBegin
	Items.Init();

	for( bso::ubyte__ i = 0; i < tmcbsc::f_amount; i++ ) {
		Item.Init();

		Item.Value = FormatsLabels[i];
		Item.ID( i );

		Items.Add( Item );
	}

	Requete.PushItems8( Items );
	Requete.Complete();
ERRErr
ERREnd
ERREpilog
}

static inline void GetFormatLabel_(
	bkdmng::backend_ &,
	bkdmng::untyped_module &,
	bkdmng::command__,
	bkdrqm::request_manager___ &Requete,
	bso::bool__ &,
	void * )
{
	bkdmng::id8__ ID;

	Requete.PopId8( ID );

	if ( *ID >= tmcbsc::f_amount )
		Requete.SendExplanationMessage( "No such format id." );
	else
		Requete.PushString( str::string( FormatsLabels[*ID] ) );

	Requete.Complete();
}

static inline void Add_(
	tmcbsc_::command Command,
	bkdmng::function__ Function,
	bkdmng::backend_ &Backend )
{
	Backend.Add( CommandsNames[Command], Function, CommandsParameters[Command] );
}



void tmcbsc_::Inform( bkdmng::backend_ &Backend )
{
	Backend.Add( "TMCGetStandards", ::GetStandards_,
		bkdmng::cEnd,
		bkdmng::cItems8,	// The standards (label, id.).
		bkdmng::cEnd );

	Backend.Add( "TMCGetStandardLabel", ::GetStandardLabel_,	
		bkdmng::cId8,	// Id of the wanted standard label.
		bkdmng::cEnd,
		bkdmng::cString,		// The wanted standard label.
		bkdmng::cEnd );

	Backend.Add( "TMCGetFormats", ::GetFormats_,
		bkdmng::cEnd,
		bkdmng::cItems8,	// The formats (label, id.).
		bkdmng::cEnd );

	Backend.Add( "TMCGetFormatLabel", ::GetFormatLabel_,	
		bkdmng::cId8,	// Id of the wanted format label.
		bkdmng::cEnd,
		bkdmng::cString,		// The wanted format label.
		bkdmng::cEnd );

}





/* Although in theory this class is inaccessible to the different modules,
it is necessary to personalize it, or certain compiler would not work properly */

class tmcbsc_personnalization
: public tmcbsc_tutor
{
public:
	tmcbsc_personnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the launching of the application  */
	}
	~tmcbsc_personnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the ending of the application  */
	}
};


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

// 'static' by GNU C++.

static tmcbsc_personnalization Tutor;

ttr_tutor &TMCBSC_Tutor = Tutor;
