/*
  'tmcdrt_' library by Claude L. Simon (simon@epeios.org)
  Requires the 'tmcdrt_' header file ('tmcdrt_.h').
  Copyright (C) 2000 Claude L. SIMON (simon@epeios.org).

  This file is part of the Epeios (http://www.epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: tmcdrt_.cpp,v 1.10 2001/10/30 14:52:51 csimon Exp $

#define TMCDRT___COMPILATION

#include "tmcdrt_.h"

class tmcdrt_tutor
: public ttr_tutor
{
public:
	tmcdrt_tutor( void )
	: ttr_tutor( TMCDRT__NAME )
	{
#ifdef TMCDRT__DBG
		Version = TMCDRT__VERSION "\b\bD $";
#else
		Version = TMCDRT__VERSION;
#endif
		Owner = TMCDRT__OWNER;
		Date = "$Date: 2001/10/30 14:52:51 $";
	}
	virtual ~tmcdrt_tutor( void ){}
};

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

#include "tmctmc_.h"

namespace tmcdrt_ {

	const char *tmcdrt_::duration::PREFIX = TMCDRT__DURATION_TYPE_PREFIX;
	const char *tmcdrt_::duration::NAME = TMCDRT__DURATION_TYPE_NAME;
	
	typedef void (* f_duration)(
		bkdmng::backend &Backend,
		duration &Duration,
		bkdmng::request_manager___ &Request );

	void tmcdrt_::duration::HANDLE(
		bkdmng::backend &Backend,
		bkdmng::untyped_module &Module,
		bkdmng::command__ Command,
		bkdmng::request_manager___ &Request,
		void *UP )
	{
		((f_duration)Module.UPs( Command ))( Backend, *this, Request );
	}


	void Initialize(
		bkdmng::backend &Backend,
		duration &Duration,
		bkdmng::request_manager___ &Request )
	{
		Duration.Init();
	}

	void WriteBegin(
		bkdmng::backend &Backend,
		duration &Duration,
		bkdmng::request_manager___ &Request )
	{
	ERRProlog
		tmctmc::hour_time_code__ T;
		bkdmng::object__ IdObjet;
		bkdmng::id8__ Standard;
	ERRBegin
		Request.PopObject( IdObjet );
		Request.PopId8( Standard );

		if ( strcmp( Backend.Module( IdObjet ).Name(), TMCTMC__TIMECODE_TYPE_NAME ) )
			ERRb();

		if ( *Standard >= tmcbsc::s_amount )
			Request.SendExplanationMessage( "No such standard." );
		else {
			T = (*(tmctmc_::timecode *)Backend.Object( IdObjet ))();
			Duration().SetBegin( tmctmc::images_time_code__( T, (tmcbsc::standard)*Standard ) );
		}

		Request.Complete();

	ERRErr
	ERREnd
	ERREpilog
	}

	void WriteEnd(
		bkdmng::backend &Backend,
		duration &Duration,
		bkdmng::request_manager___ &Request )
	{
	ERRProlog
		tmctmc::hour_time_code__ T;
		bkdmng::object__ IdObjet;
		bkdmng::id8__ Standard;
	ERRBegin
		Request.PopObject( IdObjet );
		Request.PopId8( Standard );

		if ( strcmp( Backend.Module( IdObjet ).Name(), TMCTMC__TIMECODE_TYPE_NAME ) )
			ERRu();


		if ( *Standard >= tmcbsc::s_amount )
			Request.SendExplanationMessage( "No such standard." );
		else {
			T = (*(tmctmc_::timecode *)Backend.Object( IdObjet ))();
			Duration().SetEnd( tmctmc::images_time_code__( T, (tmcbsc::standard)*Standard ) );
		}
			
		Request.Complete();

	ERRErr
	ERREnd
	ERREpilog
	}


	void ReadBegin(
		bkdmng::backend &Backend,
		duration &Duration,
		bkdmng::request_manager___ &Request )
	{
	ERRProlog
		tmctmc::hour_time_code__ T;
		bkdmng::object__ IdObjet;
		bkdmng::id8__ Standard;
	ERRBegin
		Request.PopObject( IdObjet );
		Request.PopId8( Standard );

		if ( strcmp( Backend.Module( IdObjet ).Name(), TMCTMC__TIMECODE_TYPE_NAME ) )
			ERRb();


		if ( *Standard >= tmcbsc::s_amount )
			Request.SendExplanationMessage( "No such standard." );
		else {
			T = Duration().GetBegin().ConvertToHTC( (tmcbsc::standard)*Standard );
			(*(tmctmc_::timecode *)Backend.Object( IdObjet ))() = T;
		}
	ERRErr
	ERREnd
	ERREpilog
	}

	void ReadEnd(
		bkdmng::backend &Backend,
		duration &Duration,
		bkdmng::request_manager___ &Request )
	{
	ERRProlog
		tmctmc::hour_time_code__ T;
		bkdmng::object__ IdObjet;
		bkdmng::id8__ Standard;
	ERRBegin
		Request.PopObject( IdObjet );
		Request.PopId8( Standard );

		if ( strcmp( Backend.Module( IdObjet ).Name(), TMCTMC__TIMECODE_TYPE_NAME ) )
			ERRb();


		if ( *Standard >= tmcbsc::s_amount )
			Request.SendExplanationMessage( "No such standard." );
		else {
			T = Duration().GetEnd().ConvertToHTC( (tmcbsc::standard)*Standard );
			(*(tmctmc_::timecode *)Backend.Object( IdObjet ))() = T;
		}
	ERRErr
	ERREnd
	ERREpilog
	}


	void GetResult(
		bkdmng::backend &Backend,
		duration &Duration,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id8__ Standard;
		bkdmng::id8__ Format;
		const char *Resultat;

		Request.PopId8( Standard );
		Request.PopId8( Format );

		if ( *Standard >= tmcbsc::s_amount )
			Request.SendExplanationMessage( "No such standard id." );
		else if ( *Format >= tmcbsc::f_amount )
			Request.SendExplanationMessage( "No such format id." );
		else {
			Resultat = tmctmc::ConvertToText( Duration().Result(), (tmcbsc::standard)*Standard, (tmcbsc::format)*Format );
			Request.PushString( bkdmng::string( Resultat ) );
		}
	}


	void GetSign(
		bkdmng::backend &Backend,
		duration &Duration,
		bkdmng::request_manager___ &Request )
	{
		static bso::bool__ Signe;

		Signe = Duration().Sign();

		Request.PushBoolean( Signe );
	}

	#define D( name )	#name, (void *)tmcdrt_::##name

	void duration::NOTIFY( bkdmng::untyped_module &Module )
	{
		Module.Add( D( Initialize ),
			bkdmng::cEnd,
			bkdmng::cEnd );
		
		Module.Add( D( WriteBegin ),
			bkdmng::cObject,	// The 'tmctmc_time_code' to write.
			bkdmng::cId8,		// The standard (25 i/s, 30 i/s DF, ... )
			bkdmng::cEnd,
			bkdmng::cEnd );
			
		Module.Add( D( WriteEnd ),
			bkdmng::cObject,	// The 'tmctmc_time_code' to write.
			bkdmng::cId8,		// The standard (25 i/s, 30 i/s DF, ... )
			bkdmng::cEnd,
			bkdmng::cEnd );
		
		Module.Add( D( ReadBegin ),
			bkdmng::cObject,	// The 'tmctmc_time_code' to read.
			bkdmng::cId8,		// The standard (25 i/s, 30 i/s DF, ... )
			bkdmng::cEnd,
			bkdmng::cEnd );
			
		Module.Add( D( ReadEnd ),
			bkdmng::cObject,	// The 'tmctmc_time_code' to read.
			bkdmng::cId8,		// The standard (25 i/s, 30 i/s DF, ... )
			bkdmng::cEnd,
			bkdmng::cEnd );

		Module.Add( D( GetSign ),
			bkdmng::cEnd,
			bkdmng::cBoolean,	// The wanted sign.
			bkdmng::cEnd );
			
		Module.Add( D( GetResult ),
			bkdmng::cId8,		// The standard (25 i/s, 30 i/s DF, ... )
			bkdmng::cId8,	// The format ( h:m:s:i, 'm's"i )
			bkdmng::cEnd,
			bkdmng::cString,	// The string containing the result.
			bkdmng::cEnd );
	}
}


/* Although in theory this class is inaccessible to the different modules,
it is necessary to personalize it, or certain compiler would not work properly */

class tmcdrt_personnalization
: public tmcdrt_tutor
{
public:
	tmcdrt_personnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the launching of the application  */
	}
	~tmcdrt_personnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the ending of the application  */
	}
};


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

// 'static' by GNU C++.

static tmcdrt_personnalization Tutor;

ttr_tutor &TMCDRT_Tutor = Tutor;
