/*
  Header for the 'tmcset' library by Claude L. Simon (epeios@epeios.org)
  Copyright (C) 2000 Claude L. SIMON (epeios@epeios.org) 

  This file is part of the Epeios (http://www.epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: tmcset.h,v 1.7 2001/05/13 19:44:46 csimon Exp $

#ifndef TMCSET__INC
#define TMCSET__INC

#define TMCSET_NAME		"TMCSET"

#define	TMCSET_VERSION	"$Revision: 1.7 $"	

#define TMCSET_OWNER		"the Epeios project (http://www.epeios.org/)"

#include "ttr.h"

extern class ttr_tutor &TMCSETTutor;

#if defined( XXX_DBG ) && !defined( TMCSET_NODBG )
#define TMCSET_DBG 
#endif

/* Begin of automatic documentation generation part. */

//V $Revision: 1.7 $
//C Claude L. SIMON (epeios@epeios.org)
//R $Date: 2001/05/13 19:44:46 $

/* End of automatic documentation generation part. */

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

#include "err.h"
#include "tmcdrt.h"
#include "lst.h"
#include "bitbch.h"
#include "idxque.h"
#include "bch.h"

namespace tmcset {

	using lst::list_;
	using idxque::queue_index_;

	//c A duration set.
	class duration_set_
	: public E_LIST_,
	  public E_IQUEUE_
	{
	protected:
		void LSTAllocate( tym::size__ Size )
		{
			E_IQUEUE_::Allocate( Size, aem::mFit );
			Durations.Allocate( Size, aem::mFit );
			Operations.Allocate( Size, aem::mFit );
		}
	public:
		//o Durations..
		bch::E_BUNCH_( tmcdrt::duration__ ) Durations;
		//o Les oprations. NOTA: La premire opration est ignore
		bitbch::bit_bunch_<epeios::row__> Operations;
		struct s
		: public E_LIST_::s,
		  public E_IQUEUE_::s
		{
			bch::E_BUNCH_( tmcdrt::duration__ )::s Durations;
			bitbch::bit_bunch_<epeios::row__>::s Operations;
		};
		duration_set_( s &S )
		: E_LIST_( S ),
		  E_IQUEUE_( S ),
		  Durations( S.Durations ),
		  Operations( S.Operations )
		{}
		void reset( bool P = true )
		{
			E_LIST_::reset( P );
			E_IQUEUE_::reset( P );
			Durations.reset( P );
			Operations.reset( P );
		}
		void plug( mmm::multimemory_ &M )
		{
			E_LIST_::plug( M );
			E_IQUEUE_::plug( M );
			Durations.plug( M );
			Operations.plug( M );
		}
		duration_set_ &operator =( const duration_set_ &E )
		{
			E_LIST_::operator =( E );
			E_IQUEUE_::operator =( E );
			Durations = E.Durations;
			Operations = E.Operations;

			return *this;
		}
		//f Initialisation.
		void Init( void )
		{
			E_LIST_::Init();
			E_IQUEUE_::Init();
			Durations.Init();
			Operations.Init();
		}
		//f Resultat des oprations sur tout l'ensemble
		tmctmc::images_time_code__ Total( void );
		//f Add 'TCD'. Return the position where added.
		tym::row__ Add( tmcdrt::duration__ TCD )
		{
			tym::row__ Position = E_LIST_::CreateEntry();

			if ( E_IQUEUE_::Amount() )
				InsertItemAfterNode( Position, E_IQUEUE_::Tail() );
			else
				Create( Position );
			
			Durations.Write( TCD, Position );
			Operations.Write( false, Position );

			return Position;
		}
		//f 'TCD' is placed next to 'Item'. Return the position where added.
		tym::row__ BecomeNext(
			tmcdrt::duration__ TCD,
			tym::row__ Item )
		{
			tym::row__ Position = E_LIST_::CreateEntry();
			E_IQUEUE_::BecomeNext( Position, Item );

			Durations.Write( TCD, Position );
			Operations.Write( false, Position );

			return Position;
		}
		//f 'TCD' is placed previous to 'Item'. Return the position where added.
		tym::row__ BecomePrevious(
			tmcdrt::duration__ TCD,
			tym::row__ Item )
		{
			tym::row__ Position = E_LIST_::CreateEntry();
			E_IQUEUE_::BecomePrevious( Position, Item );

			Durations.Write( TCD, Position );
			Operations.Write( false, Position );

			return Position;
		}
		//f 'Sign' becomes the sign of 'Item'.
		void SetSign(
			tym::row__ Item,
			bso::bool__ Sign )
		{
			Operations.Write( Sign, Item );
		}
		//f Return the sign of 'Item'.
		bso::bool__ GetSign( tym::row__ Item )
		{
			return Operations.Read( Item );
		}
		//f Remove 'Item'..
		void Remove( tym::row__ Element )
		{
			E_LIST_::Remove( Element );
			E_IQUEUE_::Remove( Element );
		}
		NAV( E_IQUEUE_:: )
	};
}


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

#endif
