/*
  'tmcset_' library by Claude L. Simon (simon@epeios.org)
  Requires the 'tmcset_' header file ('tmcset_.h').
  Copyright (C) 2000 Claude L. SIMON (simon@epeios.org).

  This file is part of the Epeios (http://www.epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: tmcset_.cpp,v 1.9 2001/10/30 14:52:51 csimon Exp $

#define TMCSET___COMPILATION

#include "tmcset_.h"

class tmcset_tutor
: public ttr_tutor
{
public:
	tmcset_tutor( void )
	: ttr_tutor( TMCSET__NAME )
	{
#ifdef TMCSET__DBG
		Version = TMCSET__VERSION "\b\bD $";
#else
		Version = TMCSET__VERSION;
#endif
		Owner = TMCSET__OWNER;
		Date = "$Date: 2001/10/30 14:52:51 $";
	}
	virtual ~tmcset_tutor( void ){}
};

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

#include "tmcdrt_.h"

namespace tmcset_ {

	const char *duration_set::PREFIX = TMCSET__DURATION_SET_TYPE_PREFIX;
	const char *duration_set::NAME = TMCSET__DURATION_SET_TYPE_NAME;

	typedef void (* f_set)(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request );


	void duration_set::HANDLE(
		bkdmng::backend &Backend,
		bkdmng::untyped_module &Module,
		bkdmng::command__ Command,
		bkdmng::request_manager___ &Request,
		void *UP )
	{
		((f_set)Module.UPs( Command ))( Backend, *this, Request );
	}


	// Initialisation
	void Initialize(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
		DurationSet.Init();
	}

	// Retourne vrai si l'ensemble est vide, faux sinon.
	void IsEmpty(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::boolean__ Drapeau;

		Drapeau = DurationSet.IsEmpty();

		Request.PushBoolean( Drapeau );
	}

	// Lecture d'un lment.
	void Read(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
	ERRProlog
		tmcdrt::duration__ T;
		bkdmng::id32__ Index;
		bkdmng::object__ IdObjet;
	ERRBegin
		Request.PopId32( Index );
		Request.PopObject( IdObjet );

		if ( strcmp( Backend.Module( IdObjet ).Name(), TMCDRT__DURATION_TYPE_NAME ) )
			ERRu();

		if ( !DurationSet.Exists( (bkdmng::index__)*Index ) )
			ERRu();

		T = DurationSet.Durations( (bkdmng::index__)*Index );

		(*(tmcdrt_::duration *)Backend.Object( IdObjet ))() = T;

	ERRErr
	ERREnd
	ERREpilog
	}

	// Ecriture d'un lment.
	void Write(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
	ERRProlog
		tmcdrt::duration__ T;
		bkdmng::id32__ Index;
		bkdmng::object__ IdObjet;
	ERRBegin
		Request.PopObject( IdObjet );
		Request.PopId32( Index );

		if ( strcmp( Backend.Module( IdObjet ).Name(), TMCDRT__DURATION_TYPE_NAME ) )
			ERRu();

		T = (*(tmcdrt_::duration *)Backend.Object( IdObjet ))();

		if ( !DurationSet.Exists( (bkdmng::index__)*Index ) )
			ERRu();

		DurationSet.Durations.Write( T, *Index );

	ERRErr
	ERREnd
	ERREpilog
	}


	// Ajout d'un lment aprs un autre.
	void Add(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
	ERRProlog
		tmcdrt::duration__ T;
		bkdmng::object__ IdObjet;
		bkdmng::id32__ Courant;
	ERRBegin
		Request.PopObject( IdObjet );
		Request.PopId32( Courant );

		if ( strcmp( Backend.Module( IdObjet ).Name(), TMCDRT__DURATION_TYPE_NAME ) )
			ERRu();

		T = (*(tmcdrt_::duration *)Backend.Object( IdObjet ))();

		if ( DurationSet.IsEmpty() )
			Courant = *DurationSet.Add( T );
		else if ( DurationSet.Exists( *Courant ) )
			Courant = *DurationSet.BecomeNext( T, *Courant );
		else
			ERRu();

		Request.PushId32( Courant );
	ERRErr
	ERREnd
	ERREpilog
	}

	// Ajout d'un lment aprs un autre.
	void CreateEntry(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
	ERRProlog
		tmcdrt::duration__ T;
		bkdmng::id32__ Courant;
	ERRBegin
		Request.PopId32( Courant );

		T.Init();

		if ( DurationSet.IsEmpty() )
			Courant = *DurationSet.Add( T );
		else if ( DurationSet.Exists( *Courant ) )
			Courant = *DurationSet.BecomeNext( T, *Courant );
		else
			ERRu();

		Request.PushId32( Courant );
	ERRErr
	ERREnd
	ERREpilog
	}


	// Suppression d'un lment.
	void Remove(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id32__ Element;
		tym::row__ New;

		Request.PopId32( Element );

		if ( DurationSet.IsEmpty() )
			Request.SendExplanationMessage( "No item to remove." );
		else if ( !DurationSet.Exists( *Element ) )
			Request.SendExplanationMessage( "No such item." );
		else {
			if ( ( New = DurationSet.Next( *Element ) ) == NONE )
				New = DurationSet.Previous( *Element );

			DurationSet.Remove( *Element );

			Request.PushId32( *New );
		}
	}


	// Lecture d'une liste
	void GetList(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
	ERRProlog
		bkdmng::id8__ Standard;
		bkdmng::id8__ Format;
		bkdmng::items32 Items;
		bkdmng::item32 Item;
	ERRBegin

		Request.PopId8( Standard );
		Request.PopId8( Format );

		if ( *Standard >= tmcbsc::s_amount )
			Request.SendExplanationMessage( "No such standard id." );
		else if ( *Format >= tmcbsc::f_amount )
			Request.SendExplanationMessage( "No such format id." );
		else {
			tym::row__ Courant;
			
			Items.Init();
			
			Courant = DurationSet.First();

			while( Courant != NONE )
			{
				Item.Init();

				Item.Value = tmctmc::ConvertToText( ( DurationSet.Operations.Read( Courant ) ? -(bso::slong__)DurationSet.Durations( Courant ).Result() : (bso::slong__)DurationSet.Durations( Courant ).Result() ), (tmcbsc::standard)*Standard, (tmcbsc::format)*Format );

				Item.ID( *Courant );

				Items.Add( Item );

				Courant = DurationSet.Next( Courant );
			}
			
			Request.PushItems32( Items );
		}
	ERRErr
	ERREnd
	ERREpilog

	}

	// Retourne le total.
	void GetTotal(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id8__ Standard;
		bkdmng::id8__ Format;
		const char *Resultat;

		Request.PopId8( Standard );
		Request.PopId8( Format );

		if ( *Standard >= tmcbsc::s_amount )
			Request.SendExplanationMessage( "No such standard id." );
		else if ( *Format >= tmcbsc::f_amount )
			Request.SendExplanationMessage( "No such format id." );
		else {
			Resultat = tmctmc::ConvertToText( DurationSet.Total(), (tmcbsc::standard)*Standard, (tmcbsc::format)*Format );
			Request.PushString( str::string( Resultat ) );
		}
	}

	// Monte un lement d'un cran.
	void Up(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id32__ Element;

		Request.PopId32( Element );

		if ( !DurationSet.Exists( *Element ) )
			ERRu();

		if ( ( *Element != *DurationSet.First() ) )
			DurationSet.Swap( *Element, DurationSet.Previous( *Element ) );
		else
			Request.SendExplanationMessage( "Cannot move up." );
	}

	// Descend un lement d'un cran.
	void Down(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id32__ Element;

		Request.PopId32( Element );

		if ( !DurationSet.Exists( *Element ) )
			ERRu();

		if ( Element != *DurationSet.Last() )
			DurationSet.Swap( *Element, DurationSet.Next( *Element ) );
		else
			Request.SendExplanationMessage( "Cannot move down." );
	}

	void PutSign(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id32__ Position;
		bkdmng::boolean__ Signe;

		Request.PopBoolean( Signe );
		Request.PopId32( Position );

		if ( !DurationSet.Exists( *Position ) )
			ERRu();

		DurationSet.SetSign( *Position, Signe );
	}

	void GetSign(
		bkdmng::backend &Backend,
		duration_set &DurationSet,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id32__ Position;
		bkdmng::boolean__ Signe;

		Request.PopId32( Position );

		if ( !DurationSet.Exists( *Position ) )
			ERRu();

		Signe = DurationSet.GetSign( *Position );

		Request.PushBoolean( Signe );
	}
	
	#define D( name )	#name, (void *)tmcset_::name


	void duration_set::NOTIFY( bkdmng::untyped_module &Module )
	{
//		Module.Add( D( Initialize )
		Module.Add( D( Initialize ), bkdrqm::cEnd, bkdrqm::cEnd );
		Module.Add( D( Remove ),
			bkdmng::cId32,	// Item to remove
			bkdmng::cEnd,
			bkdmng::cId32,	// Next item, or previous if none, or -1 if none.
			bkdmng::cEnd );
		Module.Add( D( IsEmpty ),
			bkdmng::cEnd,
			bkdmng::cBoolean,	// true or false wether empty or not.
			bkdmng::cEnd );
		Module.Add( D( GetList ),
			bkdmng::cId8,		// Standard,
			bkdmng::cId8,		// Format
			bkdmng::cEnd,
			bkdmng::cItems32,	// The list
			bkdmng::cEnd );
		Module.Add( D( Read ),
			bkdmng::cId32,		// The item to read.
			bkdmng::cObject,	// The duration where item is put. 
			bkdmng::cEnd,
			bkdmng::cEnd );
		Module.Add( D( Write ),
			bkdmng::cObject,	// The duration to write
			bkdmng::cId32,		// The item where duration written.
			bkdmng::cEnd,
			bkdmng::cEnd );
		Module.Add( D( Add ),
			bkdmng::cObject,	// The duration to add
			bkdmng::cId32,		// The item after where duration added.
			bkdmng::cEnd,
			bkdmng::cId32,		// The item where duration added.
			bkdmng::cEnd );
		Module.Add( D( GetTotal ),
			bkdmng::cId8,		// Standard.
			bkdmng::cId8,		// Format.
			bkdmng::cEnd,
			bkdmng::cString,	// The wanted total.
			bkdmng::cEnd );
		Module.Add( D( Up ),
			bkdmng::cId32,	// The concerned item.
			bkdmng::cEnd,
			bkdmng::cEnd );
		Module.Add( D( Down ),
			bkdmng::cId32,	// The concerned item.
			bkdmng::cEnd,
			bkdmng::cEnd );
		Module.Add( D( PutSign ),
			bkdmng::cBoolean,	// The sign.
			bkdmng::cId32,		// The concerned item.
			bkdmng::cEnd,
			bkdmng::cEnd );
		Module.Add( D( GetSign ),
			bkdmng::cId32,		// The concerned item.
			bkdmng::cEnd,
			bkdmng::cBoolean,	// The asked sign.
			bkdmng::cEnd );
		Module.Add( D( CreateEntry ),
			bkdmng::cId32,	// Item after which the nex item is put.
			bkdmng::cEnd,
			bkdmng::cId32,	// The new item.
			bkdmng::cEnd );
	}

}

/* Although in theory this class is inaccessible to the different modules,
it is necessary to personalize it, or certain compiler would not work properly */

class tmcset_personnalization
: public tmcset_tutor
{
public:
	tmcset_personnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the launching of the application  */
	}
	~tmcset_personnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the ending of the application  */
	}
};


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

// 'static' by GNU C++.

static tmcset_personnalization Tutor;

ttr_tutor &TMCSET_Tutor = Tutor;
