/*
  Header for the 'tmctmc' library by Claude L. Simon (epeios@epeios.org)
  Copyright (C) 2000 Claude L. SIMON (epeios@epeios.org) 

  This file is part of the Epeios (http://www.epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: tmctmc.h,v 1.3 2001/05/13 19:44:46 csimon Exp $

#ifndef TMCTMC__INC
#define TMCTMC__INC

#define TMCTMC_NAME		"TMCTMC"

#define	TMCTMC_VERSION	"$Revision: 1.3 $"	

#define TMCTMC_OWNER		"the Epeios project (http://www.epeios.org/)"

#include "ttr.h"

extern class ttr_tutor &TMCTMCTutor;

#if defined( XXX_DBG ) && !defined( TMCTMC_NODBG )
#define TMCTMC_DBG 
#endif

/* Begin of automatic documentation generation part. */

//V $Revision: 1.3 $
//C Claude L. SIMON (epeios@epeios.org)
//R $Date: 2001/05/13 19:44:46 $

/* End of automatic documentation generation part. */

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

#include "err.h"
#include "tmcbsc.h"


namespace tmctmc {
	typedef bso::slong__	titc__;

	//t Type of a time code digit.
	typedef bso::ubyte__ digit__;

	class hour_time_code__;

	//c Images time code.
	class images_time_code__
	{
		titc__ TC_;
	public:
		images_time_code__(
			class hour_time_code__ TC,
			tmcbsc::standard Standard );
		images_time_code__( bso::slong__ ImagesAmount  = 0 )
		{
			TC_ = ImagesAmount;
		}
		//f Convert to hour time code format following 'Standard'. The sign is lost.
		hour_time_code__ ConvertToHTC( tmcbsc::standard Standard );
		operator titc__( void )
		{
			return TC_;
		}
		images_time_code__ operator -=( images_time_code__ O )
		{
			return TC_ = TC_ - O.TC_;
		}
		images_time_code__ operator +=( images_time_code__ O )
		{
			return TC_ = TC_ + O.TC_;
		}
	};

	//c Hour time code.
	class hour_time_code__
	{
		bso::ubyte__ H_, M_, S_, I_;
	public:
		hour_time_code__(
			bso::ubyte__ H = 0,
			bso::ubyte__ M = 0,
			bso::ubyte__ S = 0,
			bso::ubyte__ I = 0 )
		{
			H_ = H;
			M_ = M;
			S_ = S;
			I_ = I;
		}
		//f Convert to images time code format following 'Standard'.
		images_time_code__ ConvertToITC( tmcbsc::standard Standard )
		{
			if ( Standard == tmcbsc::s25 )
				return images_time_code__( H_ * 60 * 60UL * 25 + M_ * 60 * 25 + S_ * 25 + I_ );
			else if ( Standard == tmcbsc::s30NDF )
				return images_time_code__( H_ * 60 * 60UL * 30 + M_ * 60 * 30 + S_ * 30 + I_ );
			else if ( Standard == tmcbsc::s30DF )
			{
				unsigned long Minutes = H_ * 60 + M_;

				return images_time_code__( Minutes * 60 * 30 + S_ * 30 + I_  - 2 * ( Minutes - Minutes / 10 ) );
			}
			else
			{
				ERRu();
				return images_time_code__( 0 ); // To avoid a 'warning'.;
			}
		}
		//f Return the amount of hours.
		bso::ubyte__ H( void )
		{
			return H_;
		}
		//f 'H' becomes the new amounts of hours.
		void H( bso::ubyte__ H )
		{
			H_ = H;
		}
		//f Return the amount of minutes.
		bso::ubyte__ M( void )
		{
			return M_;
		}
		//f 'M' becomes the new amounts of minutes.
		void M( bso::ubyte__ M )
		{
			M_ = M;
		}
		//f Return the amount of seconds
		bso::ubyte__ S( void )
		{
			return S_;
		}
		//f 'S' becomes the new amounts of seconds.
		void S( bso::ubyte__ S )
		{
			S_ = S;
		}
		//f Return the amount of images
		bso::ubyte__ I( void )
		{
			return I_;
		}
		//f 'I' becomes the new amounts of images.
		void I( bso::ubyte__ I )
		{
			I_ = I;
		}
	};



	inline images_time_code__::images_time_code__(
			hour_time_code__ TC,
			tmcbsc::standard Standard )
	{
		*this = TC.ConvertToITC( Standard );
	}


	inline hour_time_code__ images_time_code__::ConvertToHTC( tmcbsc::standard Standard )
	{
		bso::ulong__ TCI = ( *this < 0 ? (bso::ulong__)-*this : (bso::ulong__)*this );
		hour_time_code__ TCH;

		if ( Standard == tmcbsc::s30DF )
			TCI += 2 * ( TCI / ( 30 * 60 ) ) - 2 * ( TCI / ( 30 * 600 ) );

		if ( Standard == tmcbsc::s25 )
			TCH = hour_time_code__(
				(digit__)( TCI / ( 60 * 60 * 25 ) ),
				(digit__)( ( TCI % ( 60 * 60 * 25 ) ) / ( 60 * 25 ) ),
				(digit__)( ( TCI % ( 60 * 25 ) ) / 25 ),
				(digit__)( TCI % 25 ) );
		else if ( ( Standard == tmcbsc::s30NDF ) || ( Standard == tmcbsc::s30DF ) )
			TCH = hour_time_code__(
				(digit__)( TCI / ( 60 * 60 * 30 ) ),
				(digit__)( ( TCI % ( 60 * 60 * 30 ) ) / ( 60 * 30 ) ),
				(digit__)( ( TCI % ( 60 * 30 ) ) / 30 ),
				(digit__)( TCI % 30 ) );
		else
			ERRu();

		return TCH;
	}

	//f Return a string representation of 'TCI' following 'Standard' and 'Format'.
	const char *ConvertToText(
		images_time_code__ TCI,
		tmcbsc::standard Standard,
		tmcbsc::format Format );
}


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

#endif
