/*
  'tmctmc_' library by Claude L. Simon (simon@epeios.org)
  Requires the 'tmctmc_' header file ('tmctmc_.h').
  Copyright (C) 2000 Claude L. SIMON (simon@epeios.org).

  This file is part of the Epeios (http://www.epeios.org/) project.
  

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, go to http://www.fsf.org/
  or write to the:
  
                        Free Software Foundation, Inc.,
           59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//	$Id: tmctmc_.cpp,v 1.10 2001/10/30 14:52:51 csimon Exp $

#define TMCTMC___COMPILATION

#include "tmctmc_.h"

class tmctmc_tutor
: public ttr_tutor
{
public:
	tmctmc_tutor( void )
	: ttr_tutor( TMCTMC__NAME )
	{
#ifdef TMCTMC__DBG
		Version = TMCTMC__VERSION "\b\bD $";
#else
		Version = TMCTMC__VERSION;
#endif
		Owner = TMCTMC__OWNER;
		Date = "$Date: 2001/10/30 14:52:51 $";
	}
	virtual ~tmctmc_tutor( void ){}
};

/******************************************************************************/
				  /* do not modify anything above this limit */
				  /*			  unless specified			 */
				  /*******************************************/
/*$BEGIN$*/

namespace tmctmc_ {

	namespace {
		using namespace bkdtpm;
	}

	const char *timecode::PREFIX = TMCTMC__TIMECODE_TYPE_PREFIX;
	const char *timecode::NAME = TMCTMC__TIMECODE_TYPE_NAME;
	
	typedef void (* f_timecode)(
		timecode &Timecode,
		bkdmng::request_manager___ &Request );

	void timecode::HANDLE(
		bkdmng::backend &Backend,
		bkdmng::untyped_module &Module,
		bkdmng::command__ Command,
		bkdmng::request_manager___ &Request,
	void *UP )
	{
		((f_timecode)Module.UPs( Command ))( *this, Request );
	}

	int GetNumber( bkdrqm::request_manager___ &Request )
	{
		int Retour = 0;
	ERRProlog
		string Donnee;
	ERRBegin
		Donnee.Init();

		Request.PopString( Donnee );

		switch ( Donnee.Amount() ) {
		case 2:
			if ( !isdigit( Donnee( 0 ) ) || !isdigit( Donnee( 1 ) ) )
				ERRb();

			Retour = ( 10 * ( Donnee( 0 ) - '0' ) ) + ( Donnee( 1 ) - '0' );
			break;
		case 1:
			if ( !isdigit( Donnee( 0 ) ) )
				ERRb();

			Retour = Donnee( 0 ) - '0';
		case 0:
			break;
		default:
			ERRb();
			break;
		}

	ERRErr
	ERREnd
	ERREpilog
		return Retour;
	}

	char *Convert( int D )
	{
		static char T[3];

		sprintf( T, "%.2i", D );

		return T;
	}

	// Initialisation.
	void Initialize(
		timecode &Timecode,
		bkdmng::request_manager___ &Request )
	{
		Timecode.Init();
	}

	void WriteH(
		timecode &Timecode,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id8__ Standard;
		int H = GetNumber( Request );

		Request.PopId8( Standard );

		if ( *Standard >= tmcbsc::s_amount )
			Request.SendExplanationMessage( "No such standard." );
		else if ( H > 99 )
			Request.PushString( string( "Hour item must be less than 100." ) );
		else {
			Request.PushString( string( "" ) );
			Timecode().H( H );
		}
	}

	void WriteM(
		timecode &Timecode,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id8__ Standard;
		int M = GetNumber( Request );

		Request.PopId8( Standard );

		if ( *Standard >= tmcbsc::s_amount )
			Request.SendExplanationMessage( "No such standard." );
		else if ( M >= 60 )
			Request.PushString( string( "Minute item must be less than 60." ) );
		else {
			Request.PushString( string( "" ) );
			Timecode().M( M );
		}
	}

	void WriteS(
		timecode &Timecode,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id8__ Standard;
		int S = GetNumber( Request );

		Request.PopId8( Standard );

		if ( *Standard >= tmcbsc::s_amount )
			Request.SendExplanationMessage( "No such standard." );
		else if ( S >= 60 )
			Request.PushString( string( "Second item must be less than 100." ) );
		else {
			Request.PushString( string( "" ) );
			Timecode().S( S );
		}
	}

	void WriteI(
		timecode &Timecode,
		bkdmng::request_manager___ &Request )
	{
		bkdmng::id8__ Standard;
		int I = GetNumber( Request );
		bso::bool__ Error = false;

		Request.PopId8( Standard );

		switch ( *Standard ) {
		case tmcbsc::s25:
			if ( I >= 25 ) {
				Error = true;
				Request.PushString( string( "Image item must be less than 25." ) );
			}
			break;
		case tmcbsc::s30DF:
		case tmcbsc::s30NDF:
			if ( I >= 30 ) {
				Error = true;
				Request.PushString( string( "Image item must be less than 30." ) );
			}
			break;
		default:
			Error = true;
			Request.SendExplanationMessage( "No such standard." );
			break;
		}

		if ( !Error ) {
			Request.PushString( string( "" ) );
			Timecode().I( I );
		}
	}

	void ReadH(
		timecode &Timecode,
		bkdmng::request_manager___ &Request )
	{
		const char *Resultat;

		Resultat = Convert( Timecode().H() );
		Request.PushString( string( Resultat ) );
	}

	void ReadM(
		timecode &Timecode,
		bkdmng::request_manager___ &Request )
	{
		const char *Resultat;

		Resultat = Convert( Timecode().M() );
		Request.PushString( string( Resultat ) );
	}

	void ReadS(
		timecode &Timecode,
		bkdmng::request_manager___ &Request )
	{
		const char *Resultat;

		Resultat = Convert( Timecode().S() );
		Request.PushString( string( Resultat ) );
	}

	void ReadI(
		timecode &Timecode,
		bkdmng::request_manager___ &Request )
	{
		const char *Resultat;

		Resultat = Convert( Timecode().I() );
		Request.PushString( string( Resultat ) );
	}

	#define D( name )	#name, (void *)tmctmc_::##name

	void timecode::NOTIFY( bkdmng::untyped_module &Module )
	{
		Module.Add( D( Initialize ),
			bkdmng::cEnd,
			bkdmng::cEnd );

		Module.Add( D( WriteH ),
			bkdmng::cString,	// Value to write.
			bkdmng::cId8,		// Standard.
			bkdmng::cEnd,
			bkdmng::cString,	// Eventually, the reason because the value is incorrect.
			bkdmng::cEnd );

		Module.Add( D( WriteM ),
			bkdmng::cString,	// Value to write.
			bkdmng::cId8,		// Standard.
			bkdmng::cEnd,
			bkdmng::cString,	// Eventually, the reason because the value is incorrect.
			bkdmng::cEnd );
			
		Module.Add( D( WriteS ),
			bkdmng::cString,	// Value to write.
			bkdmng::cId8,		// Standard.
			bkdmng::cEnd,
			bkdmng::cString,	// Eventually, the reason because the value is incorrect.
			bkdmng::cEnd );
			
		Module.Add( D( WriteI ),
			bkdmng::cString,	// Value to write.
			bkdmng::cId8,		// Standard.
			bkdmng::cEnd,
			bkdmng::cString,	// Eventually, the reason because the value is incorrect.
			bkdmng::cEnd );

		Module.Add( D( ReadH ),
			bkdmng::cEnd,
			bkdmng::cString,	// Wanted value.
			bkdmng::cEnd );
			
		Module.Add( D( ReadM ),
			bkdmng::cEnd,
			bkdmng::cString,	// Wanted value.
			bkdmng::cEnd );
			
		Module.Add( D( ReadS ),
			bkdmng::cEnd,
			bkdmng::cString,	// Wanted value.
			bkdmng::cEnd );
			
		Module.Add( D( ReadI ),
			bkdmng::cEnd,
			bkdmng::cString,	// Wanted value.
			bkdmng::cEnd );
	}
}


/* Although in theory this class is inaccessible to the different modules,
it is necessary to personalize it, or certain compiler would not work properly */

class tmctmc_personnalization
: public tmctmc_tutor
{
public:
	tmctmc_personnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the launching of the application  */
	}
	~tmctmc_personnalization( void )
	{
		/* place here the actions concerning this library
		to be realized at the ending of the application  */
	}
};


/*$END$*/
				  /********************************************/
				  /* do not modify anything belove this limit */
				  /*			  unless specified		   	  */
/******************************************************************************/

// 'static' by GNU C++.

static tmctmc_personnalization Tutor;

ttr_tutor &TMCTMC_Tutor = Tutor;
