// MsgViewerTreeView.cpp : implementation of the CMsgViewerTreeView class
//

#include "stdafx.h"
#include "MsgViewer.h"
#include "MsgViewerDoc.h"
#include "MsgViewerTreeView.h"
#include <mimepp/mimepp.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMsgViewerTreeView

IMPLEMENT_DYNCREATE(CMsgViewerTreeView, CTreeView)


BEGIN_MESSAGE_MAP(CMsgViewerTreeView, CTreeView)
    //{{AFX_MSG_MAP(CMsgViewerTreeView)
    ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnSelchanged)
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMsgViewerTreeView construction/destruction

CMsgViewerTreeView::CMsgViewerTreeView()
{
}

CMsgViewerTreeView::~CMsgViewerTreeView()
{
}

BOOL CMsgViewerTreeView::PreCreateWindow(CREATESTRUCT& cs) 
{
    cs.style |= TVS_HASLINES | TVS_LINESATROOT | TVS_HASBUTTONS |
        TVS_SHOWSELALWAYS;

    return CTreeView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMsgViewerTreeView drawing

void CMsgViewerTreeView::OnDraw(CDC* pDC)
{
    CMsgViewerDoc* pDoc = GetDocument();
    ASSERT_VALID(pDoc);
}

/////////////////////////////////////////////////////////////////////////////
// CMsgViewerTreeView message handlers

void CMsgViewerTreeView::OnSelchanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
    NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

    HTREEITEM hItem = pNMTreeView->itemNew.hItem;
    int objId = (int) GetTreeCtrl().GetItemData(hItem);
    int currentObjId = GetDocument()->GetCurrentObjectId();
    if (currentObjId != objId) {
        GetDocument()->SetCurrentObjectId(objId);
        GetDocument()->UpdateAllViews(this);
    }
    *pResult = 0;
}

void CMsgViewerTreeView::OnInitialUpdate() 
{
    CTreeView::OnInitialUpdate();

    CMsgViewerDoc* pDoc = GetDocument();
    ASSERT_VALID(pDoc);

    DwMessage* msg = pDoc->GetMessage();
    if (msg == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();
    tree.DeleteAllItems();
    HTREEITEM hRoot = tree.InsertItem("DwMessage", TVI_ROOT);
    int objId = msg->ObjectId();
    tree.SetItemData(hRoot, objId);
    ExpandEntity(msg, hRoot);
}

/////////////////////////////////////////////////////////////////////////////
// CMsgViewerTreeView operations

CMsgViewerDoc* CMsgViewerTreeView::GetDocument() // non-debug version is inline
{
    ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMsgViewerDoc)));
    return (CMsgViewerDoc*)m_pDocument;
}


void CMsgViewerTreeView::ExpandEntity(DwEntity* aEntity, HTREEITEM aParentItem)
{
    if (aEntity == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();

    DwHeaders* headers = &aEntity->Headers();
    int objId = headers->ObjectId();
    HTREEITEM hHeaders = tree.InsertItem("DwHeaders", aParentItem);
    tree.SetItemData(hHeaders, objId);
    ExpandHeaders(headers, hHeaders);

    DwBody* body = &aEntity->Body();
    objId = body->ObjectId();
    HTREEITEM hBody = tree.InsertItem("DwBody", aParentItem);
    tree.SetItemData(hBody, objId);
    ExpandBody(body, hBody);
}


void CMsgViewerTreeView::ExpandHeaders(DwHeaders* aHeaders, HTREEITEM aParentItem)
{
    if (aHeaders == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();

    int numFields = aHeaders->NumFields();
    for (int i=0; i < numFields; ++i) {
        DwField* field = &aHeaders->FieldAt(i);
        int objId = field->ObjectId();
        HTREEITEM hField = tree.InsertItem("DwField", aParentItem);
        tree.SetItemData(hField, objId);
        ExpandField(field, hField);
    }
}


void CMsgViewerTreeView::ExpandBody(DwBody* aBody, HTREEITEM aParentItem)
{
    if (aBody == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();

    int numBodyParts = aBody->NumBodyParts();
    if (numBodyParts > 0) {
        for (int i=0; i < numBodyParts; ++i) {
            DwBodyPart* bodyPart = &aBody->BodyPartAt(i);
            int objId = bodyPart->ObjectId();
            HTREEITEM hBodyPart = tree.InsertItem("DwBodyPart", aParentItem);
            tree.SetItemData(hBodyPart, objId);
            ExpandEntity(bodyPart, hBodyPart);
        }
        return;
    }

    DwMessage* msg = aBody->Message();
    if (msg) {
        int objId = msg->ObjectId();
        HTREEITEM hMsg = tree.InsertItem("DwMessage", aParentItem);
        tree.SetItemData(hMsg, objId);
        ExpandEntity(msg, hMsg);
        return;
    }
}


void CMsgViewerTreeView::ExpandField(DwField* aField, HTREEITEM aParentItem)
{
    if (aField == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();

    DwFieldBody* fieldBody = aField->FieldBody();
    const char* className = fieldBody->ClassName();
    int objId = fieldBody->ObjectId();
    HTREEITEM hFieldBody = tree.InsertItem(className, aParentItem);
    tree.SetItemData(hFieldBody, objId);

    int classId = fieldBody->ClassId();
    switch (classId) {
    case DwMessageComponent::kCidAddressList: {
        DwAddressList* addrList = (DwAddressList*) fieldBody;
        ExpandAddressList(addrList, hFieldBody);
        }
        break;
    case DwMessageComponent::kCidMailboxList: {
        DwMailboxList* mboxList = (DwMailboxList*) fieldBody;
        ExpandMailboxList(mboxList, hFieldBody);
        }
        break;
    case DwMessageComponent::kCidMediaType: {
        DwMediaType* type = (DwMediaType*) fieldBody;
        ExpandMediaType(type, hFieldBody);
        }
        break;
    case DwMessageComponent::kCidDispositionType: {
        DwDispositionType* type = (DwDispositionType*) fieldBody;
        ExpandDispositionType(type, hFieldBody);
        }
        break;
    case DwMessageComponent::kCidGroup: {
        DwGroup* group = (DwGroup*) fieldBody;
        ExpandGroup(group, hFieldBody);
        }
        break;
    case DwMessageComponent::kCidText: {
        DwText* text = (DwText*) fieldBody;
        ExpandText(text, hFieldBody);
        }
        break;
    }
}


void CMsgViewerTreeView::ExpandAddressList(DwAddressList* aAddrList, HTREEITEM aParentItem)
{
    if (aAddrList == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();

    int numAddresses = aAddrList->NumAddresses();
    for (int i=0; i < numAddresses; ++i) {
        DwAddress* addr = &aAddrList->AddressAt(i);
        const char* className = addr->ClassName();
        int objId = addr->ObjectId();
        HTREEITEM hAddr = tree.InsertItem(className, aParentItem);
        tree.SetItemData(hAddr, objId);
        int classId = addr->ClassId();
        if (classId == DwMessageComponent::kCidGroup) {
            ExpandGroup((DwGroup*) addr, hAddr);
        }
    }
}


void CMsgViewerTreeView::ExpandMailboxList(DwMailboxList* aMboxList, HTREEITEM aParentItem)
{
    if (aMboxList == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();

    int numMailboxes = aMboxList->NumMailboxes();
    for (int i=0; i < numMailboxes; ++i) {
        DwMailbox* mbox = &aMboxList->MailboxAt(i);
        const char* className = mbox->ClassName();
        int objId = mbox->ObjectId();
        HTREEITEM hMbox = tree.InsertItem(className, aParentItem);
        tree.SetItemData(hMbox, objId);
    }
}


void CMsgViewerTreeView::ExpandMediaType(DwMediaType* aType, HTREEITEM aParentItem)
{
    if (aType == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();

    int numParameters = aType->NumParameters();
    for (int i=0; i < numParameters; ++i) {
        DwParameter* param = &aType->ParameterAt(i);
        const char* className = param->ClassName();
        int objId = param->ObjectId();
        HTREEITEM hParam = tree.InsertItem(className, aParentItem);
        tree.SetItemData(hParam, objId);
    }
}


void CMsgViewerTreeView::ExpandGroup(DwGroup* aGrp, HTREEITEM aParentItem)
{
    if (aGrp == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();

    DwMailboxList* mboxList = &aGrp->MailboxList();
    const char* className = mboxList->ClassName();
    int objId = mboxList->ObjectId();
    HTREEITEM hMboxList = tree.InsertItem(className, aParentItem);
    tree.SetItemData(hMboxList, objId);
    ExpandMailboxList(mboxList, hMboxList);
}


void CMsgViewerTreeView::ExpandDispositionType(DwDispositionType* aType, HTREEITEM aParentItem)
{
    if (aType == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();

    int numParameters = aType->NumParameters();
    for (int i=0; i < numParameters; ++i) {
        DwParameter* param = &aType->ParameterAt(i);
        const char* className = param->ClassName();
        int objId = param->ObjectId();
        HTREEITEM hParam = tree.InsertItem(className, aParentItem);
        tree.SetItemData(hParam, objId);
    }
}


void CMsgViewerTreeView::ExpandText(DwText* aText, HTREEITEM aParentItem)
{
    if (aText == 0) {
        return;
    }
    CTreeCtrl& tree = GetTreeCtrl();

    int numEncodedWords = aText->NumEncodedWords();
    for (int i=0; i < numEncodedWords; ++i) {
        DwEncodedWord* word = &aText->EncodedWordAt(i);
        const char* className = word->ClassName();
        int objId = word->ObjectId();
        HTREEITEM hParam = tree.InsertItem(className, aParentItem);
        tree.SetItemData(hParam, objId);
    }
}


/////////////////////////////////////////////////////////////////////////////
// CMsgViewerTreeView diagnostics

#ifdef _DEBUG
void CMsgViewerTreeView::AssertValid() const
{
    CTreeView::AssertValid();
}

void CMsgViewerTreeView::Dump(CDumpContext& dc) const
{
    CTreeView::Dump(dc);
}
#endif //_DEBUG
