// article_ex.cpp
//
// This example program shows how to retrieve an article from a newsgroup.
//
// Three command line arguments are required: the name or address of the
// news server, the name of the newsgroup, and the number of the article to
// retrieve.
//
// To use the program, you will need a way to get a valid article number.
// One way to do this is to use the headers_ex example program to list all
// the articles in a newsgroup.
//

#include <mimepp/mimepp.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>

using std::cout;


int main(int argc, char** argv)
{
    int retval = 1;

    // Three command line arguments are required: the name or address of
    // the news server, the name of the newsgroup, and the number of the
    // article to retrieve.

    if (argc < 4) {
        printf("Usage: %s server newsgroup number\n", argv[0]);
        exit(EXIT_FAILURE);
    }
    const char* server = argv[1];
    const char* newsgroup = argv[2];
    int articleNum = atoi(argv[3]);

    // Create an NNTP client object

    DwNntpClient nntp;

    // Open a connection to the news server. Check for a 2xx reply code,
    // which indicates success.

    int replyCode = nntp.Open(server, 119);
    if ((replyCode/100%10) == 2) {

        // Send a GROUP command, with the newsgroup name as a parameter.
        // Check for a 2xx reply code, which indicates success.

        replyCode = nntp.Group(newsgroup);
        if ((nntp.ReplyCode()/100%10) == 2) {

            // Send an ARTICLE command, with the article number as a
            // parameter.  Check for a 2xx reply code, which indicates
            // success.

            replyCode = nntp.Article(articleNum);
            if ((replyCode/100%10) == 2) {

                // If the command succeeded, get the text response, which
                // contains the headers and body of the article

                const DwString& article = nntp.TextResponse();
                cout << article;
                retval = 0;
            }
            else /* if ((replyCode/100%10) == 2) */ {
                printf("NNTP Error: %s\n", nntp.StatusResponse().c_str());
            }
        }
        else /* if ((replyCode/100%10) == 2) */ {
            printf("NNTP Error: %s\n", nntp.StatusResponse().c_str());
        }

        // Send a QUIT command

        nntp.Quit();
    }
    else /* if ((replyCode/100%10) != 2) */ {
        printf("NNTP Error: %s\n", nntp.StatusResponse().c_str());
    }

    // Close the connection

    nntp.Close();

    return retval;
}
