// smtp.h

#ifndef SMTP_H
#define SMTP_H

#include <mimepp/mimepp.h>


class SmtpErrorHandler {
public:
    virtual void Error(
        const char* aCommand,
        int aErrorCode,
        const char* aErrorMessage,
        const char* aRecipient) = 0;
};


class Smtp {

public:

    // For the constructor, you must give the name or address of the SMTP
    // server.  An IP address in decimal-dot form is allowed.
    //
    // Examples: "smtp.mydomain.org", "192.168.20.45"
    Smtp(const char* aServer);

    // If an error handler has been set, the destructor will delete it.
    virtual ~Smtp();

    // Sets an error handler object (see the interface above).  The error
    // handler is owned by this Smtp object, and it will be deleted by the
    // Smtp's objects destructor.
    void SetErrorHandler(SmtpErrorHandler* aHandler);

    // Sends a message object.  The originator and recipients are extracted
    // from the message headers.  For the originator, if there is a "Sender"
    // header field, it's value is used as the originator; otherwise the
    // "From" header field is used.  For the recipients, the addresses from
    // the "To", "Cc", and "Bcc" header fields are used.  Any "Bcc" header
    // fields are removed before the message is sent.
    //
    // The return value is SUCCESS if the message was successfully submitted
    // for all recipients, PARTIAL_SUCCESS if the message was successfully
    // submitted for at least one recipient (and failed for at least one
    // recipient), and FAILURE if the message could not be submitted for
    // any recipient.
    int SendMessage(DwMessage& aMessage);

    // Sends a serialized message.  Before the message is sent, the end-
    // of-line characters are converted to CR LF, as required by RFC 821
    // (and RFC 2821).
    //
    // The return value is SUCCESS if the message was successfully submitted
    // for all recipients, PARTIAL_SUCCESS if the message was successfully
    // submitted for at least one recipient (and failed for at least one
    // recipient), and FAILURE if the message could not be submitted for
    // any recipient.
    int SendMessage(const DwString& aMessageStr, DwMailbox& aSender,
        DwMailboxList& aRecipients);

    enum {
        UNKNOWN,
        SUCCESS,
        PARTIAL_SUCCESS,
        FAILURE
    };

private:

    char* mServer;
    SmtpErrorHandler* mErrorHandler;

    int SendHeloCommand(DwSmtpClient&, const DwString&, DwMailboxList&,
        DwMailbox&);
    int SendMailCommand(DwSmtpClient&, const DwString&, DwMailboxList&,
        DwMailbox&);
    int SendRcptCommand(DwSmtpClient&, const DwString&, DwMailboxList&);
    int SendDataCommand(DwSmtpClient&, const DwString&);

};

#endif
