// smtp_ex.cpp

#include "smtp.h"


class MySmtpErrorHandler : public SmtpErrorHandler {
public:
    virtual void Error(
        const char* aCommand,
        int aReplyCode,
        const char* aReplyMessage,
        const char* aRecipient);
};


void MySmtpErrorHandler::Error(
    const char* aCommand,
    int aReplyCode,
    const char* aReplyMessage,
    const char* aRecipient)
{
    printf("Error on SMTP command \"%s\".\n", aCommand);
    printf("The SMTP reply code was \"%d\".\n", aReplyCode);
    printf("The SMTP reply message was \"%s\".\n", aReplyMessage);
    if (aRecipient[0] != 0) {
        printf("The recipient was \"%s\".\n", aRecipient);
    }
}


int main(int, char**)
{
    // Make changes to the string literals below first!

    Smtp smtp("example.net");

    MySmtpErrorHandler* errorHandler = new MySmtpErrorHandler();
    smtp.SetErrorHandler(errorHandler);
    errorHandler = 0;

    DwMessage msg;
    DwHeaders& headers = msg.Headers();

    DwMailbox* addr = new DwMailbox("Sally Jones <sally.jones@example.net>");
    addr->Parse();
    headers.To().AddAddress(addr);
    addr = 0;

    addr = new DwMailbox("John Smith <john.smith@example.net>");
    addr->Parse();
    headers.To().AddAddress(addr);
    addr = 0;

    addr = new DwMailbox("Leroy Johnson <leroy@example.pri>");
    addr->Parse();
    headers.To().AddAddress(addr);
    addr = 0;

    addr = new DwMailbox("Jane Doe <jane.doe@example.net>");
    addr->Parse();
    headers.Cc().AddAddress(addr);
    addr = 0;

    addr = new DwMailbox("Won Kim <wkim@example.net>");
    addr->Parse();
    headers.From().AddMailbox(addr);
    addr = 0;

    headers.Subject().FromString("Test message");

    msg.Body().FromString("Test message. This is the body.\n");

    msg.Assemble();

    int result = smtp.SendMessage(msg);
    if (result == Smtp::FAILURE) {
        printf("Failure\n");
    }
    else if (result == Smtp::PARTIAL_SUCCESS) {
        printf("Partial success\n");
    }
    else if (result == Smtp::SUCCESS) {
        printf("Success\n");
    }

    return 0;
}
