/* Testing Polyhedron::affine_preimage(): the transformation is not
   invertible and the polyhedron is defined by its system of constraints.
   Copyright (C) 2001, 2002 Roberto Bagnara <bagnara@cs.unipr.it>

This file is part of the Parma Polyhedra Library (PPL).

The PPL is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

The PPL is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

For the most up-to-date information see the Parma Polyhedra Library
site: http://www.cs.unipr.it/ppl/ . */

#include "ppl_install.hh"
#include "print.hh"
#include "ehandlers.hh"

using namespace std;
using namespace Parma_Polyhedra_Library;

#define NOISY 0

int
main() {
  set_handlers();

  Variable A(0);
  Variable B(1);

  Polyhedron ph(2);
  ph.insert(A >= 0);
  ph.insert(B >= 0);
  ph.insert(A + B - 3 >= 0);

#if NOISY
  print_constraints(ph, "--- ph ---");
#endif

  ph.affine_preimage(A, B+1);

  Polyhedron known_result(2, Polyhedron::EMPTY);
  known_result.insert(vertex(B));
  known_result.insert(line(A));
  known_result.insert(ray(B));

  int retval = (ph == known_result) ? 0 : 1;

#if NOISY
    print_generators(ph, "--- ph after ph.affine_preimage(A, B+2) ---");
#endif

  return retval;
}
