/*=============================================================================
    Regular expression parser primitives

    Spirit V1.2
    Copyright (c) 2001, Hartmut Kaiser

    This software is provided 'as-is', without any express or implied
    warranty. In no event will the copyright holder be held liable for
    any damages arising from the use of this software.

    Permission is granted to anyone to use this software for any purpose,
    including commercial applications, and to alter it and redistribute
    it freely, subject to the following restrictions:

    1.  The origin of this software must not be misrepresented; you must
        not claim that you wrote the original software. If you use this
        software in a product, an acknowledgment in the product documentation
        would be appreciated but is not required.

    2.  Altered source versions must be plainly marked as such, and must
        not be misrepresented as being the original software.

    3.  This notice may not be removed or altered from any source
        distribution.

    Acknowledgements:

        Special thanks to Dan Nuffer, John (EBo) David, Chris Uzdavinis,
        and Doug Gregor. These people are most instrumental in steering
        Spirit in the right direction.

        Special thanks also to people who have contributed to the code base
        and sample code, ported Spirit to various platforms and compilers,
        gave suggestions, reported and provided bug fixes. Alexander
        Hirner, Andy Elvey, Bogdan Kushnir, Brett Calcott, Bruce Florman,
        Changzhe Han, Colin McPhail, Hakki Dogusan, Jan Bares, Joseph
        Smith, Martijn W. van der Lee, Raghavendra Satish, Remi Delcos, Tom
        Spilman, Vladimir Prus, W. Scott Dillman, David A. Greene, Bob
        Bailey, Hartmut Kaiser.

        Finally special thanks also to people who gave feedback and
        valuable comments, particularly members of Spirit's Source Forge
        mailing list and boost.org.

    URL: http://spirit.sourceforge.net/

=============================================================================*/
#ifndef SPIRIT_REGEX_IPP
#define SPIRIT_REGEX_IPP

///////////////////////////////////////////////////////////////////////////////
namespace spirit {

///////////////////////////////////////////////////////////////////////////////
// rxlit class implementation
template <typename CharT, typename StringT>
inline rxlit<CharT, StringT>::rxlit(StringT str_)
:   str(str_) {}

//////////////////////////////////
template <typename CharT, typename StringT>
inline rxlit<CharT, StringT>::rxlit(CharT const *str_)
:   str(str_) {}

//////////////////////////////////
template <typename CharT, typename StringT>
template <typename IteratorT>
inline match
rxlit<CharT, StringT>::parse(IteratorT& first, IteratorT const& last) const
{
    typedef typename impl::strip_scanner<IteratorT>
        ::iterator_type plain_iter;
    plain_iter  i1 = impl::strip_scanner<IteratorT>::get(first);
    plain_iter  i2 = impl::strip_scanner<IteratorT>::get(last);

    boost::match_results<IteratorT> what;
    boost::regex_match(i1, i2, what, str, boost::match_default);

    unsigned count = what[0].length();
    first += count;
    return match(count);
}

//////////////////////////////////
template <typename CharT, typename StringT>
template <typename IteratorT>
inline ast_match<IteratorT>
rxlit<CharT, StringT>::ast_parse(IteratorT& first, IteratorT const& last, 
        MatchTraitsT const& mt) const
{
    typedef typename impl::strip_scanner<IteratorT>
        ::iterator_type plain_iter;
    plain_iter  i1 = impl::strip_scanner<IteratorT>::get(first);
    plain_iter  i2 = impl::strip_scanner<IteratorT>::get(last);

    boost::match_results<IteratorT> what;
    boost::regex_match(i1, i2, what, str, boost::match_default);

    unsigned count = what[0].length();
    first += count;
    return ast_match<IteratorT>(count);
}

///////////////////////////////////////////////////////////////////////////////
// Generator functions implementation
template <typename CharT>
inline rxlit<CharT>
regex_p(CharT const* str)
{
    return rxlit<CharT>(str);
}

}   //  namespace Spirit

#endif // SPIRIT_REGEX_IPP
