///////////////////////////////////////////////////////////////////////////////
// Demonstrate regular expression parser objects
//
// This sample requires an installed version of the boost regex library 
// (http://www.boost.org) The sample was tested with boost V1.25.0

#include <string>
#include <iostream>

#include <boost/spirit/spirit.hpp>

// The following header must be included, if regular expression support is
// required for Spirit.
#include <boost/spirit/regex.hpp>	

///////////////////////////////////////////////////////////////////////////////
// used namespaces
using namespace std;
using namespace spirit;

extern const char pexp[] = "exp";

///////////////////////////////////////////////////////////////////////////////
// actor called after successfully matching a regular expression
void actor (const char *pfirst, const char *plast)
{
	cout << "Parsed " << plast-pfirst << " characters: ";

char cbbuffer[128];

	strncpy(cbbuffer, pfirst, plast-pfirst);
	cbbuffer[plast-pfirst] = '\0';

	cout << cbbuffer << endl;
}

///////////////////////////////////////////////////////////////////////////////
// main entry point
int main (int argc, char *argv[])
{
const char *ptest = "123 E 456";
const char *prx = "[1-9]+[ ]*E[ ]*";

	cout << "Parse " << ptest << " against regular expression: " << prx << endl;

// 1. direct use of rxlit<>
rxlit<> regexpr(prx);

const char *scan1 = ptest;
match mexpr = regexpr[&actor].parse(scan1, scan1+strlen(scan1));
	
	cout << "Parsed (really) " << mexpr.length() << " characters." << endl;

// 2. use of regex_p predefined parser object
const char *scan2 = ptest;

	regex_p(prx)[&actor].parse(scan2, scan2+strlen(scan2));

	return 0;
}



