#include "boost/spirit/spirit.hpp"
#include <iostream>
#include <stack>
#include <functional>

////////////////////////////////////////////////////////////////////////////
using namespace std;
using namespace spirit;

typedef tree_node_ch_iter<ast_node<> > iter_t;

typedef char const*         iterator_t;
typedef ast_match_traits<iterator_t> match_traits_t;
typedef skipper<iterator_t, match_traits_t> skipper_t;
typedef scanner<iterator_t, match_traits_t> scanner_t;
typedef rule<scanner_t, match_traits_t> rule_t;


//  grammar rules
rule_t expression, term, factor, integer;

////////////////////////////////////////////////////////////////////////////
long evaluate(ast_match<> hit);
long eval_expression(iter_t const& i);
long eval_term(iter_t const& i);
long eval_factor(iter_t const& i);
long eval_integer(iter_t const& i);

long evaluate(ast_match<> hit)
{
    return eval_expression(hit.ast.root().ch_begin());
}

long eval_expression(iter_t const& i)
{
    rule_id_t id = i->value.rule_id;
    assert(id == &expression);

    iter_t chi = i->ch_begin();
    long lhs = eval_term(chi);
    for (++chi; chi != i->ch_end(); ++chi)
    {
        char op = *(chi->value.first);
        ++chi;
        long rhs = eval_term(chi);
        if (op == '+')
            lhs += rhs;
        else if (op == '-')
            lhs -= rhs;
        else
            assert(0);
    }
    return lhs;
}

long eval_term(iter_t const& i)
{
    rule_id_t id = i->value.rule_id;
    assert(id == &term);

    iter_t chi = i->ch_begin();
    long lhs = eval_factor(chi);
    for (++chi; chi != i->ch_end(); ++chi)
    {
        char op = *(chi->value.first);
        ++chi;
        long rhs = eval_factor(chi);
        if (op == '*')
            lhs *= rhs;
        else if (op == '/')
            lhs /= rhs;
        else
            assert(0);
    }
    return lhs;
}

long eval_factor(iter_t const& i)
{
    rule_id_t id = i->value.rule_id;
    assert(id == &factor);

    iter_t chi = i->ch_begin();
    id = chi->value.rule_id;
    if (id == &integer)
        return eval_integer(chi);
    else if (*(chi->value.first) == '(')
    {
        ++chi;
        return eval_expression(chi);
    }
    else if (*(chi->value.first) == '-')
    {
        ++chi;
        return -eval_factor(chi);
    }
    else
    {
        assert(0);
        return 0;
    }
}

long eval_integer(iter_t const& i)
{
    rule_id_t id = i->value.rule_id;
    assert(id == &integer);
    return strtol(i->value.first, 0, 10);
}

////////////////////////////////////////////////////////////////////////////
int
main()
{

    //  Start grammar definition
    integer     =   lexeme[ (!ch_p('-') >> +digit) ];
    factor      =   integer
                |   '(' >> expression >> ')'
                |   ('-' >> factor);
    term        =   factor >>
                    *(  ('*' >> factor)
                      | ('/' >> factor)
                    );
    expression  =   term >>
                    *(  ('+' >> term)
                      | ('-' >> term)
                    );
    //  End grammar definition


    cout << "/////////////////////////////////////////////////////////\n\n";
    cout << "\t\tThe simplest working calculator...\n\n";
    cout << "/////////////////////////////////////////////////////////\n\n";
    cout << "Type an expression...or [q or Q] to quit\n\n";

    while (true)
    {
        char str[256];
        cin.getline(str, 256);
        if (str[0] == 'q' || str[0] == 'Q')
            break;

        const char* str_begin = str;
        const char* str_end = str;
        while (*str_end)
            ++str_end;

        skipper_t skip(space, str_end);
        scanner_t first(str_begin, &skip);
        scanner_t last(str_end, &skip);

        ast_match<> hit = expression.ast_parse(first, last, match_traits_t());


        if (hit && first == last)
        {
            cout << "parsing succeeded\n";
            cout << "result = " << evaluate(hit) << "\n\n";
        }
        else
        {
            cout << "parsing failed\n";
        }
    }

    cout << "Bye... :-) \n\n";
    return 0;
}


