#define SPIRIT_DEBUG  ///$$$ DEFINE THIS WHEN DEBUGGING $$$///

#include <iostream>
#include <string>
#include "boost/spirit/spirit.hpp"

#ifdef SPIRIT_DEBUG
// REVIEW: should we add this file from the boost distribution to spirit?
// copied from boost/libs/tuple/src/tuple.cpp
namespace boost {
namespace tuples {
namespace detail {

const int format_info::stream_index[number_of_manipulators] 
   = { std::ios::xalloc(), std::ios::xalloc(), std::ios::xalloc() };

}
}
}
#endif

/////////////////////////////////////////////////////////////////////////////////////////
using namespace std;
using namespace spirit;

/////////////////////////////////////////////////////////////////////////////////////////
void str_actor (std::string const &str)
{
    cout
        << "Closure parser return value: " 
        << "\'" << str << "\'"
        << endl;
}

/////////////////////////////////////////////////////////////////////////////////////////
int
main()
{
    cout << "/////////////////////////////////////////////////////////\n\n";
    cout << "\t\tClosure tests...\n\n";
    cout << "/////////////////////////////////////////////////////////\n\n";

    {
        rule<char const*>   reverse;//  Our reverse rule
        closure<char>       loc;    //  Our closure
        closure_member<0>   ch;     //  Member 0 of our closure

        reverse = loc[

            anychar[loc(ch)]  >> !reverse >> ch_p(loc(ch))
        ];

        SPIRIT_DEBUG_RULE(reverse);

        //  Should Pass
        cout << (parse("xyzzyx", reverse).full? "parses Ok\n\n" : "fails parsing\n\n");

        //  Should fail
        cout << (parse("abcabc", reverse).full ? "parses Ok\n\n" : "fails parsing\n\n");
    }

    {
        rule<>              tags, start_tag, end_tag;
        closure<string>     loc;    //  Our closure
        closure_member<0>   str;    //  Member 0 of our closure

        start_tag = '<' >> lexeme[(+alpha)[loc(str)]] >> '>';
        end_tag = "</" >> str_p(loc(str)) >> '>';

        tags = loc[

            start_tag >> *tags >> end_tag
        ];

        SPIRIT_DEBUG_RULE(tags);
        SPIRIT_DEBUG_RULE(start_tag);
        SPIRIT_DEBUG_RULE(end_tag);

        //  Should Pass
        char const* i1 = "<table><p><font></font></p></table>";
        cout << (parse(i1, tags, space).full? "parses Ok\n\n" : "fails parsing\n\n");

        //  Should Pass
        char const* i2 = "<html><head></head><body></body></html>";
        cout << (parse(i2, tags, space).full? "parses Ok\n\n" : "fails parsing\n\n");

        //  Should fail
        char const* i3 = "<table><p><font></p></font></table>";
        cout << (parse(i3, tags, space).full? "parses Ok\n\n" : "fails parsing\n\n");
    }

// here the closure_member<0> is evaluated as 'return value' from the
// rule 'tags'
    {
        rule<>              tags, start_tag, end_tag;
        closure<string> loc;            //  Our closure
        closure_member<0>   str;        //  Member 0 of our closure

        start_tag 
            =   '<' 
            >>  lexeme[(+alpha)[loc(str)]] 
            >>  '>';

        end_tag 
            =   "</" 
            >>  str_p(loc(str)) 
            >>  '>';

        tags = loc[start_tag >> *tags >> end_tag][&str_actor];

        //  Should Pass
        char const* i1 = "<table><p><font></font></p></table>";
        cout << (parse(i1, tags, space).full? "parses Ok\n\n" : "fails parsing\n\n");

        //  Should Pass
        char const* i2 = "<html><head></head><body></body></html>";
        cout << (parse(i2, tags, space).full? "parses Ok\n\n" : "fails parsing\n\n");

        //  Should fail
        char const* i3 = "<table><p><font></p></font></table>";
        cout << (parse(i3, tags, space).full? "parses Ok\n\n" : "fails parsing\n\n");
    }
}


