#include <iostream>
#include <string>
#include "boost/spirit/spirit.hpp"

///////////////////////////////////////////////////////////////////////////////
using namespace std;
using namespace spirit;

int n;

void
add1000(char)
{
    n += 1000;
}

void
add_roman(char const* /*first*/, char const* /*last*/, int data)
{
    n += data;
}

int
main()
{
    cout << "/////////////////////////////////////////////////////////\n\n";
    cout << "\t\tRoman Numerals Parser\n\n";
    cout << "/////////////////////////////////////////////////////////\n\n";
    cout << "Type a Roman Numeral ...or [q or Q] to quit\n\n";

    //  Start grammar definition

    symbols<>
        hundreds, tens, ones;

    char const*
        h[] =   { "C", "CC", "CCC", "CD", "D", "DC", "DCC", "DCCC", "CM" }
        ;

    char const*
        t[] =   { "X", "XX", "XXX", "XL", "L", "LX", "LXX", "LXXX", "XC" }
        ;

    char const*
        o[] =   { "I", "II", "III", "IV", "V", "VI", "VII", "VIII", "IX" }
        ;

    for (int i = 1; i < 10; ++i)
    {
        add(hundreds, h[i-1], i * 100);
        add(tens, t[i-1], i * 10);
        add(ones, o[i-1], i);
    }

    rule<char const*> roman
        =   +ch_p('M')[&add1000]
        ||  hundreds[&add_roman]
        ||  tens[&add_roman]
        ||  ones[&add_roman];

    //  End grammar definition

    while (true)
    {
        char str[256];
        cin.getline(str, 256);
        if (str[0] == 'q' || str[0] == 'Q')
            break;

        n = 0;
        if (parse(str, roman).full)
        {
            cout << "parsing succeeded\n";
            cout << "result = " << n << "\n\n";
        }
        else
        {
            cout << "parsing failed\n";
        }
    }

    cout << "Bye... :-) \n\n";

    return 0;
}
