#include <boost/spirit/spirit.hpp>

#include <iostream>
#include <assert.h>
#include <stdio.h> // for sprintf

///////////////////////////////////////////////////////////////////////////////
using namespace std;
using namespace spirit;


///////////////////////////////////////////////////////////////////////////////
int
main()
{
    // testing good C escapes
    char const* str = "a\\b\\t\\n\\f\\r\\\"\\'\\\\\\120\\x2e";
    char const* end(str + strlen(str));

    char c;
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == 'a');
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\b');
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\t');
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\n');
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\f');
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\r');
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\"');
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\'');
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\\');
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\120');
    assert(c_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\x2e');
    assert(str == end);

    // test bad c escapes
    str = "\\z";
    end = str + strlen(str);
    assert(!c_escape_ch_p[ref(c)].parse(str, end));

    // testing good lex escapes
    str = "a\\b\\t\\n\\f\\r\\\"\\'\\\\\\120\\x2e\\z\\a";
    end = str + strlen(str);

    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == 'a');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\b');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\t');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\n');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\f');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\r');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\"');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\'');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\\');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\120');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\x2e');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == 'z');
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == 'a');
    assert(str == end);


    // test bad lex escapes
    str = "\\xz";
    end = str + strlen(str);
    assert(!lex_escape_ch_p[ref(c)].parse(str, end));


    // test out of range octal escape
    str = "\\777";
    end = str + strlen(str);
    assert(!lex_escape_ch_p[ref(c)].parse(str, end));

#if CHAR_MAX == 128
    str = "\\200";
    end = str + strlen(str);
    assert(!lex_escape_ch_p[ref(c)].parse(str, end));

    str = "\\177";
    end = str + strlen(str);
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\177');
#elif CHAR_MAX == 256
    str = "\\400";
    end = str + strlen(str);
    assert(!lex_escape_ch_p[ref(c)].parse(str, end));

    str = "\\377";
    end = str + strlen(str);
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\177');

#endif

    // test out of range hex escape
    str = "\\xFFF";
    end = str + strlen(str);
    assert(!lex_escape_ch_p[ref(c)].parse(str, end));

#if CHAR_MAX == 128
    str = "\\X80";
    end = str + strlen(str);
    assert(!lex_escape_ch_p[ref(c)].parse(str, end));

    str = "\\X7f";
    end = str + strlen(str);
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\x7f');
#elif CHAR_MAX == 256
    str = "\\x100";
    end = str + strlen(str);
    assert(!lex_escape_ch_p[ref(c)].parse(str, end));

    str = "\\xFf";
    end = str + strlen(str);
    assert(lex_escape_ch_p[ref(c)].parse(str, end));
    assert(c == '\xFf');

#endif


    // test wide chars

    wchar_t const* wstr = L"a\\b\\t\\n\\f\\r\\\"\\'\\\\\\120\\x2e";
    wchar_t const* wend(wstr + wcslen(wstr));

    wchar_t wc;
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'a');
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'\b');
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'\t');
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'\n');
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'\f');
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'\r');
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'\"');
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'\'');
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'\\');
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'\120');
    assert(c_escape_ch_p[ref(wc)].parse(wstr, wend));
    assert(wc == L'\x2e');
    assert(wstr == wend);

    // test bad wc escapes
    wstr = L"\\z";
    wend = wstr + wcslen(wstr);
    assert(!c_escape_ch_p[ref(wc)].parse(wstr, wend));


    // test out of range octal escape
    char octmax[16];
    sprintf(octmax, "\\%o", (unsigned int)std::numeric_limits<wchar_t>::max());
    str = octmax;
    end = octmax + strlen(octmax);
    assert(lex_escape_ch_p[ref(wc)].parse(str, end));
    assert(wc == std::numeric_limits<wchar_t>::max());

    sprintf(octmax, "\\%o", (unsigned int)std::numeric_limits<wchar_t>::max() 
            + 1);
    str = octmax;
    end = octmax + strlen(octmax);
    assert(!lex_escape_ch_p[ref(wc)].parse(str, end));


    // test out of range hex escape

    char hexmax[16];
    sprintf(hexmax, "\\x%x", (unsigned int)std::numeric_limits<wchar_t>::max());
    str = hexmax;
    end = hexmax + strlen(hexmax);
    assert(lex_escape_ch_p[ref(wc)].parse(str, end));
    assert(wc == std::numeric_limits<wchar_t>::max());

    sprintf(hexmax, "\\x%x", (unsigned int)std::numeric_limits<wchar_t>::max() 
            + 1);
    str = hexmax;
    end = hexmax + strlen(hexmax);
    assert(!lex_escape_ch_p[ref(wc)].parse(str, end));



    return 0;

}
