#include "boost/multi_pass.hpp"
#include <iterator>
#include <strstream>
#include <cassert>
#include <memory> // for auto_ptr

using namespace std;
using namespace boost;

typedef multi_pass<istream_iterator<char> > default_multi_pass_t;

/*
typedef multi_pass<
    istream_iterator<char>,
    multi_pass_policies::first_owner,
    multi_pass_policies::no_check,
    multi_pass_policies::fixed_size_queue,
    6
> fixed_multi_pass_t;
*/
// this replaces the above
typedef look_ahead<istream_iterator<char>, 6> fixed_multi_pass_t;

typedef multi_pass<
    istream_iterator<char>,
    multi_pass_policies::first_owner,
    multi_pass_policies::buf_id_check,
    multi_pass_policies::std_deque
> first_owner_multi_pass_t;

void test_default_multi_pass()
{
    cout << "-*= test_default_multi_pass =*-\n";
    istream_iterator<char> end;
    auto_ptr<default_multi_pass_t> mpend(new default_multi_pass_t(end));

    {
        strstream ss;
        ss << "test string";

        istream_iterator<char> a(ss);
        auto_ptr<default_multi_pass_t> mp1(new default_multi_pass_t(a));

        while (*mp1 != *mpend)
        {
            cout << *((*mp1)++);
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> b(ss);
        auto_ptr<default_multi_pass_t> mp2(new default_multi_pass_t(b));
        auto_ptr<default_multi_pass_t> mp3(new default_multi_pass_t(b));
        *mp3 = *mp2;

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp2;
            ++*mp2;
        }

        mp3.reset();

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> a(ss);
        auto_ptr<default_multi_pass_t> mp1(new default_multi_pass_t(a));
        auto_ptr<default_multi_pass_t> mp2(new default_multi_pass_t(*mp1));

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp1;
            ++*mp1;
        }

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        while (*mp1 != *mpend)
        {
            cout << **mp1;
            ++*mp1;
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> b(ss);
        auto_ptr<default_multi_pass_t> mp2(new default_multi_pass_t(b));
        auto_ptr<default_multi_pass_t> mp3(new default_multi_pass_t(b));
        *mp3 = *mp2;

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp2;
            ++*mp2;
        }

        mp3.reset();
        ++*mp2;

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> a(ss);
        auto_ptr<default_multi_pass_t> mp1(new default_multi_pass_t(a));
        auto_ptr<default_multi_pass_t> mp2(new default_multi_pass_t(*mp1));

        assert(*mp1 == *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp1 <= *mp2);
        for (int i = 0; i < 4; ++i)
        {
            cout << **mp1;
            ++*mp1;
        }

        assert(*mp1 != *mp2);
        assert(*mp1 > *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp2 < *mp1);
        assert(*mp2 <= *mp1);
        while (*mp2 != *mp1)
        {
            cout << **mp2;
            ++*mp2;
        }

        assert(*mp1 == *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp1 <= *mp2);
        while (*mp1 != *mpend)
        {
            cout << **mp1;
            ++*mp1;
        }

        assert(*mp1 != *mp2);
        assert(*mp1 > *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp2 < *mp1);
        assert(*mp2 <= *mp1);
        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        assert(*mp1 == *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp1 <= *mp2);
        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> a(ss);
        auto_ptr<default_multi_pass_t> mp1(new default_multi_pass_t(a));
        auto_ptr<default_multi_pass_t> mp2(new default_multi_pass_t(a));
        assert(*mp1 != *mp2);
        ++*mp1;
        assert(*mp1 != *mp2);

    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> b(ss);
        auto_ptr<default_multi_pass_t> mp2(new default_multi_pass_t(b));
        auto_ptr<default_multi_pass_t> mp3(new default_multi_pass_t(b));
        *mp3 = *mp2;

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp2;
            ++*mp2;
        }

        mp2->clear_queue();

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        try
        {
            cout << **mp3; // this should throw illegal_backtracking
            assert(0);
        }
        catch (const ::boost::multi_pass_policies::illegal_backtracking& e)
        {
        }
        cout << endl;
    }


}

void test_fixed_multi_pass()
{
    cout << "-*= test_fixed_multi_pass =*-\n";
    istream_iterator<char> end;
    auto_ptr<fixed_multi_pass_t> mpend(new fixed_multi_pass_t(end));

    {
        strstream ss;
        ss << "test string";

        istream_iterator<char> a(ss);
        auto_ptr<fixed_multi_pass_t> mp1(new fixed_multi_pass_t(a));

        while (*mp1 != *mpend)
        {
            cout << *((*mp1)++);
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> b(ss);
        auto_ptr<fixed_multi_pass_t> mp2(new fixed_multi_pass_t(b));
        auto_ptr<fixed_multi_pass_t> mp3(new fixed_multi_pass_t(*mp2));

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp2;
            ++*mp2;
        }

        mp3.reset();

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> a(ss);
        auto_ptr<fixed_multi_pass_t> mp1(new fixed_multi_pass_t(a));
        auto_ptr<fixed_multi_pass_t> mp2(new fixed_multi_pass_t(*mp1));

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp1;
            ++*mp1;
        }

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        while (*mp1 != *mpend)
        {
            cout << **mp1;
            ++*mp1;
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> b(ss);
        auto_ptr<fixed_multi_pass_t> mp2(new fixed_multi_pass_t(b));
        auto_ptr<fixed_multi_pass_t> mp3(new fixed_multi_pass_t(*mp2));

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp2;
            ++*mp2;
        }

        mp3.reset();
        ++*mp2;

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> a(ss);
        auto_ptr<fixed_multi_pass_t> mp1(new fixed_multi_pass_t(a));
        auto_ptr<fixed_multi_pass_t> mp2(new fixed_multi_pass_t(*mp1));

        assert(*mp1 == *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp1 <= *mp2);
        for (int i = 0; i < 4; ++i)
        {
            cout << **mp1;
            ++*mp1;
        }

        assert(*mp1 != *mp2);
        assert(*mp1 > *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp2 < *mp1);
        assert(*mp2 <= *mp1);
        while (*mp2 != *mp1)
        {
            cout << **mp2;
            ++*mp2;
        }

        assert(*mp1 == *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp1 <= *mp2);
        while (*mp1 != *mpend)
        {
            cout << **mp1;
            ++*mp1;
        }

        assert(*mp1 != *mp2);
        assert(*mp1 > *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp2 < *mp1);
        assert(*mp2 <= *mp1);
        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        assert(*mp1 == *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp1 <= *mp2);
        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> a(ss);
        auto_ptr<fixed_multi_pass_t> mp1(new fixed_multi_pass_t(a));
        auto_ptr<fixed_multi_pass_t> mp2(new fixed_multi_pass_t(a));
        assert(*mp1 != *mp2);
        ++*mp1;
        assert(*mp1 != *mp2);

    }

}


void test_first_owner_multi_pass()
{
    cout << "-*= test_first_owner_multi_pass =*-\n";
    istream_iterator<char> end;
    auto_ptr<first_owner_multi_pass_t> mpend(new first_owner_multi_pass_t(end));

    {
        strstream ss;
        ss << "test string";

        istream_iterator<char> a(ss);
        auto_ptr<first_owner_multi_pass_t> mp1(new first_owner_multi_pass_t(a));

        while (*mp1 != *mpend)
        {
            cout << *((*mp1)++);
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> b(ss);
        auto_ptr<first_owner_multi_pass_t> mp2(new first_owner_multi_pass_t(b));
        auto_ptr<first_owner_multi_pass_t> mp3(new first_owner_multi_pass_t(*mp2));

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp2;
            ++*mp2;
        }

        mp3.reset();

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> a(ss);
        auto_ptr<first_owner_multi_pass_t> mp1(new first_owner_multi_pass_t(a));
        auto_ptr<first_owner_multi_pass_t> mp2(new first_owner_multi_pass_t(*mp1));

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp1;
            ++*mp1;
        }

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        while (*mp1 != *mpend)
        {
            cout << **mp1;
            ++*mp1;
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> b(ss);
        auto_ptr<first_owner_multi_pass_t> mp2(new first_owner_multi_pass_t(b));
        auto_ptr<first_owner_multi_pass_t> mp3(new first_owner_multi_pass_t(*mp2));

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp2;
            ++*mp2;
        }

        mp3.reset();
        ++*mp2;

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> a(ss);
        auto_ptr<first_owner_multi_pass_t> mp1(new first_owner_multi_pass_t(a));
        auto_ptr<first_owner_multi_pass_t> mp2(new first_owner_multi_pass_t(*mp1));

        assert(*mp1 == *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp1 <= *mp2);
        for (int i = 0; i < 4; ++i)
        {
            cout << **mp1;
            ++*mp1;
        }

        assert(*mp1 != *mp2);
        assert(*mp1 > *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp2 < *mp1);
        assert(*mp2 <= *mp1);
        while (*mp2 != *mp1)
        {
            cout << **mp2;
            ++*mp2;
        }

        assert(*mp1 == *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp1 <= *mp2);
        while (*mp1 != *mpend)
        {
            cout << **mp1;
            ++*mp1;
        }

        assert(*mp1 != *mp2);
        assert(*mp1 > *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp2 < *mp1);
        assert(*mp2 <= *mp1);
        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        assert(*mp1 == *mp2);
        assert(*mp1 >= *mp2);
        assert(*mp1 <= *mp2);
        cout << endl;
    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> a(ss);
        auto_ptr<first_owner_multi_pass_t> mp1(new first_owner_multi_pass_t(a));
        auto_ptr<first_owner_multi_pass_t> mp2(new first_owner_multi_pass_t(a));
        assert(*mp1 != *mp2);
        ++*mp1;
        assert(*mp1 != *mp2);

    }

    {
        strstream ss;
        ss << "test string";
        istream_iterator<char> b(ss);
        auto_ptr<first_owner_multi_pass_t> mp2(new first_owner_multi_pass_t(b));
        auto_ptr<first_owner_multi_pass_t> mp3(new first_owner_multi_pass_t(*mp2));

        for (int i = 0; i < 4; ++i)
        {
            cout << **mp2;
            ++*mp2;
        }

        mp2->clear_queue();

        while (*mp2 != *mpend)
        {
            cout << **mp2;
            ++*mp2;
        }

        try
        {
            cout << **mp3; // this should throw illegal_backtracking
            assert(0);
        }
        catch (const ::boost::multi_pass_policies::illegal_backtracking& e)
        {
        }
        cout << endl;
    }

}

int main(int, char**)
{

    test_default_multi_pass();
    test_fixed_multi_pass();
    test_first_owner_multi_pass();

}

