#include <iostream>
#include "boost/spirit/spirit.hpp"

///////////////////////////////////////////////////////////////////////////////
using namespace std;
using namespace spirit;

typedef wchar_t const* witerator_t;
typedef skipper<witerator_t> wskipper_t;
typedef scanner<witerator_t> wscanner_t;
typedef rule<wscanner_t> wrule_t;


///////////////////////////////////////////////////////////////////////////////
/* This is provided so that we can *see* the output (gcc 2.95.x doesn't have
 * wcout) */
ostream& operator<<(ostream& out, wchar_t c)
{
	out << char(c);
    return out;
};
ostream& operator<<(ostream& out, wchar_t const* c)
{
    while (*c)
	    out << *c++;
    return out;
};


///////////////////////////////////////////////////////////////////////////////
template <typename RuleT, typename CharT>
static void
parse(RuleT& r, CharT const* expr)
{
    parse_info<CharT const*>
        result = parse(expr, r, space);

    if (result.full)
    {
        cout << "\t\t" << expr << "\tPass\n";
    }
    else
    {
        cout << "\t\t" << expr << "\tFail\n";
        cout << "\t\t";
        for (int i = 0; i < (result.stop - expr); i++)
            cout << " ";
        cout << "^\n";
    }
}

///////////////////////////////////////////////////////////////////////////////
int
main()
{
    cout << "/////////////////////////////////////////////////////////\n\n";
    cout << "\t\tSpirit Regression Tests...\n\n";
    cout << "/////////////////////////////////////////////////////////\n\n";

    cout << "strings: str_p(\"joel\")\n\n";
    rule<>  str = str_p("joel");
    parse(str, "joel");
    parse(str, "joelo");
    parse(str, "joal");
    parse(str, "joe");
    cout << "\n";

    cout << "no case strings: nocase[\"joel\"]\n\n";
    rule<>  ncstr = nocase["joel"];
    parse(ncstr, "joel");
    parse(ncstr, "JOEL");
    parse(ncstr, "joAl");
    parse(ncstr, "JoEl");
    cout << "\n";

    cout << "no case strings: nocase[L\"joel\"]\n\n";
    wrule_t  wncstr = nocase[L"joel"];
    parse(wncstr, L"joel");
    parse(wncstr, L"JOEL");
    parse(wncstr, L"joAl");
    parse(wncstr, L"JoEl");
    cout << "\n";

    cout << "anychar >> str_p(\"art\")\n\n";
    rule<>  any = anychar >> str_p("art");
    parse(any, "dart");
    parse(any, "cart");
    parse(any, "tart");
    parse(any, "art");
    parse(any, "hurt");
    parse(any, "dark");
    cout << "\n";

    cout << "one or more: +ch_p('a') >> 'b'\n\n";
    rule<>  onem = +ch_p('a') >> 'b';
    parse(onem, "ab");
    parse(onem, "aaab");
    parse(onem, "b");
    parse(onem, "baaa");
    cout << "\n";

    cout << "zero or more: *ch_p('a') >> 'b'\n\n";
    rule<>  zerom = *ch_p('a') >> 'b';
    parse(zerom, "b");
    parse(zerom, "ab");
    parse(zerom, "aaab");
    parse(zerom, "baaa");
    cout << "\n";

    cout << "repeat: +(nocase[range_p('a','z')] | range_p('0','9'))\n\n";
    rule<>  rep1 = +(nocase[range_p('a','z')] | range_p('0','9'));
    parse(rep1, "aaaaaa123");
    parse(rep1, "aaaAAaa123A");
    cout << "\n";

    cout << "repeat: +(nocase[range_p(L'a',L'z')] | range_p(L'0',L'9'))\n\n";
    wrule_t  wrep1 = +(nocase[range_p(L'a',L'z')] | range_p(L'0',L'9'));
    parse(wrep1, L"aaaaaa123");
    parse(wrep1, L"aaaAAaa123A");
    cout << "\n";

    cout << "repeat: +(str_p(\"abc\"))\n\n";
    rule<>  rep2 = +(str_p("abc"));
    parse(rep2, "abcabcabcabc");
    parse(rep2, "abcabcabcabcc");
    cout << "\n";

    cout << "any and repeat: *str_p(\"fo\") >> *ch_p('o') >> *(anychar - '\\0')\n\n";
    rule<>  anyrep = *str_p("fo") >> *ch_p('o') >> *(anychar - '\0');
    parse(anyrep, "fo");
    parse(anyrep, "foo");
    parse(anyrep, "fooo");
    parse(anyrep, "foobar");
    parse(anyrep, "foox");
    cout << "\n";

    cout << "sequence: *nocase[\"0x\"] >> +xdigit\n\n";
    rule<>  seq = *nocase["0x"] >> +xdigit;
    parse(seq, "0x3FFF");
    parse(seq, "0XBFFF");
    parse(seq, "0x3GFF");
    cout << "\n";

    cout << "optional: +digit >> '.' >> +digit >> !(nocase['e'] >> +digit)\n";
    rule<>  opt = +digit >> '.' >> +digit >> !(nocase['e'] >> +digit);
    parse(opt, "3.666");
    parse(opt, "3.666e25");
    parse(opt, "3.666E25");
    parse(opt, "3.6A6");
    cout << "\n";

    cout << "optional: +digit >> L'.' >> +digit >> !(nocase[L'e'] >> +digit)\n";
    wrule_t  wopt = +digit >> L'.' >> +digit >> !(nocase[L'e'] >> +digit);
    parse(wopt, L"3.666");
    parse(wopt, L"3.666e25");
    parse(wopt, L"3.666E25");
    parse(wopt, L"3.6A6");
    cout << "\n";

    cout << "or: str_p(\"joel\") | str_p(\"mariel\")\n\n";
    rule<>  alt = str_p("joel") | str_p("mariel");
    parse(alt, "joel");
    parse(alt, "mariel");
    parse(alt, "tenji");
    parse(alt, "tutit");
    parse(alt, "kim");
    cout << "\n";

    cout << "or2:\nr = str_p(\"joel\")\nr = str_p(\"mariel\")\n\n";
    rule<>  alt2;
    alt2 = str_p("joel");
    alt2 = str_p("mariel");
    parse(alt2, "joel");
    parse(alt2, "mariel");
    parse(alt2, "tenji");
    parse(alt2, "tutit");
    parse(alt2, "kim");
    cout << "\n";

    cout << "and: +((anychar - '\\0') & 'x')\n\n";
    rule<>  and_ = +((anychar - '\0') & 'x');
    parse(and_, "xxxxxxxxxx");
    parse(and_, "xxxxxaxxxx");
    parse(and_, "aaaaaaa");
    cout << "\n";

    cout << "diff: +(alpha - 'a')\n\n";
    rule<>  diff = +(alpha - 'a');
    parse(diff, "abcdefg");
    parse(diff, "bcdefgh");
    cout << "\n";

    cout << "xor: alpha ^ xdigit\n\n";
    rule<>  xor_ = alpha ^ xdigit;
    parse(xor_, "0");
    parse(xor_, "1");
    parse(xor_, "2");
    parse(xor_, "3");
    parse(xor_, "4");
    parse(xor_, "5");
    parse(xor_, "6");
    parse(xor_, "7");
    parse(xor_, "8");
    parse(xor_, "9");
    parse(xor_, "a");
    parse(xor_, "b");
    parse(xor_, "c");
    parse(xor_, "d");
    parse(xor_, "e");
    parse(xor_, "f");
    parse(xor_, "g");
    parse(xor_, "h");
    parse(xor_, "i");
    cout << "\n";

    cout << "repeat exact 8: alpha.repeat(8)\n\n";
    rule<>  rep8 = alpha.repeat(8);
    parse(rep8, "abcdefg");
    parse(rep8, "abcdefgh");
    parse(rep8, "abcdefghi");
    parse(rep8, "abcdefg");
    parse(rep8, "a");

    cout << "repeat 2 to 8: alpha.repeat(2, 8)\n\n";
    rule<>  rep28 = alpha.repeat(2, 8);
    parse(rep28, "abcdefg");
    parse(rep28, "abcdefgh");
    parse(rep28, "abcdefghi");
    parse(rep28, "abcdefg");
    parse(rep28, "a");

    cout << "repeat 2 or more\n";
    rule<>  rep2_ = alpha.repeat(2, more);
    parse(rep2_, "abcdefg");
    parse(rep2_, "abcdefgh");
    parse(rep2_, "abcdefghi");
    parse(rep2_, "abcdefg");
    parse(rep2_, "a");

	 cout << "repeat 0\n";
	 rule<>  rep0 = alpha.repeat(0);
	 parse(rep0, "");
	 parse(rep0, "a");

    cout << "longest parse: +digit >> '.' >> +digit\n\n";
    rule<>  real = +digit >> '.' >> +digit;
    rule<>  integ = +digit;
    rule<>  num = longest[ integ | real ];
    parse(num, "123456");
    parse(num, "456.1245");
    parse(num, "123456A");

    cout << "shortest parse: +digit >> '.' >> +digit\n\n";
    rule<>  snum = shortest[ integ | real ];
    parse(snum, "123456");
    parse(snum, "456.1245");
    parse(snum, "123456A");

	 cout << "lexeme\n";
	 rule<>  lexeme1 = lexeme[ (+digit >> ch_p(' ') >> alpha) ];
	 parse(lexeme1, "123 a");
	 parse(lexeme1, "123a");

	 cout << "lexeme wide chars\n";
	 wrule_t  lexeme2 = lexeme[ (+digit >> ch_p(L' ') >> alpha) ];
	 parse(lexeme2, L"123 a");
	 parse(lexeme2, L"123a");

    cout << "Back Tracking\n";
    rule<>  b = ch_p('B');
    rule<>  a = ch_p('A') >> ( ch_p('B') | 'D' ) >> 'E' >> 'F' >> 'H'
            | 'A' >> b >> 'E' >> 'G' >> 'H';


    cout << "\tShould parse OK...\n";
    parse(a, "ABEFH");
    parse(a, "ADEFH");
    parse(a, "ABEGH");
    cout << "\n";

    cout << "\tShould fail parsing...\n";
    parse(a, "AEFH");
    parse(a, "ADEGH");

    return 0;
}


